//date: 19/06/2000
/* ************************************************************************ */
/* *                            File -- Headers.h                          * */
/* ************************************************************************ */
extern "C"
{
 #include<stdio.h>
 #include<stdlib.h>
}

#include<iostream>
#include<vector>
#include<map>
#include<set>
#include<algorithm>
#include<list>
#include<string>
#include<cstdlib>
#include<iomanip>
#include<fstream>


#define SEPARATOR_AUTO                0   //separation character between 2 auta 
#define SEPARATOR_FILE                255 //end character of file of auta 
#define EMPTY_WORD                    '1' //empty word by default
#define DIESE                         '#'
#define TRANS_COL_SIZE                35  //size of colume to display the 
                                          //transitions
#define STATE_COL_SIZE                5   //size of colume to display the 
                                          //states

#define ORIGINE_HISTORY               10  //the types of history
#define REVERSE_HISTORY               20
#define DETERMINISTIC_HISTORY         30
#define CODETERMINISTIC_HISTORY       40
#define PRODUCT_HISTORY               50
#define UNION_HISTORY                 51
#define CONCATENATION_HISTORY         52 
#define COMPLEMENT_HISTORY            53
#define MINIMAL_HISTORY               54
#define NO_SPONTANEOUS_HISTORY        55
#define COMPOSE_TA_A_LTOL_HISTORY     56
#define COMPOSE_TA_T_LTOEXP_HISTORY   57
#define COMPOSE_TT_LTOL_HISTORY       58
#define COMPOSE_TT_LTOW_HISTORY       59
#define EXPRESSION_HISTORY            60
#define COMPOSE_TT_LTOEXP_HISTORY     61
#define FROM_WTOEXP_HISTORY           62
#define REVERSE_INOUT_WTOEXP_HISTORY  63
#define REVERSE_INOUT_LTOL_HISTORY    64
#define UNIFORMIZE_HISTORY            65
#define NO_HISTORY                    99

#define ITGRAPH                       100 //the types of objects
#define INTERFACE                     101
#define AUTOMATA                      102
#define GENERAL_AUTOMATA              103
#define TRANSDUCER                    104
#define LTOL_TRANSDUCER               105
#define LTOEXP_TRANSDUCER             106
#define WTOEXP_TRANSDUCER             107


#define MAX_STATE_DISPLAY             50  //max number of states to display
                                          //in small mode 
#define MAX_TRANSITION_DISPLAY        100 //max number of transitions to display
                                          //in small mode 
#define MAX_LINE                      256
#define STAR                          '*' //operator "*" in expression
#define ADDITION                      '+' //operator "+" in expression
#define PRODUCT                       '/' //operator "." in expression

#define SUBTRACTION                   '-' //operator "-" in expression
#define COMPLEMENT                    '~' //operator "~" in expression
#define INTERSECTION                  '^' //operator "^" in expression

#define STAR_PRIORITY                 1006//priority of operation "*"
#define COMPLEMENT_PRIORITY           1005  
#define PRODUCT_PRIORITY              1004 
#define SUBTRACTION_PRIORITY          1002//priority of operation "-"     
#define ADDITION_PRIORITY             1002  
#define INTERSECTION_PRIORITY         1002//prio(ADDITION)==prio(INTER)
       

typedef std::list<string> ListOfSymbols; //type of list of string 
typedef std::list<int> ListOfInts; //type of list of integer 

//forward declarations of classes
template<class T>
class State;
template<class T>
class Transition;
template<class T>
class ITGraph;
template<class T>
class Automata;
template<class T>
class Interface;
class TreeLabel;
class NodeTree;
class LinearTree;
class ConFirst;
template<class T>
class GeneralAuto;
template<class T>
class Transducer;
template<class T, class U>
class Boot;

//class "StateHeap","HeapNode","Heap"
//  is used for function "language" of class "GeneralAuto"
template<class T>
class StateHeap
{
public:
   int numAncestors;
   int numSuccessors;
   int indexHeap; 
   std::list<State<T> >::iterator iter;   
   StateHeap(){}
   StateHeap(const StateHeap& s)
   {
      numAncestors = s.numAncestors;
      numSuccessors= s.numSuccessors;
      indexHeap    = s.indexHeap;  
      iter         = s.iter;
   } 

   void operator = (const StateHeap& s)
   {
      numAncestors = s.numAncestors;
      numSuccessors= s.numSuccessors;
      indexHeap    = s.indexHeap; 
      iter         = s.iter; 
   } 
  
};
class HeapNode
{
 public:
   int index;
   int weight; 
   HeapNode(){}
   HeapNode(const HeapNode& h)
   { 
      index = h.index;
      weight= h.weight;

   } 
   void operator = (const HeapNode& s)
   {
      index = s.index;
      weight= s.weight;
   }
};

template<class T>
class Heap
{ 
public:
  Heap(){}
  Heap(int size) { array.resize(size+1); numEle = 0;}
  bool insert(HeapNode& ele,std::vector<StateHeap<T> >& s);
  bool remove(HeapNode& ele,std::vector<StateHeap<T> >& s);
  HeapNode getArray(int i) { return array[i];}  

  bool deleteD(int k, std::vector<StateHeap<T> >& s); 
  int  size(void) {return array.size();}
  int  getNumEle(void) { return numEle; }
  bool isUpdate(int weight, int index);
  bool empty(void) 
       { if(numEle==0) return true; else return false;} 
private:

  void upHeap(int k,int& pos, std::vector<StateHeap<T> >& s);
  void downHeap(int k,std::vector<StateHeap<T> >& s );
  int  numEle;
  //array[0] for "sentinent"

  std::vector<HeapNode> array;
};


//This class is used for the function "LTOEXPTransducer::traceExp"
//that is called from  "LTOEXPTransducer::composeLTOEXP"
template<class T>
class Trace
{
  public: 
   Trace() {}   
   Trace(const Trace<T>& x) { mark = x.mark; path = x.path;}
   void operator = (const Trace<T>& p1) { mark = p1.mark; path = p1.path;} 
   bool operator == (const Trace<T>& p1)
      {if(mark==p1.mark&&path==p1.path) return true;
       else return false;
      } 

   bool operator < (const Trace<T>& p1) const
     {
       if(path < p1.path) return true;
       else return false; 
     }
   int mark;
   TreeLabel path; 
};

//for only the last version of product
template<class T>
class QueuePos
{
  public:
  int i,j;
  std::list<State<T> >::iterator iter;
  QueuePos(int x, int y,std::list<State<T> >::iterator itx)  
    { i=x;j=y;iter=itx;}
  ~QueuePos(){} 
};

template<class T>
class  Binome
{
  public:
     std::list<State<T> >::iterator org;
     std::list<State<T> >::iterator dst;
};

//for building language ( used in function "createTuples" )
template<class T>
class  Quadre
{
  public:
     std::list<State<T> >::iterator org;
     std::list<State<T> >::iterator dst;
     T    left;
     T    right; 
};

//for building language ( used in function "createTuples" )
template<class T>
class  QuadreX
{
  public:
     std::list<State<T> >::iterator org;
     std::list<State<T> >::iterator dst;
     NodeTree* tree;
};

template<class T>
class  OrgDstLabel
{
  public:
     string org;
     string dst;
     T      lab;
};   

class  NameHist
{
  public:
     string name;
     ListOfSymbols hists;
};   

template<class T>
class  NameSet 
{
  public:
     string name;
     std::list<std::list<State<T> >::iterator> states;
};

template<class T>
class  LabelSet 
{
 public:
     T label;
     std::list<std::list<State<T> >::iterator> states; 
};

class  PairS 
{
 public:
     PairS() {}
     PairS(int o, int d):org(o), dst(d) {  }
     PairS(const PairS& p) 
     { this->org = p.org;this->dst=p.dst;}
     int org;
     int dst;
};

//class "MetaNode" describe the link between a state and transition
template<class T>
class MetaNode 
{
 public:
  
  MetaNode(std::list<Transition<T> >::iterator ptr);
  MetaNode(const MetaNode<T>& n);
  std::list<Transition<T> >::iterator getTransPtr(void) const;
  void setTransPtr(std::list<Transition<T> >::iterator ptr );
 private:     	
  std::list<Transition<T> >::iterator transPtr; //pointer to Transition
};


/* ********************************** */
/* *        Defining a state        * */
/* ********************************** */   

template<class T>
class  State 
{
 public:
  // normal constructor
  State(int id, string name,ListOfSymbols his, bool accFlag, bool coaccFlag);
  //copy constructor
  State(const State& state);
  bool operator == (const State& s);
  int  getIdOfState(void) const;
  void setIdOfState(int id);
  string  getName(void) const;
  void setName(string n);
  ListOfSymbols getHistory(void) const; // product a new list
  void setHistory(const ListOfSymbols& his); // dsttroy the old
  bool getAccessibleFlag(void);
  void setAccessibleFlag(bool flag);
  bool getCoaccessibleFlag(void);
  void setCoaccessibleFlag(bool flag);
  //The methods work with the entrances 
  std::list<MetaNode<T> > getAllEntrances(void) const; // product the new
  void setAllEntrances(const std::list<MetaNode<T> >& list); // destroy the old
  
  void addEntrance(std::list<Transition<T> >::iterator ptr); // add an entrance to list
  
  bool deleteEntrance(std::list<Transition<T> >::iterator ptr); //delete an entrance from list
 
 
  //The methods work with the exits 
  std::list<MetaNode<T> > getAllExits(void) const; // product the new
  void setAllExits(const std::list< MetaNode<T> >& list); // destroy the old
  void addExit(std::list<Transition<T> >::iterator ptr); // add an exit to list
  bool deleteExit(std::list<Transition<T> >::iterator ptr); //delete an exit from list

  bool isExitByLabel(const T& lab);    
  bool isInitialDummy(void); 
  bool isTerminalDummy(void);

 private: 
       int  idOfState;
       string     name;
       ListOfSymbols   history;   // to climb to his parents       
       bool accessibleFlag;
       bool coaccessibleFlag;
       std::list<MetaNode<T> >     entrances; // list of transitions of entrances  
       std::list<MetaNode<T> >     exits;    // list of transitions of exits 
};


/* ********************************** */
/* *      Defining a transition     * */
/* ********************************** */
template<class T>
class Transition 
{
 public:
        Transition(int id, std::list<State<T> >::iterator org, 
                   std::list<State<T> >::iterator dst, const T& lab); 
        Transition(int id, std::list<State<T> >::iterator org, 
                 std::list<State<T> >::iterator dst,NodeTree* tree, string e); 

        Transition(const Transition<T>& trans);
        bool operator == (const Transition<T>& t);
        ~Transition() { }
        int    getIdOfTrans(void);
        void   setIdOfTrans(int id);
        std::list<State<T> >::iterator getOrgPtr(void) const;
        void   setOrgPtr(std::list<State<T> >::iterator s);
        std::list<State<T> >::iterator getDstPtr(void) const;
        void   setDstPtr(std::list<State<T> >::iterator s);
        T      getLabel(void) const;  // product the new

        //special !!!
        //These 2 functions is used for processing direct of operators on tree
        //"TreeLabel" 
        NodeTree*     getLabelPointer(void) { return label.getTree();}
        void          setLabelPointer(NodeTree* t) { label.setTree(t); }
        bool          empty(void); 
        void   setLabel(const T& lab); // destroy the old
        bool   isSpontaneous(void);
 private:
  	int    idOfTransition;
  	std::list<State<T> >::iterator  orgPtr ;
  	std::list<State<T> >::iterator  dstPtr ;
  	T      label; 
};

//abstract class 
//It don't never exist his instance 

class Graph 
{
  public:
    virtual void    setName(string)=0;
    virtual string  getName(void) const=0;
    virtual void    setInitialName(string)=0;
    virtual void    setTerminalName(string)=0; 
    virtual bool    store(string filename)=0;
    virtual int     who(void)=0; 
    virtual Graph*  reverse(string)=0;
    virtual void    display(long)=0;
    virtual std::vector<string> getAlphaSet(void)=0;
};

/* ********************************* */
/*           Class ITGraph         * */
/* ********************************* */

//class ITGraph contains all of base methodes for 
//"Automata", "GeneralAuto", "Transduccer" ...
template<class T>
class ITGraph:public Graph  
{
public:
        //constructors
        ITGraph(string name);
  	ITGraph(string name, int numberOfStates);
        ITGraph(const ITGraph<T>& graph);
        //copy ITGraph g to "this"
        void   reproduct(const ITGraph<T>& g);
        //quickly version of "reproduct" but too many of memory
        void   copyFast(ITGraph<T>* g);
        //view history infos of ITGraph and all of states  
        void   infos(void);

        //the methods of access
 	string getName(void) const;
	void   setName(string name);
        //for compatibility with "Graph"
        std::vector<string> getAlphaSet(void){return std::vector<string>(0); }
        //
        string getInitialName(void) const; 
        void   setInitialName(string name);
        string getTerminalName(void) const;
        void   setTerminalName(string name);
        bool   isTerminalState(string name);
        bool   isInitialState(string name);
        bool   isLastState(std::list<State<T> >::iterator i);
        bool   isStateByName(string name);
        bool   isLastTrans(std::list<Transition<T> >::iterator i);
        
        int   getId(void) const; 
        void  setId(int num);
  	int   getNoStates(void) const;
	void  setNoStates(int num);
	int   getNoTrans(void);
	void  setNoTrans(int num);
	int   getNoInitStates(void);
	void  setNoInitStates(void);
	void  setNoInitStates(int num);
	int   getNoTermStates(void);
	void  setNoTermStates(void);
	void  setNoTermStates(int num);
        //get all of iterators that point to states(trans)
        //It is usefully when we want to access to "listOfStates"(listOfTrans)
        // from outside
	std::list<std::list<State<T> >::iterator>  
             getIterListOfStates(void);
	std::list<std::list<Transition<T> >::iterator>
             getIterListOfTrans(void);
        //get all of names of terminal states(initial states) 
        ListOfSymbols getTermNames(void) const;
        ListOfSymbols getInitNames(void) const;
       
        //the methodes of manipulation
        void  renameStates(std::vector<string> strs);
        int   getNoOrder(std::list<State<T> >::iterator i);
        //set initial state ( terminal)
        bool  setInitialState(string name, char em=EMPTY_WORD);
        bool  setTerminalState(string name,char em=EMPTY_WORD);

  	bool  addState(string name);
        bool  addState(State<T>* s );
        bool  addState(string name, ListOfSymbols his);
        bool  addState(string name, int id, ListOfSymbols his);
        //This function do not test si state "name" is exist
        std::list<State<T> >::iterator 
              addState(string name, ListOfSymbols his, bool flag);

        bool  deleteState(string name);
        bool  deleteState(std::list<State<T> >::iterator s);
	bool  addTransition(string nameOrg, string nameDst,
                            const T& label);
        bool  addTransition(std::list<State<T> >::iterator orgPtr, 
              std::list<State<T> >::iterator dstPtr,const T& label); 

        //This function do not test si transition is exist
        bool  addTransition(std::list<State<T> >::iterator orgPtr, 
              std::list<State<T> >::iterator dstPtr,const T& label,bool f);

        bool  addTransition(std::list<State<T> >::iterator orgPtr, 
              std::list<State<T> >::iterator dstPtr,
              NodeTree* label, string empty, bool f);

        bool  deleteTransition(std::list<Transition<T> >::iterator i);
	bool  deleteTransition(string nameOrg,string nameDst,
                               const T& label);
              
        bool  deleteTransition(std::list<State<T> >::iterator nameOrg,
              std::list<State<T> >::iterator nameDst,const T& label); 
       
        bool  deleteTransition(std::list<State<T> >::iterator orgPtr,
                                  std::list<State<T> >::iterator dstPtr);

	bool  setStateHist(string name, const ListOfSymbols& his);
	ListOfSymbols getStateHist(string name);		
	int   getHistType(void) const;
	void  setHistType(int type);
	ListOfSymbols getHistOfITGraph(void) const;
	void  setHistOfITGraph(ListOfSymbols his);
	int   getDetermFlag(void) const;
	void  setDetermFlag(int flag);
	int   getCodetermFlag(void) const;
     	void  setCodetermFlag(int flag);
	int   getAmbiguousFlag(void) const;
	void  setAmbiguousFlag(int flag);
        int   getCompleteFlag(void) const;
	void  setCompleteFlag(int flag);
        int   getNormalFlag(void) const;
	void  setNormalFlag(int flag);
        std::list<NameHist>         getNameHists(void) const;
        std::list<OrgDstLabel<T> >  getOrgDstLabels(void) const;

        std::list<std::list<Transition<T> >::iterator> getTransIters(void);
	//void  setListOfStates(ListOfInts his);
        // new
        // search a transition with label
        bool searchTransitionByLabel(
             std::list<State<T> >::iterator i1, 
             std::list<State<T> >::iterator i2,const T& lab);
    
        // search a transition
        std::list<Transition<T> >::iterator searchTransition(
             std::list<State<T> >::iterator org, 
             std::list<State<T> >::iterator dst, const T& label);
        //search a transition by pointeur 'org' and 'dst'  
        std::list<T> searchTransByOrgDst(
             std::list<State<T> >::iterator org, 
             std::list<State<T> >::iterator dst);

        // search a state by name
	std::list<State<T> >::iterator searchStateByName(string name);
        // search a state by history
	std::list<State<T> >::iterator searchStateByHistory
                                       (const ListOfSymbols& list);
        // find all of successors of a state with a label given
        std::list< State<T> >  successors(string name, const T& lab);
        // find all of predcessors of a state with a label given
        std::list< State<T> >  predcessors(string name, const T& lab);
        ITGraph<T>* unionG(const ITGraph<T>& graph, string name);
        ITGraph<T>* concatenateG(const ITGraph<T>& graph, string name);	
        string      intToString(long i);
        void        printChar(char a, int times);
        int         getNoColWin(long id);  
        virtual int who(void) { return ITGRAPH; }
        virtual ITGraph<T>* reverse(string name);
        virtual ~ITGraph() {};
        virtual bool   store(string filename) {}
        virtual void   display(long winId){}

protected:
        int  maxIdState; // to create key for the states
        int  maxIdTransition; // to create key for the transitions
        //the lists  
        std::list<State<T> >  listOfStates;
	std::list<Transition<T> >  listOfTrans;
 	ListOfSymbols  histOfITGraph;//2 elements point to 2 names of components              
private:       
  	string           name; // name of ITGraph
        int              id; // the key and other purposes
  	int              numberOfStates;
	int              numberOfTransitions;
	int              numberOfInitStates;
	int              numberOfTermStates;
	int              histType ;//values constants 
	int              deterministicFlag; // 3 values: -1,0,1 
	int              codeterministicFlag;// 3 values: -1,0,1 
	int              ambiguousFlag; // 3 values: -1,0,1
	int              completeFlag; // 3 values: -1,0,1
        int              normalFlag; // 3 values: -1,0,1
       
}; 

template<class T>
class Interface
{
  public:
      Interface();
      //Those functions allow to enter an automaton, transduccer
      //from keyboard 
      Automata<T>*    createAutomata(string name);
      GeneralAuto<T>* createGeneralAuto(string name);
      Transducer<T>*  createTransducer(string name);
      bool            writeITGraph(const ITGraph<T>& graph);
      ITGraph<T>*     readITGraph(const ITGraph<T>& graph);
      static string   intToString(int i);
      static void     strToExp(string str, std::vector<string>&exp, 
                               bool point=false); 
      //Those functions allow to enter an expression
      //from keyboard with test of syntax.  
      static int      readStrings(std::vector<string>& v,string& s);
      static int      readLabel(string& v,std::vector<string>&exp,
                           std::vector<string>&alpha );
      static int      readStateWithLabel(std::vector<string>& v,
                      std::vector<string>& outExp,std::vector<string>&alpha );
      virtual int     who(void){ return INTERFACE;}
};


template<class T>
class Automata: public ITGraph<T> 
{
public:
   Automata(string name):ITGraph<T>(name){}
   Automata(string name, int noOfStates, std::vector<string> alpha);
   Automata(string filename, string name, long pos);
   Automata(const Automata<T>& autoT);
   Automata(const ITGraph<T>& graph,std::vector<string> alpha);
   
   int unsigned  getAlphaSize(void);    
   void setAlphaSet(std::vector<string>& alpha);
   std::vector<string> getAlphaSet(void) const;
   void setAccessible(void);
   void setCoaccessible(void);
   void setDeterministic(void);
   void setCodeterministic(void);
   void setAmbiguous(void);
   void setComplete(void);
   void accessAutoSelf(void);
   void coaccessAutoSelf(void);
   void completeSelf(void);
   bool emptyLang(void);
   bool recognize(std::vector<string>& word);

   Automata<T>* accessAuto(string name);
   Automata<T>* coaccessAuto(string name);
   void         trimAuto(string name);
   Automata<T>* determinize(string name); 	
   Automata<T>* codeterminize(string name); 
   
   Automata<T>* complement(string name);
   Automata<T>* directComp(string name,std::vector<string>& alpha,
                           bool flag=false); 
   Automata<T>* minimize(string name); 
   Automata<T>* delAllSpon(string name);     

   Automata<T>* wordAuto(std::vector<string>& word);	
   Automata<T>* productAutos(Automata<T>* auto1,string name);
  
   //divers  
   //delete of spontaneous transitions
   std::list<Binome<T> > getSponList(void);
   T getSponLabel(void);
   bool   isEmptyWord(string& word); 
   string getEmptyWord(void) { return alphaSet[0]; }
   bool  deleteSponTrans(std::list<State<T> >::iterator orgPtr, 
                              std::list<State<T> >::iterator dstPtr);

   bool  addSponTrans(std::list<State<T> >::iterator orgPtr, 
                           std::list<State<T> >::iterator dstPtr);

   //for building language ( used in function "deleteState" )
   bool createTuples(std::list<State<T> >::iterator i,
                     std::list<Quadre<T> >& tuples,T& startLab);
   void testMetaNodes(void);
   virtual bool    store(string filename);
   virtual Automata<T>*  reverse(const string name = "reverse_");
   virtual ~Automata() {}
   virtual int who(void) { return AUTOMATA; }
   virtual void   display(long winId);
   
protected:
   ITGraph<T>::listOfStates;
   ITGraph<T>::listOfTrans;

private:	

   //the first element is empty word !!!
   std::vector<string> alphaSet;
   std::list<LabelSet<T> >  createDstSets(const NameSet<T>&  ele);
   virtual void     displaySmall(void);
   virtual void     displayBig(long winId); 
};

class NodeTree
{
  public:
    NodeTree(string s, bool b, NodeTree* l,NodeTree* r,NodeTree* p ):
             symbolS(s), isSymbol(b), left(l), right(r), parent(p) {} 
    string symbolS;
    bool isSymbol;
    NodeTree* left;
    NodeTree* right;
    NodeTree* parent;
};

// class TreeLabel, representation of expression in a label of transition
// This class implement the most important algorithms !!! 

// An "TreeLabel"is empty if
// 1) "emptyWord" is empty value for user
// 2) "tree" == NULL
// 3) "infixSerie.size"==0
// 4) "zero" <> 0
//or
// If "infixSerie.size"==1 then infixSerie[0] == emptyWord
// and "tree"==NULL 
//or function "empty" return value "true"
 
class  TreeLabel 
{
 public:

  TreeLabel(char empty=EMPTY_WORD) 
    { zero=false; tree = NULL;
      emptyWord = "";
      emptyWord = emptyWord + empty; 
      infixSerie.resize(0);
    } // pay attention !

  TreeLabel(NodeTree* t,char empty=EMPTY_WORD) 
    { zero=false; tree = t;
      emptyWord = "";
      emptyWord = emptyWord + empty; 
      infixSerie.resize(0);
    } // pay attention !

  TreeLabel(bool f) 
    { zero=f; tree = NULL;
      emptyWord = "";
      emptyWord = emptyWord + EMPTY_WORD; 
      infixSerie.resize(0);
    } // pay attention !

  // right expression
  //"expression" can include the empty word that is considered an operand !  
  TreeLabel(std::vector<string>& expression,char empty=EMPTY_WORD); 

  TreeLabel(const TreeLabel& treeLab);
  NodeTree* getTree(void) { return tree;}
  void      setTree(NodeTree* t) { tree = t;}
  //test syntax of expression
  static int test(std::vector<string>& expression, 
                  std::vector<string>& alpha, bool skip=false);
  bool testWord(bool point=false); 
  std::vector<string> getInfixSerie(void) const { return infixSerie; }
  void setInfixSerie(std::vector<string>& infix)  { infixSerie = infix; }
  int unsigned getInfixSize(void) const { return infixSerie.size();}
  int unsigned size(void) const 
      {string tmp = getInfixString(); return tmp.size();}
  string getInfixString(void) const;
  ~TreeLabel();
  void operator = (const TreeLabel& p1);
  TreeLabel operator * (void);
  TreeLabel operator / (const TreeLabel& p1); // to replace the operator '.'
  TreeLabel operator + (const TreeLabel& p1);
  bool operator == (const TreeLabel& p1);
  bool operator < (const TreeLabel& p1) const;
  bool empty(void) const;//empty "TreeLabel"
  bool isZero(void) const;//null word
  bool isEmptyWord(string& w) const;//empty word
  string getEmptyWord(void) const;
  void setEmptyWord(string w);
  TreeLabel linearize(std::map<string,string>& h,std::vector<string>& alpha);
 
  NodeTree* copyTree(NodeTree* t,NodeTree* parent); 
  static string intToString(int);
  static bool isSymbol(char a);
  
  TreeLabel c_continuation(NodeTree* var,ListOfSymbols& first);
  TreeLabel toProjection(std::map<string,string>& h); 
  bool lambdaOfExp(void) const;
  bool reLambdaOfExp(NodeTree* t) const;
  void varList(std::list<NodeTree*>& l);
  void reVarList(NodeTree* tree,std::list<NodeTree*>& l);

  void print(void)
    {
      rePrint(tree);
    }

  void rePrint(NodeTree* t)
    {
      if(t!=NULL) { rePrint(t->left); cout<<t->symbolS<<" ";
      rePrint(t->right);}
    }

  void  firstSet(ListOfSymbols& f);

  TreeLabel TreeLabel::reChoiceWord(NodeTree* t);
  TreeLabel choiceWord(void);  

  Automata<TreeLabel>* expToAuto(string name,std::vector<string>&alpha);  
  std::vector<string> createInfixSerie(NodeTree* t); 
  void      delTree(NodeTree* t);

 protected:
 
  NodeTree* reTreeLabel(NodeTree*t, int l, int r,
            std::vector<string>& ex,int& error);
  unsigned int getPrio(char a);  
  

  NodeTree* toLinear(NodeTree* t,NodeTree* parent,
            std::map<string,string>& h,std::vector<string>& alpha, int& count);
  void createCMap(TreeLabel& linear, std::map<string,ConFirst>& cMap);
  void cMapToQuoMap(std::map<string,ConFirst>& cMap,
        std::map<string,string>&  h, std::map<TreeLabel,ListOfSymbols>& quo);
  
  //data
  std::vector<string> infixSerie;
  NodeTree* tree;
  string emptyWord;
  bool zero;
};

class  ExTreeLabel: public TreeLabel
{

public:

    ExTreeLabel(std::vector<string>& alpha,
                char empty=EMPTY_WORD):TreeLabel(empty),alphaSet(alpha){}

    ExTreeLabel(std::vector<string>& exp,std::vector<string>& alpha,
                char empty=EMPTY_WORD);
 
    ExTreeLabel(std::vector<string>& alpha,bool f):
                TreeLabel(f),alphaSet(alpha) { } // pay attention !

    ExTreeLabel::ExTreeLabel(const TreeLabel& lb, std::vector<string>& alpha)
                 :TreeLabel(lb),alphaSet(alpha) { }

    ~ExTreeLabel() { }
    std::vector<string> getAlpha(void);
    static int test(std::vector<string>& exp, 
                    std::vector<string>& alpha, bool skip=false);

    //this function return result on "t" 
    void deleteSubInter(NodeTree*& t);

    //this function copy to "TreeLabel" 
    TreeLabel  toTreeLabel(std::vector<string>& alpha);
    void reToTreeLabel(NodeTree*& t,std::vector<string>& alpha);

protected:
 
    TreeLabel::infixSerie;    
    TreeLabel::tree;
    TreeLabel::emptyWord;
    TreeLabel::zero; 
    NodeTree* reTreeLabel(NodeTree*t, int l, int r,
            std::vector<string>& ex,int& error);

    std::vector<string> alphaSet;
};

class ConFirst
{
  public:
     ConFirst(){};
     ConFirst(ListOfSymbols l,TreeLabel t):first(l),tree(t) { } 
     ListOfSymbols first;
     TreeLabel tree;
     ~ConFirst(){}
};


template<class T>
class GeneralAuto: public Automata<T> 
{
  public: 
   GeneralAuto(string name):Automata<T>(name)
   {
     this->additionTrans();
   }

   GeneralAuto(const Automata<T>& auto1);

   GeneralAuto(const Automata<string>& auto1);

   GeneralAuto(std::string name, int noOfStates, std::vector<string> alpha);

   GeneralAuto(string filename, string name, long pos):
     Automata<T>(filename,name,pos){}

   GeneralAuto(const GeneralAuto<T>& generalAuto);
   void additionTrans(void);
   void additionTrans(std::list<State<T> >::iterator org,
                      std::list<State<T> >::iterator des);
   bool deleteState(std::string name);
   void deleteState(std::list<State<T> >::iterator i);
   T    language(ListOfSymbols& order);

   //if l.size()==1 and infix[0]=="emptyWord" then 
   //language = empty word !!!
   //if l.size()==0 then zero language    

   void language(std::vector<string>& l);
   void heapLanguage(std::vector<string>& l);
   
   void ancestors(std::list<State<T> >::iterator i,std::vector<int>& index);
   void successors(std::list<State<T> >::iterator i,std::vector<int>& index);
   int getNumAncestors(std::list<State<T> >::iterator i);
   int getNumSuccessors(std::list<State<T> >::iterator i);

   virtual ~GeneralAuto() {}  
   virtual int who(void) { return GENERAL_AUTOMATA; }
  
  protected:
   Automata<T>::listOfStates;
   Automata<T>::listOfTrans;
};

//This class represent the label for transition of transducer
// An "TransducerLabel" is empty if
// 1) "output" is empty TreeLabel
// 2) "input"=="" ( empty string)

template<class T>
class TransducerLabel
{
  public:
    TransducerLabel(string t, T u);
    TransducerLabel(bool z);
    TransducerLabel(const string t="");
    //dummy
    TransducerLabel(char a){}
    TransducerLabel(const TransducerLabel<T>& lab);
    void operator = (const TransducerLabel<T>& t);
    ~TransducerLabel(){}   
    bool isEmptyInput(void);
    bool isEmptyOutput(void);
    bool operator == (const TransducerLabel<T>& t);
    bool operator < (const TransducerLabel<T>& t) const;
    bool empty(void);
    string getInput(void) const { return input;}
    T getOutput(void) const { return output;}

    //dummy
    NodeTree* getTree(void) { return NULL;}
    void      setTree(NodeTree* t) { }

    void setInput(const string&t)  { input  = t;}
    void setOutput(const T&u) { output = u;}  
    int unsigned size() { return (input.size()+output.size());}
    //dummy
    int unsigned getInfixSize(void) { return (input.size()+output.size());}
    string getInout(void) { return inout;}
    void   setInout(string t)  { inout  = t;}

    //dummy function for determinization of LTOLTransducer
    std::vector<string> getInfixSerie() const; 
    string getInfixString() const; 
    bool isZero() { if(zero) return true; else return false;} 

  private:
    string input;
    T output;
    string inout;
    bool zero;
};

//class "Transducer" contains the base functions for
//the class "LTOLTransducer","LTOEXPTransducer","WTOEXPTransducer"  
template<class T>
class Transducer: public ITGraph<T> 
{
  public:
   Transducer(string name):ITGraph<T>(name){}
   Transducer(string name, int noOfStates, std::vector<string> inAlpha, 
                           std::vector<string> outAlpha); 
   Transducer(const ITGraph<T>& itgraph, std::vector<string> inAlpha, 
              std::vector<string> outAlpha); 
   Transducer(string filename, string name, long pos);
   Transducer(const Transducer<T>& t);
   std::vector<string> getInputAlpha(void) const;
   bool isInputEmptyWord(string& w);
   std::vector<string> getOutputAlpha(void) const;
   bool isOutputEmptyWord(string& w);
   int unsigned getInputAlphaSize(void) const {return inputAlpha.size();}
   int unsigned getOutputAlphaSize(void) const {return outputAlpha.size();}
   void    setInputAlpha(std::vector<string>& t);
   void    setOutputAlpha(std::vector<string>& t);
   
   //for compatibility with "Graph"
   std::vector<string> getAlphaSet(void) {return std::vector<string>(0);}
   Automata<TreeLabel>* autoWord(std::vector<string>& word);
   bool setOnInitialState(string name,string input,const TreeLabel& output); 
   bool setOnInitialState(string name,const T& lab);  
   bool setOffInitialState(string name,string input,const TreeLabel& output);
   bool setOffInitialState(string name,const T& lab);
   bool setOffInitialState(void); 

   bool setOnTerminalState(string name,string input,const TreeLabel& output);
   bool setOnTerminalState(string name,const T& lab);
   bool setOffTerminalState(void);
   bool setOffTerminalState(string name,string input,const TreeLabel& output);
   bool setOffTerminalState(string name,const T& lab);
   //"productTA_T" compute product of transducter and automaton w.r.t
   // input of transducer
   Transducer<T>* productTA_T(Automata<TreeLabel>* auto1, string name); 
   GeneralAuto<TreeLabel>* getImage(string name);

   virtual Transducer<T>* uniformize(string name);
  
   virtual Automata<TreeLabel>* getDomain(string name);        
   virtual Transducer<T>*  reverse(string name);
   virtual void   display(long id);
   virtual ~Transducer() {}
   virtual int who(void) { return TRANSDUCER; }
   virtual bool    store(string filename);
  
  protected:
   ITGraph<T>::listOfStates;
   ITGraph<T>::listOfTrans;

  private:
   std::vector<string>  inputAlpha;
   std::vector<string>  outputAlpha; 
  
   virtual void displaySmall(void);
   virtual void displayBig(long id);  

};

template<class T>
class LTOLTransducer: public Transducer<T> 
{  
 public:
   LTOLTransducer(string name):Transducer<T>(name) { }
   LTOLTransducer(string name, int noOfStates, std::vector<string> inAlpha,
                               std::vector<string> outAlpha):
       Transducer<T>(name,noOfStates,inAlpha,outAlpha){}
   LTOLTransducer(const ITGraph<T>& itgraph,std::vector<string> inAlpha,
                                            std::vector<string> outAlpha):
       Transducer<T>(itgraph,inAlpha,outAlpha) {} 
   LTOLTransducer(string filename, string name, long pos):
                  Transducer<T>(filename,name,pos) {}
   LTOLTransducer(const LTOLTransducer<T>& t);
   LTOLTransducer(const Transducer<T>& t):Transducer<T>(t) { }
   LTOLTransducer<T>* compose2T( LTOLTransducer<T>* t, string name);
   LTOLTransducer<T>* reverseInOut(string name);

   void setDeterministic(void);
   virtual LTOLTransducer<T>*  uniformize(string name);
   virtual LTOLTransducer<T>*  reverse(string name);
   virtual ~LTOLTransducer() {}
   virtual int who(void) { return  LTOL_TRANSDUCER; }
   
protected:
   Transducer<T>::listOfStates;
   Transducer<T>::listOfTrans;
};

template<class T>
class WTOEXPTransducer;

template<class T>
class LTOEXPTransducer: public Transducer<T> 
{  
 public:
   LTOEXPTransducer(string name):Transducer<T>(name) { }
   LTOEXPTransducer(const Transducer<T>& t):Transducer<T>(t) { }
   LTOEXPTransducer(string name, int noOfStates,std::vector<string> inAlpha,
                                 std::vector<string> outAlpha):
                    Transducer<T>(name,noOfStates,inAlpha,outAlpha){}
   LTOEXPTransducer(string filename, string name, long pos):
         Transducer<T>(filename,name,pos) {}
   LTOEXPTransducer(const LTOEXPTransducer<T>& t);
   LTOEXPTransducer(const WTOEXPTransducer<T>& t);
   LTOEXPTransducer(const LTOLTransducer<T>& t);

   bool testLTOW(void);

   void traceExp(std::vector<string> alpha, TreeLabel& exp, 
                 std::list<State<T> >::iterator org1, 
                 std::list<Trace<T> >& trace, bool terminal);
   std::list<Trace<T> > 
        cleanTrace2(std::list<Trace<T> >& trace);
   LTOEXPTransducer<T>* reverseInOut(string name);
   LTOEXPTransducer<T>* composeLTOW(LTOEXPTransducer<T>* t, string name);
   LTOEXPTransducer<T>* composeLTOEXP(LTOEXPTransducer<T>* t, string name);
   
   virtual ~LTOEXPTransducer() {}
   virtual LTOEXPTransducer<T>*  uniformize(string name);
   virtual LTOEXPTransducer<T>*  reverse(string name);
   virtual int who(void) { return  LTOEXP_TRANSDUCER; }

protected:
   Transducer<T>::listOfStates;
   Transducer<T>::listOfTrans;
};

template<class T>
class WTOEXPTransducer: public Transducer<T> 
{  
public:
   WTOEXPTransducer(string name):Transducer<T>(name) { }
   WTOEXPTransducer(string name, int noOfStates,std::vector<string> inAlpha,
                                 std::vector<string> outAlpha):
       Transducer<T>(name,noOfStates,inAlpha,outAlpha){}
   WTOEXPTransducer(string filename, string name, long pos):
         Transducer<T>(filename,name,pos) {}
   WTOEXPTransducer(const Transducer<T>& t):Transducer<T>(t) { }
   WTOEXPTransducer(const WTOEXPTransducer<T>& t);
   WTOEXPTransducer(const LTOEXPTransducer<T>& t);

   void  starZ(TreeLabel**& matrix, int size);     
   WTOEXPTransducer<T>*   wordToLetter(string name);
   LTOEXPTransducer<T>*   delEmptyInput(string name);    
   LTOEXPTransducer<T>*   toLTOEXP(string name);
   LTOEXPTransducer<T>*   reverseInOut(string name);
   bool    testWTOW(void);
   bool    testInputIsEmpty(void);
   virtual GeneralAuto<TreeLabel>* getDomain(string name); 
   virtual ~WTOEXPTransducer() {}
   virtual WTOEXPTransducer<T>*  reverse(string name);
   virtual int who(void) { return  WTOEXP_TRANSDUCER; }
   
protected:
   Transducer<T>::listOfStates;
   Transducer<T>::listOfTrans;
};

template<class T, class U>
class Boot
{
  public:

  Boot() 
  {
    maxIdAuto = 1;
    maxIdGen  =1;
    maxIdTransducer = 1;
  }

  Boot(const Boot<T,U>& b) 
  {
    // do many thing !!??
  }

  ~Boot()
  {
    std::map<string,std::vector<Graph*> >::iterator i1;
    for(i1=autoList.begin();i1!=autoList.end();++i1)
       delete(i1->second[0]);

    std::map<string,std::vector<TreeLabel*> >::iterator i2;
    for(i2=expList.begin();i2!=expList.end();++i2)
      { delete(i2->second[0]);delete(i2->second[1]); } 

    std::list<Graph* >::iterator i0;
    for(i0=undoList.begin();i0!=undoList.end();++i0)
       delete (*i0);

  }
  
  void analyser(std::vector<string> command);     
  void createFunc(std::vector<string> command);
  void killFunc(std::vector<string> command);
  void displayFunc(std::vector<string> command);
  void listFunc(std::vector<string> command); 
  void unionFunc(std::vector<string> command); 
  void determinizeFunc(std::vector<string> command); 
  void codeterminizeFunc(std::vector<string> command);
  void minimizeFunc(std::vector<string> command);
  void reverseFunc(std::vector<string> command);
  void productFunc(std::vector<string> command);
  void complementFunc(std::vector<string> command);
  void completeFunc(std::vector<string> command);
  void coaccautoFunc(std::vector<string> command);
  void accautoFunc(std::vector<string> command);
  void isambiguousFunc(std::vector<string> command);
  void delstateFuncA(std::vector<string> command);
  void delstateFuncG(std::vector<string> command);
  void delstateFuncGH(std::vector<string> command);

  void trimFunc(std::vector<string> command);
  void shell(string name); 
  void copyFunc(std::vector<string> command);
  void iscodetermFunc(std::vector<string> command);
  void isdetermFunc(std::vector<string> command);
  void isemptylangFunc(std::vector<string> command);
  void recognizeFunc(std::vector<string> command);
  void addstateFunc(std::vector<string> command);
  void addtransFunc(std::vector<string> command);
  void deltransFunc(std::vector<string> command);
  void renamestateFunc(std::vector<string> command);
  void concatFunc(std::vector<string> command);
  void checkFunc(std::vector<string> command);
  void delspontransFunc(std::vector<string> command);
  void displayHisOfAuto(ListOfSymbols s,int);
  void displayHisOfState(ListOfSymbols s, ListOfSymbols t, int type,int);
 
  void producttwFunc(std::vector<string> command);
  void producttaFunc(std::vector<string> command);
  void compose2tFunc(std::vector<string> command);
  void imageFunc(std::vector<string> command);
  void domainFunc(std::vector<string> command);
  void composeLTOEXPFunc(std::vector<string> command); 
  void toLTOEXPFunc(std::vector<string> command); 
  void toLUTOEXPFunc(std::vector<string> command);
  void expToAutoFunc(std::vector<string> command);
  void reverseInOutFunc(std::vector<string> command);
  void uniformizeFunc(std::vector<string> command);

  void defineFunc(std::vector<string> command);
  void lookFunc(std::vector<string> command);

  bool loadAutoFunc(std::vector<string> command);
  bool replace(string n1,string n2,Graph* tmp);
  bool storeAutoFunc(std::vector<string> command);
  void storeAllFunc(std::vector<string> command);
  void deleteAutoFunc(std::vector<string> command);
  bool viewFunc(std::vector<string> command);
  std::map<string,int> getNames(string filename);
  //called only for initialization
  void loadAllFunc(std::vector<string> command);

  int  filter2N1P(std::vector<string>& command);
  int  filter2Paras(std::vector<string>& command, string action);
  int  filter1Para(std::vector<string>& command, string action);
  int  filter3Name(std::vector<string>& command);
  int  filter1Name(std::vector<string>& command);
  int  filter2Name(std::vector<string>& command);
  int  filterTrans(std::vector<string>& command);
  bool isSameAlpha(std::vector<string> a1,std::vector<string> a2);
  
  void    help(void);
  void    prompt(void);
  int     readStrings(std::vector<string>& v,string& s); 
  int     readStrings(std::vector<string>& v,string tmp, bool flag);
  void    printLine(char ch, int num);
  void    printBlancs(int num);
  string  intToString(int i);
  void    updateWinId(void);
  long    getWinId(void) const{ return winId;}
  void    setWinId(long id) {winId = id;}
 
private:

     //id of window
     long winId;
     //automata index
     int maxIdAuto;
     //general automata index
     int maxIdGen;
     //transducer index
     int maxIdTransducer;

     // list of Graphs for machines
     std::map<string,std::vector<Graph*> >  autoList;
   
     // list of expressions
     // std::vector<TreeLabel*>[0] : TreeLabel
     // std::vector<TreeLabel*>[1] : ExTreeLabel
     std::map<string,std::vector<TreeLabel*> >  expList;
    
     // list of deleted objects 
     std::list<Graph* > undoList;

};//end of class Boot








