 
//Date: 6/4/2000

/* ************************************************************************* */
/* *                         File -- ITGraph.cpp                           * */
/* ************************************************************************* */



/* ************************************************ */
/* *        Defining the methodes of state        * */
/* ************************************************ */   

template<class T>
MetaNode<T>::MetaNode(std::list<Transition<T> >::iterator ptr)
        { this->transPtr = ptr; }

template<class T>
MetaNode<T>::MetaNode(const MetaNode<T>& n)
{ this->transPtr = n.transPtr; }

template<class T>
std::list<Transition<T> >::iterator MetaNode<T>::getTransPtr(void) const
        { return  this->transPtr;}

template<class T>
void MetaNode<T>::setTransPtr(std::list<Transition<T> >::iterator ptr ) { this->transPtr = ptr;}

  // normal constructor
template<class T>
State<T>::State(int id, string name, ListOfSymbols his, bool accFlag, bool coaccFlag)
       {  this->idOfState = id;
          this->name      =  name;
          this->history  = ListOfSymbols(his);
          this->accessibleFlag   =  accFlag;
          this->coaccessibleFlag =  coaccFlag;
       } 

template<class T>  //copy constructor
State<T>::State(const State<T>& state)
       {  
          this->idOfState        =  state.idOfState;
          this->name      =  state.name;
          this->history          =  ListOfSymbols(state.history);
          this->accessibleFlag   =  state.accessibleFlag;
          this->coaccessibleFlag =  state.coaccessibleFlag;
          this->entrances        =  std::list<MetaNode<T> >(state.entrances);
          this->exits            =  std::list<MetaNode<T> >(state.exits);       
       }
template<class T>
int State<T>::getIdOfState(void) const 
{ return idOfState; }   
template<class T>    
void State<T>::setIdOfState(int id) { this->idOfState = id; }

template<class T> string  State<T>::getName(void) const 
{ return this->name; }

template<class T> void State<T>::setName(string n) { this->name = n; }
template<class T>
ListOfSymbols State<T>::getHistory(void) const  // product a new list
{
  ListOfSymbols tmp = ListOfSymbols(this->history);
  return tmp;
}
template<class T>
void State<T>::setHistory(const ListOfSymbols& his) // destroy the old
{
  this->history.clear();
  this->history = his; //the old is automaticlly destroyed  
}
template<class T>
bool State<T>::getAccessibleFlag(void)
{
  return this->accessibleFlag;
}
template<class T>
void  State<T>::setAccessibleFlag(bool flag)
{
   this->accessibleFlag = flag;
}
template<class T>
bool State<T>::getCoaccessibleFlag(void)
{
  return this->coaccessibleFlag;
}
template<class T>
void State<T>::setCoaccessibleFlag(bool flag)
{
   this->coaccessibleFlag = flag;
}

template<class T> //The methods work with the entrances 
std::list<MetaNode<T> > State<T>::getAllEntrances(void) const // product the new
{
  std::list<MetaNode<T> > tmp = std::list<MetaNode<T> >(this->entrances);
  return tmp;
}
template<class T>
void State<T>::setAllEntrances(const std::list<MetaNode<T> >& list) // destroy the old
{
  this->entrances.clear();
  this->entrances = std::list<MetaNode<T> >(list); // destroy automatic
}
template<class T>
void State<T>::addEntrance(std::list<Transition<T> >::iterator ptr) // add an entrance to list
{
  MetaNode<T> tmp = MetaNode<T>::MetaNode(ptr); 
  this->entrances.insert(this->entrances.end(), tmp); 
}
template<class T>
bool State<T>::deleteEntrance(std::list<Transition<T> >::iterator ptr)//delete an entrance from list
{
  std::list<MetaNode<T> >::iterator i;
  for(i=this->entrances.begin();i!=this->entrances.end();++i)
    if( i->getTransPtr()==ptr ) { this->entrances.erase(i); return true; }
  return false;
}
 
template<class T>  //The methods work with the exits 
std::list<MetaNode<T> > State<T>::getAllExits(void) const// product the new
{
  std::list<MetaNode<T> > tmp = std::list<MetaNode<T> >(this->exits);
  return tmp;
}
template<class T>
void State<T>::setAllExits(const std::list<MetaNode<T> >& list) // destroy the old
{
  this->exits.clear();  
  this->exits = std::list<MetaNode<T> >(list); // destroy automatic
}

template<class T>
void State<T>::addExit(std::list<Transition<T> >::iterator ptr) // add an exit to list
{
  MetaNode<T> tmp = MetaNode<T>::MetaNode(ptr);
  this->exits.insert(this->exits.end(), tmp); 
}

template<class T>
bool  State<T>::deleteExit(std::list<Transition<T> >::iterator ptr) //delete an exit from list
{
  std::list<MetaNode<T> >::iterator i;
  for(i=this->exits.begin();i!=this->exits.end();++i)
    if(i->getTransPtr()==ptr) { this->exits.erase(i); return true; }
  return false;
} 
template<class T>
bool  State<T>::isExitByLabel(const T& l)
{
  std::list<MetaNode<T> >::const_iterator i;
  for(i=exits.begin();i!=exits.end();++i)
     if(i->getTransPtr()->getLabel()==l) return true;
  return false;
}

template<class T>
bool State<T>::isInitialDummy(void)
{
     if(getIdOfState()==0) return true;
     return false;
}

template<class T> 
bool State<T>::isTerminalDummy(void)
{
     if(getIdOfState()==-1) return true;
     return false;
}

/* ******************************************** */
/* *      Defining the methodes of transition * */
/* ******************************************** */
template<class T> 
Transition<T>::Transition(int id, std::list<State<T> >::iterator org, 
                   std::list<State<T> >::iterator dst, const T& lab)
  {  
           this->idOfTransition = id;
           this->orgPtr = org;
           this->dstPtr = dst;
           T tmp = T(lab);
           this->label = T(tmp);
  }

template<class T> 
Transition<T>::Transition(int id, std::list<State<T> >::iterator org, 
               std::list<State<T> >::iterator dst, NodeTree* tree, string e)
  {  
           this->idOfTransition = id;
           this->orgPtr = org;
           this->dstPtr = dst;
           T tmp = T(tree,e[0]);
           this->label = T(tmp);
  }

template<class T> 
Transition<T>::Transition(const Transition<T>& trans)
	{
               this->idOfTransition = trans.idOfTransition;
               this->orgPtr = trans.orgPtr;
               this->dstPtr = trans.dstPtr;
               this->label = T(trans.label);  
        } 



template<class T> 
int    Transition<T>::getIdOfTrans(void) { return this->idOfTransition; }
template<class T> 
void   Transition<T>::setIdOfTrans(int id) { this->idOfTransition = id; }
template<class T> 
std::list<State<T> >::iterator Transition<T>::getOrgPtr(void) const
        {  
               return this->orgPtr;
        }

template<class T>   
void   Transition<T>::setOrgPtr(std::list<State<T> >::iterator s)
        { 
               this->orgPtr = s;
        } 

template<class T> 
std::list<State<T> >::iterator Transition<T>::getDstPtr(void) const
        { 
               return this->dstPtr;
        }
  
template<class T> 
void   Transition<T>::setDstPtr(std::list<State<T> >::iterator s)
        { 
               this->dstPtr = s;          
        }

template<class T> // make a new instance of T 
T  Transition<T>::getLabel(void) const 
        { 
               T tmp = T(label);
               return tmp;
        }     

template<class T>         
void   Transition<T>::setLabel(const T& lab)//make an instance T before
                                          // to pass  
	{
               this->label = T(lab);
	}


template<class T> 
bool Transition<T>::isSpontaneous(void)
{
  if(this->getLabel().empty()) return true;
  else return false;
} 

//very special !!!
template<class T> 
bool Transition<T>::empty(void)
{
  if(this->label.getTree()==NULL) return true;
  if(this->label.getTree()->symbolS==this->label.getEmptyWord()) return true;
  else return false;
} 

/* ************************************** */
/* *   The methodes of class ITGraph    * */
/* ************************************** */

//constructors
template<class T> 
ITGraph<T>::ITGraph(string name)
{
  this->maxIdState = 1 ;
  this->maxIdTransition =  1 ;
  this->numberOfStates = 0; 
  this->numberOfTransitions = 0;
  this->name = name;
  this->histOfITGraph.insert(this->histOfITGraph.end(),"chaos");
  //chaos is name of states that have not his history !
  ListOfSymbols tmp;
  tmp.insert(tmp.end(),"chaos");  
  this->setHistType(NO_HISTORY);
  State<T> s1(0, "i_" + name, tmp, false, false);
  listOfStates.insert(listOfStates.end(),s1);//copy this element tolist
  State<T> s2(-1, "t_" + name, tmp, false, false);
  listOfStates.insert(listOfStates.end(),s2);//copy this element tolis
  deterministicFlag    = -1; // don't know if 
  codeterministicFlag  = -1;
  ambiguousFlag        = -1;
  completeFlag         = -1;
  normalFlag           = -1;  
}

template<class T> 
ITGraph<T>::ITGraph(string name, int number)
{
  int i;

  this->maxIdState = number + 1 ;
  this->maxIdTransition =  1 ;
  this->name = name;
  this->numberOfStates = number;
  this->numberOfTransitions = 0;
  this->histOfITGraph.insert(this->histOfITGraph.end(),"chaos");
  ListOfSymbols tmp;
  tmp.insert(tmp.end(),"chaos");
  this->setHistType(NO_HISTORY);
  //chaos is name of states that have not his history !
  State<T> s1(0, "i_" + name, tmp, false, false);
  listOfStates.insert(listOfStates.end(),s1);//copy this element tolist
  State<T> s2(-1, "t_" + name, tmp, false, false);
  listOfStates.insert(listOfStates.end(),s2);//copy this element tolis
       
  for(i=1;i<=numberOfStates;i++) 
     {
       State<T> s1(i, intToString(i) , tmp, false, false);
       listOfStates.insert(listOfStates.end(),s1);
     }

  deterministicFlag    = -1; // don't know if 
  codeterministicFlag  = -1;
  ambiguousFlag        = -1;
  completeFlag         = -1;
  normalFlag           = -1;
 
}

template<class T> 
ITGraph<T>::ITGraph(const ITGraph<T>& graph)
{ 
  this->maxIdState = 1 ;
  this->maxIdTransition = 1; 
  this->numberOfStates = 0;
  this->numberOfTransitions = 0;
  this->setName(graph.getName());
  this->setHistOfITGraph(graph.getHistOfITGraph());
  this->setHistType(graph.getHistType());
  this->deterministicFlag    = -1; // don't know if 
  this->codeterministicFlag  = -1;
  this->ambiguousFlag        = -1;
  this->completeFlag         = -1;
  this->normalFlag           = -1;
  ListOfSymbols tmp =graph.listOfStates.begin()->getHistory();
  State<T> s1(0, "i_" + graph.getName(), tmp, false, false);
  this->listOfStates.insert(this->listOfStates.end(),s1);
  std::list<State<T> >::const_iterator i = graph.listOfStates.begin();
  ++i;
  tmp =i->getHistory();
  State<T> s2(-1, "t_" +graph.getName() , tmp, false, false);
  this->listOfStates.insert(this->listOfStates.end(),s2);
  this->reproduct(graph);
}

template<class T> 
void ITGraph<T>::reproduct(const ITGraph<T>& g)
{
    std::list<State<T> >::const_iterator iInit;
    std::list<Transition<T> >::const_iterator iTrans;
       
    for(iInit=g.listOfStates.begin();iInit!=g.listOfStates.end();++iInit)
      {
       this->addState(iInit->getName(),iInit->getIdOfState(),
                      iInit->getHistory());
      
      }
    for(iTrans=g.listOfTrans.begin();iTrans!=g.listOfTrans.end();++iTrans)
      {
        this->addTransition(iTrans->getOrgPtr()->getName(),
                            iTrans->getDstPtr()->getName(),iTrans->getLabel());
      }
    this->histOfITGraph.insert(this->histOfITGraph.end(),"chaos");
    this->histType             = g.histType; 
    this->deterministicFlag    = g.deterministicFlag;
    this->codeterministicFlag  = g.codeterministicFlag;
    this->ambiguousFlag        = g.ambiguousFlag;
    this->completeFlag         = g.completeFlag;
    this->normalFlag           = g.normalFlag;
    this->setHistOfITGraph(g.getHistOfITGraph());
    this->setNoInitStates();
    this->setNoTermStates();  
}
      
template<class T> 
void ITGraph<T>::copyFast(ITGraph<T>* g)
{
  std::list<State<T> >::iterator iInit,s;
  std::list<Transition<T> >::iterator iTrans;
  
  int numStates   = g->getNoStates();numStates += 2;
  std::vector<std::list<State<T> >::iterator> stateV = 
       vector<std::list<State<T> >::iterator>(numStates);
  int i=0;
  for(iInit=g->listOfStates.begin();iInit!=g->listOfStates.end();++iInit)
    {
      if(i>1) iInit->setIdOfState(i);
      ++i;
    }
  
  i=2; s = this->listOfStates.begin();     
  stateV[0] = s;
  ++s;
  stateV[1] = s;
  iInit = g->listOfStates.begin();   
  ++iInit;++iInit;
 
  for(;iInit!=g->listOfStates.end();++iInit)
    {
       stateV[i++] = this->addState(iInit->getName(),iInit->getHistory(),true);
    }

  for(iTrans=g->listOfTrans.begin();iTrans!=g->listOfTrans.end();++iTrans)
    {
       int org = iTrans->getOrgPtr()->getIdOfState();
       int dst = iTrans->getDstPtr()->getIdOfState();
       if(dst==-1) dst = 1;
       this->addTransition(stateV[org],stateV[dst],iTrans->getLabel());
    }

  this->histOfITGraph.insert(this->histOfITGraph.end(),"chaos");
  this->histType             = g->histType; 
  this->deterministicFlag    = g->deterministicFlag;
  this->codeterministicFlag  = g->codeterministicFlag;
  this->ambiguousFlag        = g->ambiguousFlag;
  this->completeFlag         = g->completeFlag;
  this->normalFlag           = g->normalFlag;
  this->setHistOfITGraph(g->getHistOfITGraph());
  this->setNoInitStates();
  this->setNoTermStates();  
}

template<class T> 
ITGraph<T>* ITGraph<T>::reverse(string name)
{
  
  ITGraph<T>* tmpITGraph = new ITGraph<T>(this->name, 0); 
  tmpITGraph->reproduct(*this);
  //std::list<State<T> >::iterator iI    = tmpITGraph->listOfStates.begin();
  //cout << "\n(rev)id ini= "<< iI->getIdOfState();
  //cout << "\n(rev)id ter= "<< (++iI)->getIdOfState();

  tmpITGraph->histType = REVERSE_HISTORY;
  tmpITGraph->deterministicFlag    = -1; // don't know if 
  tmpITGraph->codeterministicFlag  = -1;
  tmpITGraph->ambiguousFlag        = -1;
  tmpITGraph->completeFlag         = -1;
  tmpITGraph->normalFlag           = -1;
  tmpITGraph->histOfITGraph.clear();
  tmpITGraph->histOfITGraph.insert(tmpITGraph->histOfITGraph.end(),
                                   this->getName()); // his mother 

  //reverse on transition
  std::list<State<T> >::iterator iInit    = tmpITGraph->listOfStates.begin();
  std::list<State<T> >::iterator iInitTmp = iInit; 
  std::list<State<T> >::iterator iTerm    = ++iInitTmp;
  
  std::list<Transition<T> >::iterator iTrans;  
  
  for(iTrans=tmpITGraph->listOfTrans.begin();
      iTrans!=tmpITGraph->listOfTrans.end();++iTrans) 
    { 
    std::list<State<T> >::iterator ptr  = iTrans->getOrgPtr();

    if(iTrans->getOrgPtr()==iInit) 
      { 
        iTrans->setOrgPtr(iTrans->getDstPtr());
        iTrans->setDstPtr(iTerm);
      
      
      }
    else if(iTrans->getDstPtr() == iTerm)
      {
        iTrans->setDstPtr(ptr);
        iTrans->setOrgPtr(iInit); 
       
      }
    else
      { 
        iTrans->setOrgPtr( iTrans->getDstPtr());
        iTrans->setDstPtr(ptr);
      }
  }
  
  //first, on two states : initial, terminal
  iInit = tmpITGraph->listOfStates.begin();
  iInitTmp = iInit; 
  iTerm = ++iInitTmp;

  std::list<MetaNode<T> > nodeTmp2;
 
  nodeTmp2   =  iInit->getAllExits(); 
  
  iInit->setAllExits( iTerm->getAllEntrances() );
  iTerm->setAllEntrances( nodeTmp2 ); 
 

  //reverse on states 
   std::list<State<T> >::iterator iTmp;
  // skip the initial state and terminal state
  ++iTerm;
  for(iTmp=iTerm;iTmp!=tmpITGraph->listOfStates.end();++iTmp) 
  {
      // swap two list
    std::list<MetaNode<T> > nodesTmp  =  iTmp->getAllEntrances();
    iTmp->setAllEntrances( iTmp->getAllExits() );
    iTmp->setAllExits(nodesTmp);
  }
  tmpITGraph->setNoInitStates();
  tmpITGraph->setNoTermStates();
  //iI    = tmpITGraph->listOfStates.begin();
  //cout << "\n(rev2)id ini= "<< iI->getIdOfState();
  //cout << "\n(rev2)id ter= "<< (++iI)->getIdOfState();

  return  tmpITGraph;
}
     
//the methods of access
template<class T> 
string ITGraph<T>::getName(void) const
{
  return this->name;
}
template<class T> 
void ITGraph<T>::setName(string name) 
{
  this->name = name;
}
template<class T> 
int ITGraph<T>::getId(void) const { return this->id; }

template<class T> 
void ITGraph<T>::setId(int num) {this->id=num;}

template<class T> 
string ITGraph<T>::getInitialName(void) const
{
  std::list<State<T> >::const_iterator i = listOfStates.begin();
  return (i->getName());  
}

template<class T> 
void ITGraph<T>::setInitialName(string name)
{
  std::list<State<T> >::iterator i = listOfStates.begin();
  i->setName(name);
}
template<class T> 
string ITGraph<T>::getTerminalName(void) const
{
  std::list<State<T> >::const_iterator i = listOfStates.begin();
  ++i;
  return (i->getName());  
}

template<class T> 
void ITGraph<T>::setTerminalName(string name)
{
   std::list<State<T> >::iterator i = listOfStates.begin();
   ++i;
   i->setName(name);
}



template<class T> 
int ITGraph<T>::getNoStates(void) const {return this->numberOfStates;}

template<class T> 
void ITGraph<T>::setNoStates(int num){this->numberOfStates=num;}
template<class T> 
int ITGraph<T>::getNoTrans(void){return this->numberOfTransitions;}
template<class T> 
void ITGraph<T>::setNoTrans(int num){this->numberOfTransitions=num;}
template<class T> 
int ITGraph<T>::getNoInitStates(void){return this->numberOfInitStates;}
template<class T> 
void ITGraph<T>::setNoInitStates(void)
{
  int c = 0;
  std::list<State<T> >::iterator i = this->listOfStates.begin();
  std::list<MetaNode<T> > nodeList = i->getAllExits();
  std::list<MetaNode<T> >::iterator k;
  for(k=nodeList.begin();k!=nodeList.end();++k)
      if(k->getTransPtr()->getDstPtr()->getIdOfState()!=-1) ++c;
  this-> numberOfInitStates = c;  
}

template<class T> 
void ITGraph<T>::setNoInitStates(int num){this->numberOfInitStates=num;}
template<class T> 
int ITGraph<T>::getNoTermStates(void){return this->numberOfTermStates;}

template<class T> 
void ITGraph<T>::setNoTermStates(void)
{
  int c = 0;
  std::list<State<T> >::iterator i = this->listOfStates.begin();
  ++i;
  std::list<MetaNode<T> > nodeList = i->getAllEntrances();
  std::list<MetaNode<T> >::iterator k; 
  for(k=nodeList.begin();k!=nodeList.end();++k)
      if(k->getTransPtr()->getOrgPtr()->getIdOfState()!=0) ++c;
  this-> numberOfTermStates = c;  
}

template<class T> 
void ITGraph<T>::setNoTermStates(int num){this->numberOfInitStates=num;}
 
template<class T> 
bool ITGraph<T>::setStateHist(string name,
                              const ListOfSymbols& his)
{
  std::list<State<T> >::iterator i;
  for(i=this->listOfStates.begin();i!=this->listOfStates.end();++i)
    {
      if (i->getName() == name) 
      { 
          
         i->setHistory(ListOfSymbols(his) );
         return true;
      }
    }
  return false;
}

template<class T> 
ListOfSymbols ITGraph<T>::getStateHist(string name)
{
  std::list<State<T> >::iterator i;
  for(i=this->listOfStates.begin();i!=this->listOfStates.end();++i)
    {
      if (i->getName()==name) 
      {  
         ListOfSymbols l = ListOfSymbols(this->histOfITGraph); 
         return l;
      }
    }           
  return NULL;
}
		
template<class T> 
int ITGraph<T>::getHistType(void) const
{
  return this->histType;     
}
template<class T> 
void ITGraph<T>::setHistType(int type)
{
  this->histType =  type;
}

template<class T> 
ListOfSymbols ITGraph<T>::getHistOfITGraph(void) const
{
  ListOfSymbols tmp = ListOfSymbols(this->histOfITGraph);
  return tmp; 
}
template<class T> 
void ITGraph<T>::setHistOfITGraph(ListOfSymbols his)
{
  this->histOfITGraph = his;
}
template<class T> 
int ITGraph<T>::getDetermFlag(void) const
{ return this->deterministicFlag;}

template<class T> 
void ITGraph<T>::setDetermFlag(int i)
{ this->deterministicFlag = i;}

template<class T> 
int ITGraph<T>::getCodetermFlag(void) const
{return this->codeterministicFlag;}

template<class T> 
void ITGraph<T>::setCodetermFlag(int i)
{this->codeterministicFlag = i;}

template<class T> 
void ITGraph<T>::setAmbiguousFlag(int i)
{this->ambiguousFlag = i;}

template<class T> 
int ITGraph<T>::getAmbiguousFlag(void) const
{return this->ambiguousFlag;}

template<class T> 
int ITGraph<T>::getCompleteFlag(void) const
{return this->completeFlag;}
template<class T> 
void ITGraph<T>::setCompleteFlag(int flag)
{this->completeFlag = flag;}

template<class T> 
int ITGraph<T>::getNormalFlag(void) const
{return this->normalFlag;}
template<class T> 
void ITGraph<T>::setNormalFlag(int flag)
{this->normalFlag = flag;}

template<class T> 
std::list<std::list<State<T> >::iterator> ITGraph<T>::getIterListOfStates(void)
{
  std::list<std::list<State<T> >::iterator> l;
  std::list<State<T> >::iterator i = listOfStates.begin();
  for(;i!=listOfStates.end();++i) l.insert(l.end(),i);
  return l;
}


template<class T> 
std::list<std::list<Transition<T> >::iterator>
ITGraph<T>::getIterListOfTrans(void)
{
  std::list<std::list<Transition<T> >::iterator> l;
  std::list<Transition<T> >::iterator i = listOfTrans.begin();
  for(;i!=listOfTrans.end();++i) l.insert(l.end(),i);
  return l;
}


template<class T> 
ListOfSymbols ITGraph<T>::getTermNames(void) const
{
  ListOfSymbols l;
  std::list<State<T> >::const_iterator i = listOfStates.begin();
  ++i;
  std::list<MetaNode<T> > lNodes = i->getAllEntrances();
  std::list<MetaNode<T> >::const_iterator k;
  for(k=lNodes.begin();k!=lNodes.end();++k)
     l.insert(l.end(), k->getTransPtr()->getOrgPtr()->getName());
  return l;
}

template<class T> 
ListOfSymbols ITGraph<T>::getInitNames(void) const
{
  ListOfSymbols l;
  std::list<State<T> >::const_iterator i = listOfStates.begin();
  std::list<MetaNode<T> > lNodes = i->getAllExits();
  std::list<MetaNode<T> >::const_iterator k;
  for(k=lNodes.begin();k!=lNodes.end();++k)
     l.insert(l.end(), k->getTransPtr()->getDstPtr()->getName());
  return l;
}


template<class T> 
std::list<NameHist> ITGraph<T>::getNameHists(void) const
{
   std::list<NameHist> tmpList;
   std::list<State<T> >::const_iterator i;
   NameHist tmp;
   for(i=listOfStates.begin();i!=listOfStates.end();++i)
     {
        tmp.name  = i->getName();
        tmp.hists = i->getHistory();
        tmpList.insert(tmpList.end(),tmp); 
     } 
   return tmpList;
}

template<class T> 
std::list<OrgDstLabel<T> > ITGraph<T>::getOrgDstLabels(void) const
{
   std::list<OrgDstLabel<T> > tmpList;
   std::list<Transition<T> >::const_iterator i;
  
   for(i=listOfTrans.begin();i!=listOfTrans.end();++i)
     { 
        OrgDstLabel<T> tmp;
        tmp.org   = i->getOrgPtr()->getName();
        tmp.dst   = i->getDstPtr()->getName();
        tmp.lab   = i->getLabel(); 
        tmpList.insert(tmpList.end(),tmp);
     } 
   return tmpList;
}

template<class T> 
std::list<std::list<Transition<T> >::iterator> ITGraph<T>::getTransIters(void)
{
  std::list<std::list<Transition<T> >::iterator> tmp;
  std::list<Transition<T> >::iterator i;
  for(i=listOfTrans.begin();i!=listOfTrans.end();++i)
    {   
       tmp.insert(tmp.end(), i);
    }
  return tmp;
}
template<class T> 
bool ITGraph<T>::isLastState(std::list<State<T> >::iterator i)
{
  if(i==listOfStates.end()) return true;
  else return false;
}

template<class T> 
bool ITGraph<T>::isLastTrans(std::list<Transition<T> >::iterator i)
{
  if(i==listOfTrans.end()) return true;
  else return false;
}


//the methodes of manipulation



template<class T> 
void ITGraph<T>::renameStates(std::vector<string> strs)
{
  int unsigned i=0; 
  std::list<State<T> >::iterator iState;
  std::list<State<T> >::iterator k = listOfStates.begin();;++k;++k;
  for(iState=k;iState!=listOfStates.end();++iState)
    {
      iState->setName(strs[i++]);
      //cout << "i = " << i << flush;
    }  //cout << "OK rename " << i << flush;
}

template<class T> 
int ITGraph<T>::getNoOrder(std::list<State<T> >::iterator i)
{
  if(i->getIdOfState()==-1) return  1;
  if(i->getIdOfState()== 0) return  0;
  int k;
  std::list<State<T> >::iterator iState = this->listOfStates.begin();
  ++iState;++iState;
  for(k=2;k<=this->getNoStates()+1;++k)
    {
      if(i==iState) return k;
      ++iState;
    } 
  return -2;
}

template<class T> 
bool ITGraph<T>::isTerminalState(string name)
{
  std::list<State<T> >:: iterator iState = this->searchStateByName(name);
  if(iState == listOfStates.end()) return false; 
  std::list<MetaNode<T> > listNodes = iState->getAllExits();
  std::list<MetaNode<T> >::iterator iNode;
  for(iNode=listNodes.begin();iNode!=listNodes.end();iNode++)
  if(iNode->getTransPtr()->getDstPtr()->getIdOfState()==-1) return true;
  return false;
}

template<class T> 
bool ITGraph<T>::isInitialState(string name)
{
  std::list<State<T> >:: iterator iState = this->searchStateByName(name);
  if(iState == listOfStates.end()) return false; 
  std::list<MetaNode<T> > listNodes = iState->getAllEntrances();
  std::list<MetaNode<T> >::iterator iNode;
  for(iNode=listNodes.begin();iNode!=listNodes.end();iNode++)
  if(iNode->getTransPtr()->getOrgPtr()->getIdOfState()==0) return true;
  return false;
}

template<class T> 
bool ITGraph<T>::setInitialState(string name,char em=EMPTY_WORD)
{
  std::list<State<T> >::iterator i = searchStateByName(name);
  if(i==listOfStates.end()) return false; 
  T t1 = T(em);
  if(addTransition(getInitialName(), name, t1)) 
     {++numberOfInitStates;return true;}
  else return false;
}

template<class T> 
bool ITGraph<T>::setTerminalState(string name,char em=EMPTY_WORD)
{  
  std::list<State<T> >::iterator i = searchStateByName(name);
  if(i==listOfStates.end()) return false; 

  T t1 = T(em);
  if(addTransition(name, getTerminalName(), t1)) 
    {   
      ++numberOfTermStates;
      return true;
    }
  else return false;
}

template<class T> 
bool ITGraph<T>::addState(string name)
{
  if(this->searchStateByName(name) != listOfStates.end()) return false; 
    // if the state exist
  else
    {
      ListOfSymbols tmp1;
      tmp1.insert(tmp1.end(),"chaos");
      State<T> tmp = State<T>::State(this->maxIdState, name,tmp1,false,false);
      ++this->maxIdState; 
      this->listOfStates.insert(listOfStates.end(), tmp); 
      ++numberOfStates;
      return true;
    }
}

template<class T> 
bool ITGraph<T>::addState(string name, ListOfSymbols his)
{
   if(this->searchStateByName(name) != listOfStates.end()) return false; 
    // if the state exist
  else
    {
     
      State<T> tmp = State<T>::State(this->maxIdState, name,his,false,false);
      ++this->maxIdState; 
      this->listOfStates.insert(listOfStates.end(), tmp); 
      ++numberOfStates;
      this->setNoInitStates();
      this->setNoTermStates();
      return true;
    }  
}

template<class T> 
std::list<State<T> >::iterator 
ITGraph<T>::addState(string name, ListOfSymbols his, bool flag)
{
   if(this->searchStateByName(name) != listOfStates.end()) 
      return listOfStates.end(); 
    // if the state exist
  else
    {
     
      State<T> tmp = State<T>::State(this->maxIdState, name,his,false,false);
      ++this->maxIdState; 
      this->listOfStates.insert(listOfStates.end(), tmp); 
      ++numberOfStates;
      this->setNoInitStates();
      this->setNoTermStates();
      return (--listOfStates.end());
    }  
}
template<class T> 
bool  ITGraph<T>::addState(string name, int id, ListOfSymbols his)
{
  if(this->searchStateByName(name) != listOfStates.end()) return false; 
    // if the state exist
  else
    {
     
      State<T> tmp = State<T>::State(this->maxIdState, name,his,false,false);
      ++this->maxIdState; 
      this->listOfStates.insert(listOfStates.end(), tmp); 
      ++numberOfStates;
      tmp.setIdOfState(id);
      this->setNoInitStates();
      this->setNoTermStates();
      return true;
    }  
}

template<class T> 
bool ITGraph<T>::addState(State<T>* s )
{
  string name = s->getName();
  if(this->searchStateByName(name) != listOfStates.end()) return false; 
    // if the state exist
  else
    {
      ++this->maxIdState; 
      this->listOfStates.insert(listOfStates.end(), *s); 
      ++numberOfStates;
      this->setNoInitStates();
      this->setNoTermStates();
      return true;
    }
}

template<class T> 
bool ITGraph<T>::deleteState(string name)
{
  std::list<State<T> >::iterator s = this->searchStateByName(name);
  if(s==listOfStates.end()) return false;//if the state is not exist
  this->deleteState(s);

}

template<class T> 
bool ITGraph<T>::deleteState(std::list<State<T> >::iterator s)
{ //cout << "\n enter delete.." << flush;
  if(s==listOfStates.end()) return false;
  else
  {
    std::list<MetaNode<T> >::iterator i;
    std::list<MetaNode<T> > tmp = s->getAllEntrances();
    std::list<Transition<T> >::iterator k;
    for(i=tmp.begin();i!=tmp.end();++i)
      {
       k = i->getTransPtr(); 
       k->getOrgPtr()->deleteExit(k);
       this->deleteTransition(k);
       //cout << " deleting entrance..." << flush;
      }

    tmp = s->getAllExits();
    for(i=tmp.begin();i!=tmp.end();++i)
      { 
       k = i->getTransPtr(); 
       k->getDstPtr()->deleteEntrance(k);
       this->deleteTransition(k);
       
       //cout << " deleting exit..." << flush;
      }
    //entrances.clear();
    //exits.clear();
    if(s->getIdOfState()!=0&&s->getIdOfState()!=-1)
       {
	 //cout << s->getName() << flush;
        this->listOfStates.erase(s);
        --numberOfStates;
        //cout << " deleted.." << flush;
       }
    this->setNoInitStates();
    this->setNoTermStates();
    return true; 
  }
}

template<class T> 
bool ITGraph<T>::addTransition(string nameOrg, string nameDst,
                            const T& label)
{ 
  
  std::list<State<T> >::iterator iii=listOfStates.begin();
  std::list<State<T> >::iterator s1 = this->searchStateByName(nameOrg);
  if(s1==listOfStates.end()) return false;//if the state is not exist

  std::list<State<T> >::iterator s2 = this->searchStateByName(nameDst); 
  if(s2==listOfStates.end()) return false;//if the state is not exist 
  //cout <<"\nin add by name: "<< label << endl << flush;
  this->addTransition(s1, s2,label);
  return true;
}

template<class T> 
bool ITGraph<T>::addTransition(std::list<State<T> >::iterator orgPtr, 
                    std::list<State<T> >::iterator dstPtr, const T& label)
{
  if(orgPtr==listOfStates.end()||dstPtr==listOfStates.end())
    { 
      return false;
    }
  // if transition exist
  if(searchTransition(orgPtr,dstPtr, label)!=listOfTrans.end()) 
     return false;
  Transition<T> tmp = Transition<T>::Transition(this->maxIdTransition, orgPtr,
                                                dstPtr, label);
  
  ++(this->maxIdTransition);
  this->listOfTrans.insert( this->listOfTrans.end(), tmp);
  ++numberOfTransitions;
  std::list<Transition<T> >::iterator iTmp = this->listOfTrans.end();
  --iTmp; 
  
  std::list<MetaNode<T> > nn = orgPtr->getAllExits(); 
  std::list<MetaNode<T> >::iterator kk;

  orgPtr->addExit(iTmp);
  dstPtr->addEntrance(iTmp);
  //T jj = T(tmp.getLabel());
  //cout <<"\nAdd trans: "<<orgPtr->getName()<<","<<
  //  dstPtr->getName()<<"label =" <<jj << endl << flush;
  
  return true;
}

template<class T> 
bool ITGraph<T>::addTransition(std::list<State<T> >::iterator orgPtr, 
                 std::list<State<T> >::iterator dstPtr, const T& label, bool f)
{
 
  Transition<T> tmp = Transition<T>::Transition(this->maxIdTransition, 
                      orgPtr, dstPtr, label);
  
  ++(this->maxIdTransition);
  this->listOfTrans.insert( this->listOfTrans.end(), tmp);
  ++numberOfTransitions;
  std::list<Transition<T> >::iterator iTmp = this->listOfTrans.end();
  --iTmp; 
  
  std::list<MetaNode<T> > nn = orgPtr->getAllExits(); 
  std::list<MetaNode<T> >::iterator kk;

  orgPtr->addExit(iTmp);
  dstPtr->addEntrance(iTmp);
  
  return true;
}

template<class T> 
bool ITGraph<T>::addTransition(std::list<State<T> >::iterator orgPtr, 
                 std::list<State<T> >::iterator dstPtr, 
                 NodeTree* tree, string empty, bool f)
{
 
  
  Transition<T> tmp = Transition<T>::Transition(this->maxIdTransition, 
                      orgPtr, dstPtr, tree, empty);
  
  ++(this->maxIdTransition);
  this->listOfTrans.insert( this->listOfTrans.end(), tmp);
  ++numberOfTransitions;
  std::list<Transition<T> >::iterator iTmp = this->listOfTrans.end();
  --iTmp; 
  
  std::list<MetaNode<T> > nn = orgPtr->getAllExits(); 
  std::list<MetaNode<T> >::iterator kk;

  orgPtr->addExit(iTmp);
  dstPtr->addEntrance(iTmp);
  
  return true;
}

template<class T> 
bool ITGraph<T>::deleteTransition(std::list<Transition<T> >::iterator i)
{
 if(i==listOfTrans.end()) return false;
 i->getOrgPtr()->deleteExit(i);
 i->getDstPtr()->deleteEntrance(i);
 this->listOfTrans.erase(i);
 --numberOfTransitions;
 //T jj = T(i->getLabel());
 //cout <<"\ndelete trans: "<< jj << endl << flush;
 return true;
}

template<class T> 
bool ITGraph<T>::deleteTransition(string nameOrg,string nameDst,
                               const T& label)
{
  std::list<State<T> >::iterator s1 = this->searchStateByName(nameOrg);
  if(s1==listOfStates.end()) return false;//if the state is not exist
  std::list<State<T> >::iterator s2 = this->searchStateByName(nameDst);
  if(s2==listOfStates.end()) return false;//if the state is not exist 
  this->deleteTransition(s1, s2,label);
  return true;
}

template<class T> 
bool ITGraph<T>::deleteTransition(std::list<State<T> >::iterator orgPtr,
                                  std::list<State<T> >::iterator dstPtr, const T& label)
{
  std::list<Transition<T> >::iterator t;
  if(orgPtr==listOfStates.end()||dstPtr==listOfStates.end()) return false;

  t = searchTransition(orgPtr, dstPtr, label);
  if(t==listOfTrans.end()) return false;
  orgPtr->deleteExit(t);
  dstPtr->deleteEntrance(t);
  this->listOfTrans.erase(t);
  --numberOfTransitions;
  return true;
}

template<class T> 
bool ITGraph<T>::deleteTransition(std::list<State<T> >::iterator orgPtr,
                                  std::list<State<T> >::iterator dstPtr)
{
  
  if(orgPtr==listOfStates.end()||dstPtr==listOfStates.end()) return false;

  std::list<MetaNode<T> > exits = orgPtr->getAllExits();
  std::list<MetaNode<T> >::iterator iE;

  while(true)
    {
      exits = orgPtr->getAllExits();
      for(iE=exits.begin();iE!=exits.end();++iE)
      if(iE->getTransPtr()->getDstPtr()==dstPtr)
	{  
          orgPtr->deleteExit(iE->getTransPtr());
          dstPtr->deleteEntrance(iE->getTransPtr());
          this->listOfTrans.erase(iE->getTransPtr());
          --numberOfTransitions;
          break;
	} 
      if(iE==exits.end()) break;  
    }

  return true;
}

// new
// search a transition with label
template<class T> 
bool ITGraph<T>::searchTransitionByLabel(
             std::list<State<T> >::iterator i1, 
             std::list<State<T> >::iterator i2,const T& lab)
{
  if(i1==listOfStates.end()||i2==listOfStates.end()) return false;
  std::list<MetaNode<T> > listNodes = i1->getAllExits();
  std::list<MetaNode<T> >::iterator iNode;
  for(iNode=listNodes.begin();iNode!=listNodes.end();++iNode)
     if(iNode->getTransPtr()->getDstPtr()==i2&&iNode->getTransPtr()->getLabel()==lab)
       return true;
  return false; 
  
}

    
// search a transition
template<class T> 
std::list<Transition<T> >::iterator ITGraph<T>::
searchTransition(std::list<State<T> >::iterator org, std::list<State<T> >::iterator dst, 
                  const T& lab)
{
  std::list<Transition<T> >::iterator i;
  if(org==listOfStates.end()||dst==listOfStates.end()) return listOfTrans.end();

  for(i=this->listOfTrans.begin();i!=this->listOfTrans.end();++i)
  {
    if(i->getLabel() == lab) 
      if(i->getOrgPtr()==org&&i->getDstPtr()==dst) return i;	  
  }
  return  listOfTrans.end();
}

//search a transition by pointeur 'org' and 'dst'  
template<class T> 
std::list<T> ITGraph<T>::searchTransByOrgDst(
             std::list<State<T> >::iterator org, 
             std::list<State<T> >::iterator dst)
{
 std::list<T> tmp;
 std::list<Transition<T> >::iterator i;
 for(i=listOfTrans.begin();i!=listOfTrans.end();++i)
    if(i->getOrgPtr()==org&&i->getDstPtr()==dst)
      {
        T t = T(i->getLabel()); 
        tmp.insert(tmp.end(),t);
      }
 return tmp; 
}
    
//search a state by name
template<class T> 
std::list<State<T> >::iterator ITGraph<T>::searchStateByName(string name)
{
  std::list<State<T> >::iterator i;
  
  for(i=this->listOfStates.begin();i!=this->listOfStates.end();i++)
     if(i->getName()==name) return i;    
               
  return i;
}
template<class T> 
bool ITGraph<T>::isStateByName(string name)
{
  if(searchStateByName(name)==listOfStates.end()) 
    return false;
  else return true;
}

// search a state by history

template<class T> 
std::list<State<T> >::iterator ITGraph<T>::searchStateByHistory(const ListOfSymbols& list)
{
  std::list<State<T> >::iterator i;
  for(i=this->listOfStates.begin();i!=this->listOfStates.end();i++)
     if(i->getHistory()==list)   return i;             
  return i;
}

// find all of successors of a state with a label given
template<class T> 
std::list<State<T> > ITGraph<T>:: successors(string name, const T& lab)
{
  std::list<State<T> > tmp;
  std::list<State<T> >::iterator s = searchStateByName(name);
  if (s==this->listOfStates.end()) return s; 

  std::list<MetaNode<T> >::iterator i;
  for(i=s->getAllExits().begin();i!=s->getAllExits().end();++i)
     if(i->getTransPtr()->getLabel()==lab) 
       {
         State<T> sTmp = State<T>::State( *(i->getTransPtr()->getDstPtr()) ); 
         tmp.insert(tmp.end(), sTmp);
       }
  return tmp; 
}

// find all of predcessors of a state with a label given
template<class T> 
std::list<State<T> > ITGraph<T>::predcessors(string name, const T& lab)
{
  std::list<State<T> > tmp;
  std::list<State<T> >::iterator s = searchStateByName(name);
  if (s==this->listOfStates.end()) return s; 
  std::list<MetaNode<T> >::iterator i;
  for(i=s->getAllEntrances().begin();i!=s->getAllEntrances().end();++i)
     if(i->getTransPtr()->getLabel()==lab) 
       {
         State<T> sTmp = State<T>::State(*(i->getTransPtr()->getOrgPtr())); 
         tmp.insert(tmp.end(), sTmp);
       }
  return tmp; 
}

template<class T>
ITGraph<T>* ITGraph<T>::unionG(const ITGraph<T>& g, string nameT)
{ 
   ITGraph<T>* tmp = new ITGraph<T>(this->getName(),0);
   tmp->reproduct(*this);
   tmp->setName(nameT);
   tmp->setTerminalName("t_"+nameT);
   tmp->setInitialName("i_"+nameT);

   //cout<<"\nafter repro " <<endl <<flush;
   std::map<string,int> reachedNames;
   std::list<State<T> >::iterator i1;
   int count =1;
   i1=tmp->listOfStates.begin();
   ++i1;++i1;
   for(;i1!=tmp->listOfStates.end();++i1)
     { //cout<<"\nadd name= "<<i1->getName()<<endl;
        ListOfSymbols tmp;
        tmp.insert(tmp.end(), i1->getName());
        i1->setHistory(tmp); 
        i1->setName(intToString(count));count++;
      }
   //cout<<"\nbefore state, count = " <<count<<endl <<flush;
  
   std::list<State<T> >::const_iterator k=g.listOfStates.begin();
   ++k;++k;
   std::list<State<T> >::const_iterator iS;
   for(iS=k;iS!=g.listOfStates.end();++iS)
     { //cout<<"\nadd: "<<iS->getName()<<endl;
        ListOfSymbols tmp1;
        tmp1.insert(tmp1.end(), iS->getName()); 
        reachedNames[iS->getName()] = count++;
        tmp->addState(intToString(reachedNames[iS->getName()]),tmp1); 
     } 

   //cout<<"\nafter state, count = " <<count<<endl <<flush;

   std::list<State<T> >::const_iterator b;
   k = tmp->listOfStates.begin();
   b = k;
   ++k;
   std::list<Transition<T> >::const_iterator iT; 
   //cout<<"\nbefore trans " <<endl <<flush;
   for(iT=g.listOfTrans.begin();iT!=g.listOfTrans.end();++iT)
     {   //cout<<"\nadding " <<endl <<flush;
      if(iT->getOrgPtr()->getIdOfState()==0)
      tmp->addTransition(b->getName(),
      intToString(reachedNames[iT->getDstPtr()->getName()]),iT->getLabel());

      else if(iT->getDstPtr()->getIdOfState()==-1)
      tmp->addTransition(intToString(reachedNames[iT->getOrgPtr()->getName()]),
                  k->getName(), iT->getLabel());
      else
      tmp->addTransition(intToString(reachedNames[iT->getOrgPtr()->getName()]),
      intToString(reachedNames[iT->getDstPtr()->getName()]), iT->getLabel());
     }

   tmp->setHistType(UNION_HISTORY);
   ListOfSymbols sList;
   sList.insert(sList.end(), this->getName());
   sList.insert(sList.end(), g.getName());
   tmp->setHistOfITGraph(sList);
   tmp->setNoInitStates();
   tmp->setNoTermStates();
   return tmp;
}

template<class T>
ITGraph<T>* ITGraph<T>::concatenateG(const ITGraph<T>& g, string name)
{
   ITGraph<T>* tmp = new ITGraph<T>(this->getName());
   tmp->reproduct(*this);
   tmp->setName(name);
   tmp->setTerminalName("t_"+name);
   tmp->setInitialName("i_"+name);

   std::map<string,int> reachedNames;
   std::list<State<T> >::iterator i1;
   int count = 1;
   i1=tmp->listOfStates.begin();
   ++i1;++i1;
   for(;i1!=tmp->listOfStates.end();++i1)
     { 
        ListOfSymbols tmp;
        tmp.insert(tmp.end(), i1->getName());
        i1->setHistory(tmp); 
        i1->setName(intToString(count));count++;
      }
  
   ListOfSymbols termList = tmp->getTermNames();
   ListOfSymbols initList = g.getInitNames();


   //delete all of terminal transitions  of "tmp"
   // not general yet if label of transitions is not null ?! 
   std::list<State<T> >::iterator kk=tmp->listOfStates.begin();
   ++kk;
   std::list<MetaNode<T> > enList = kk->getAllEntrances();
   std::list<MetaNode<T> >::iterator iN;
   for(iN=enList.begin();iN!=enList.end();++iN)
     {
       iN->getTransPtr()->getOrgPtr()->deleteExit(iN->getTransPtr());
       iN->getTransPtr()->getDstPtr()->deleteEntrance(iN->getTransPtr());
       tmp->deleteTransition(iN->getTransPtr());
       //cout<<"delete ter "<<endl;
     }

   //add all of states of "g"
   std::list<State<T> >::const_iterator iS;  
   std::list<State<T> >::const_iterator k=g.listOfStates.begin();
   ++k;++k;
   for(iS=k;iS!=g.listOfStates.end();++iS)
     { //cout<<"\nadd: "<<iS->getName()<<endl;
        ListOfSymbols tmp1;
        tmp1.insert(tmp1.end(), iS->getName()); 
        reachedNames[iS->getName()] = count++;
        tmp->addState(intToString(reachedNames[iS->getName()]),tmp1); 
     } 

   //add transitions null
   ListOfSymbols::iterator iL1,iL2;
   T dummy = T();
   for(iL1=termList.begin();iL1!=termList.end();++iL1)
   for(iL2=initList.begin();iL2!=initList.end();++iL2)
      tmp->addTransition(*iL1,intToString(reachedNames[*iL2]),dummy);
     

   //add transitions of g unless initial transitions
  
   std::list<State<T> >::const_iterator b;
   k = tmp->listOfStates.begin();
   b = k;
   ++k;
   std::list<Transition<T> >::const_iterator iT; 
   //cout<<"\nbefore trans " <<endl <<flush;
   for(iT=g.listOfTrans.begin();iT!=g.listOfTrans.end();++iT)
     {
      if(iT->getOrgPtr()->getIdOfState()!=0&&
         iT->getDstPtr()->getIdOfState()!=-1)
      tmp->addTransition(intToString(reachedNames[iT->getOrgPtr()->getName()]),
      intToString(reachedNames[iT->getDstPtr()->getName()]),iT->getLabel());
      else
      if(iT->getDstPtr()->getIdOfState()==-1)
      tmp->addTransition(intToString(reachedNames[iT->getOrgPtr()->getName()]),
      k->getName(),iT->getLabel());
     }

   tmp->setHistType(CONCATENATION_HISTORY);
   ListOfSymbols sList;
   sList.insert(sList.end(), this->getName());
   sList.insert(sList.end(), g.getName());
   tmp->setHistOfITGraph(sList);
   tmp->setNoInitStates();
   tmp->setNoTermStates();
   return tmp;  
}	

template<class T>
string ITGraph<T>::intToString(long i)
{
  string result="";
  char media;
  
  if(i==0) 
  { 
    media =  48;
    result = result+media;
    return result;
  }
 
  while(i!=0) 
  {
    media = (i % 10) + '0';
    string tmp = "";
    tmp=tmp+media;
    result = tmp + result;
    i = i / 10;
  }
  return result;
}


template<class T> 
void ITGraph<T>::infos(void)
{  
  std::list<State<T> >::iterator iState,kState, tState;
  std::list<MetaNode<T> >::iterator iMetaNode,i;
  std::list<Transition<T> >::iterator iTrans;
  
  cout<<"\n-----INFOS----- " << endl; 
  cout<<"\nName: " << this->getName() << endl; 
  cout<<"History Type: ";
  switch(this->getHistType())
    { case DETERMINISTIC_HISTORY: 
      { 
           cout<< "DETERMINISTIC_HISTORY"<<endl;
           break;
      }
      case CODETERMINISTIC_HISTORY:  
      {       
           cout<< "CODETERMINISTIC_HISTORY"<<endl;
           break;
      }
      case PRODUCT_HISTORY:
      {       
           cout<< "PRODUCT_HISTORY"<<endl;
           break;
      } 
      case UNION_HISTORY:
      {       
           cout<< "UNION_HISTORY"<<endl;
           break;
      } 
      case CONCATENATION_HISTORY:
      {       
           cout<< "CONCATENATION_HISTORY"<<endl;
           break;
      } 
      case COMPLEMENT_HISTORY:
      {       
           cout<< "COMPLEMENT_HISTORY"<<endl;
           break;
      } 
      case MINIMAL_HISTORY:
      {       
           cout<< "MINIMAL_HISTORY"<<endl;
           break;
      } 
      case REVERSE_HISTORY:
      {       
           cout<< "REVERSE_HISTORY"<<endl;
           break;
      } 
      case NO_SPONTANEOUS_HISTORY:
      {       
           cout<< "NO_SPONTANEOUS_HISTORY"<<endl;
           break;
      } 
    } 

  cout<<"History of automaton : "; 
  this->displayHisOfAuto(this->getHistOfITGraph()); 
  cout<<"Alphabet : ";
  if(this->who()==AUTOMATA||this->who()==GENERAL_AUTOMATA)
    {
      std::vector<string> alpha = this->getAlphaSet();
      for(int unsigned i=0;alpha.size();++i) cout<<alpha[i]<<" "<<endl;
    }  
  cout<<endl;
  cout << "Number of states: " << this->getNoStates() << endl;
  T result = T();

  kState = this->listOfStates.begin();++kState;++kState;
  for(iState=kState;iState!=this->listOfStates.end(); iState++)
    this->displayHisOfState(iState->getHistory());

  cout << endl;
  int num;  
  num = this->getNoInitStates();
  if(this->getNoStates()==0) num =0;
  cout<<"Number of initial  states: "<< num <<endl;
  if(num>0)
    {
     cout << "Initial  states: " << endl;
     iState = this->listOfStates.begin();
     std::list<MetaNode<T> > listExits = iState->getAllExits();  

     for(iMetaNode=listExits.begin();iMetaNode!=listExits.end(); iMetaNode++)
       {
        if(iMetaNode->getTransPtr()->getDstPtr()->getIdOfState()!=-1)         
        cout<<iMetaNode->getTransPtr()->getDstPtr()->getName()<<" ";
       }
    }

  cout << endl; 

  num = this->getNoTermStates();
  if(this->getNoStates()==0) num =0;
  cout << "Number of terminal states: " << num << endl;
  if(num>0)
    {
      cout << "Terminal  states: " << endl;
      iState = this->listOfStates.begin();
      ++iState; 
      std::list<MetaNode<T> > listEns = iState->getAllEntrances();
      for(iMetaNode=listEns.begin();iMetaNode!=listEns.end(); iMetaNode++)
      { 
         if(iMetaNode->getTransPtr()->getOrgPtr()->getIdOfState()!=0) 
         cout << iMetaNode->getTransPtr()->getOrgPtr()->getName()<<" ";
      }         
    }

  cout << endl; 
  int count = 0;
  num = this->getNoTrans();
  if(this->getNoStates()==0) num =0;
  cout<<"Number of transitions: "<< num <<endl;
  cout << "Transitions: " << endl;
  if(num>0)
  {
     iState = this->listOfStates.begin();
     std::list<MetaNode<T> > listExits = iState->getAllExits();  

     for(i=listExits.begin();i!=listExits.end(); i++)
       {
         string dstName = i->getTransPtr()->getDstPtr()->getName();
         T lab = T(i->getTransPtr()->getLabel()); 
         if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1)
	   {    
             cout<<"--"<<lab<<"->"<<dstName<<endl;++count;
             if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	   }
       }
     ++iState;
     kState = iState;
     ++iState;  
     tState = iState;
     for(;iState!=listOfStates.end();++iState)
       { 
         listExits = iState->getAllExits();  
         for(i=listExits.begin();i!=listExits.end(); ++i)
         {
           string dstName = i->getTransPtr()->getDstPtr()->getName();
           string orgName = i->getTransPtr()->getOrgPtr()->getName();
           if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1&&
              this->isInitialState(iState->getName()))
	     { 
               T lab = T(i->getTransPtr()->getLabel()); 
               cout<<orgName<<"--"<<lab<<"->"<<dstName<<endl;++count;
               if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	     }
         }
       }
     iState = tState; 
     for(;iState!=listOfStates.end();++iState)
       { 
         listExits = iState->getAllExits();  
         for(i=listExits.begin();i!=listExits.end(); ++i)
         {
           string dstName = i->getTransPtr()->getDstPtr()->getName();
           string orgName = i->getTransPtr()->getOrgPtr()->getName();
           if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1&&
              !(this->isInitialState(iState->getName())))
	     { 
               T lab = T(i->getTransPtr()->getLabel()); 
               cout<<orgName<<"--"<<lab<<"->"<<dstName<<endl;++count;
               if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	     }
         }
       }

     iState = kState;;
     listExits = iState->getAllEntrances(); 
      for(i=listExits.begin();i!=listExits.end(); i++)
       {
         string orgName = i->getTransPtr()->getOrgPtr()->getName();
         if(i->getTransPtr()->getOrgPtr()->getIdOfState()==0) orgName = "";
         T lab = T(i->getTransPtr()->getLabel()); 
         cout<<orgName<<"--"<<lab<<"->"<<endl;++count;
	 if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
       }
   
  }//if (num>0)
  
  if(this->getNoStates()==0)
    {
     std::list<MetaNode<T> > l = listOfStates.begin()->getAllExits(); 
     T lab = T(l.begin()->getTransPtr()->getLabel());
     cout<<"Result = "<< lab;
    }
  cout << endl; 
}

template<class T> 
void ITGraph<T>::printChar(char x, int times)
{
  for(int i=0;i<times;++i) cout<<x;
}

template<class T> 
int ITGraph<T>::getNoColWin(long id)
{
  unsigned int i; 
  string tt;
  string tmp1 = "tmp";
  char q = 255;
  tmp1 = tmp1+q; 
  string tmp2 = "xwininfo -id " + intToString(id) +" > " + tmp1;

  int xi = system(tmp2.c_str());
  if(xi==-1) { cout<<"Can't update infos of window1 !"<<endl;return -1;}
  std::ifstream	inAuto(tmp1.c_str());
  if(!inAuto) { cout<<"Can't update infos of window2 !"<<endl;
                return -1;}
  int compte=0;
  while(std::getline(inAuto,tt))  
       {compte++;if(compte>21) break;
       }
  if(compte<=21){ cout<<"Can't update infos of window3 !"<<endl;
                  inAuto.close(); remove(tmp1.c_str());return -1;} 
  i=0;  
  string t9="";
  while(tt[i]==' '&& i<tt.size()) i++;
  while(tt[i]!=' '&& i<tt.size()) { t9=t9+tt[i]; i++;}
  if(t9!="-geometry"){ cout<<"Can't update infos of window4 !"<<endl;
                       inAuto.close(); remove(tmp1.c_str());return -1;}
  t9="";
  while(tt[i]==' '&& i<tt.size()) i++;
  while(tt[i]!=' '&& tt[i]!='x'&&i<tt.size()) { t9=t9+tt[i]; i++;}
  
  inAuto.close();
  remove(tmp1.c_str());
  return (atoi(t9.c_str()));
}

/*********************************|THE END|*********************************/	










