
/**********************************************************************/
/*            The methodes of TransducerLabel                         */
/**********************************************************************/
template<class T>
TransducerLabel<T>::TransducerLabel(bool f) 
    { 
      zero = f;
      input  = "";
      std::vector<string> x = std::vector<string>(1);
      x[0] = "";
      output = T(x);
      inout  = ""; 
    } // pay attention !

template<class T>
TransducerLabel<T>::TransducerLabel(const string t="")
{
   input  = "";
   std::vector<string> x = std::vector<string>(1);
   x[0] = "";
   output = T(x);
   inout  = ""; 
   zero=false;
}


template<class T>
TransducerLabel<T>::TransducerLabel(string t, T u)
{
  input  = t;
  output = u;
  //attention ???
  inout  = t + u.getInfixString();
  zero=false;  
}

template<class T>
TransducerLabel<T>::TransducerLabel(const TransducerLabel<T>& lab)
{ 
  this->input  = lab.input; 
  this->output = lab.output;
  this->inout  = lab.inout;
  this->zero   = lab.zero; 
}

template<class T>
void TransducerLabel<T>::operator = (const TransducerLabel<T>& t)
{ 
  this->input = t.input; 
  this->output= t.output;
  this->inout = t.inout;
  this->zero  = t.zero; 
}

template<class T>
bool TransducerLabel<T>::isEmptyInput(void)
{
     if(input.empty()) return true; else return false;
}

template<class T>
bool TransducerLabel<T>::isEmptyOutput(void)
{
     if(output.empty()) return true; else return false;
}

template<class T>
bool TransducerLabel<T>::operator == (const TransducerLabel<T>& t)
{
        if(this->input==t.input&&this->output==t.output) return true;
        else return false;    
}

template<class T>
bool TransducerLabel<T>::operator < (const TransducerLabel<T>& t) const
{
        if(input < t.input&&output < t.output) return true;
        else return false; 
}

template<class T>
bool TransducerLabel<T>::empty(void)
{
        if(input.empty()&&output.empty()) return true; 
        else return false;
}


//dummy function for determinization of LTOLTransducer 
template<class T>    
string TransducerLabel<T>::getInfixString() const 
{
         string tmp =  "";
         tmp = input+"/"+output.getInfixString();
         return tmp; 
}
//dummy function 
template<class T>  
std::vector<string> TransducerLabel<T>::getInfixSerie() const
{
  return std::vector<string>(0);
}

/***********************************************************************/
/*            The methodes of Transducer                               */
/***********************************************************************/
template<class T>
Transducer<T>::Transducer(string name, int noOfStates, 
              std::vector<string> inAlpha, 
              std::vector<string> outAlpha)
:ITGraph<T>(name,noOfStates)
{
   this->inputAlpha   = inAlpha;  
   this->outputAlpha  = outAlpha; 
}

template<class T>
Transducer<T>::Transducer(const Transducer<T>& t):ITGraph<T>(t.getName(),0)
{
  this->reproduct(t);
  this->inputAlpha   = t.inputAlpha; 
  this->outputAlpha  = t.outputAlpha; 
}

template<class T>
std::vector<string> Transducer<T>::getInputAlpha(void) const
{return inputAlpha;}

template<class T>
std::vector<string> Transducer<T>::getOutputAlpha(void) const
{return outputAlpha;}

template<class T>
void Transducer<T>::setInputAlpha(std::vector<string>& input)
{this->inputAlpha = input; }

template<class T>
void Transducer<T>::setOutputAlpha(std::vector<string>& output)
{this->outputAlpha = output;}

template<class T>
Transducer<T>::Transducer(const ITGraph<T>& itgraph,
	      std::vector<string> inAlpha, 
              std::vector<string> outAlpha):ITGraph<T>(itgraph)
{
  this->setInputAlpha(inAlpha);
  this->setOutputAlpha(outAlpha);
}

template<class T>
Automata<TreeLabel>* Transducer<T>::getDomain(string name)
{
 Automata<TreeLabel>* tmp = new Automata<TreeLabel>(this->getName(),0,
                      this->getInputAlpha());
 std::list<State<T> >::iterator iS = this->listOfStates.begin();
 ++iS;++iS;
 for(;iS!=this->listOfStates.end();++iS) tmp->addState(iS->getName());

 std::list<Transition<T> >::iterator iT = this->listOfTrans.begin();
 for(;iT!=this->listOfTrans.end();++iT)
   {
     std::vector<string> exp;
     Interface<T>::strToExp(iT->getLabel().getInput(),exp); 
     tmp->addTransition(iT->getOrgPtr()->getName(),
          iT->getDstPtr()->getName(),
          TreeLabel(exp,this->getInputAlpha()[0][0]));
   }
 tmp->setName(name);
 tmp->setInitialName("i_"+name);
 tmp->setTerminalName("t_"+name);
 tmp->setNoInitStates();
 tmp->setNoTermStates();
 return tmp;
}

template<class T>
GeneralAuto<TreeLabel>* Transducer<T>::getImage(string name)
{
 GeneralAuto<TreeLabel>* tmp = new GeneralAuto<TreeLabel>(this->getName(),0,
                         this->getOutputAlpha());
 std::list<State<T> >::iterator iS = this->listOfStates.begin();
 ++iS;++iS;
 for(;iS!=this->listOfStates.end();++iS) tmp->addState(iS->getName());

 std::list<Transition<T> >::iterator iT = this->listOfTrans.begin();
 for(;iT!=this->listOfTrans.end();++iT) 
     tmp->addTransition(iT->getOrgPtr()->getName(),
          iT->getDstPtr()->getName(),iT->getLabel().getOutput());
 
 tmp->additionTrans();
 
 tmp->setName(name);
 tmp->setInitialName("i_"+name);
 tmp->setTerminalName("t_"+name);
 tmp->setNoInitStates();
 tmp->setNoTermStates();
 return tmp;
}
 
//very interesting ! 
template<class T>
static std::ostream& operator << (std::ostream& out,
                                  const TransducerLabel<T>& n)
{
  string tmp1 = n.getInput();
  string tmp2 = n.getOutput().getInfixString();
  out << tmp1;out<<" / ";out<<tmp2; 
  return out;
}

template<class T>
bool Transducer<T>::isInputEmptyWord(string& w)
{
  if(w==this->getInputAlpha()[0]) return true;
  else return false;
}

template<class T>
bool Transducer<T>::isOutputEmptyWord(string& w)
{
  if(w==this->getOutputAlpha()[0]) return true;
  else return false;
}

template<class T>
bool Transducer<T>::setOnInitialState(string name,string input,
                                    const TreeLabel& output)
{
  std::list<State<T> >::iterator i = searchStateByName(name);
  if(i==listOfStates.end()) return false;
  T t1 = T(input,output);
  if(addTransition(getInitialName(), name, t1)) 
     {
       int tmp = getNoInitStates();
       setNoInitStates(tmp+1);
       return true;
     }
  else return false;
}

template<class T>
bool Transducer<T>::setOnTerminalState(string name,string input,
                                     const TreeLabel& output)
{
  std::list<State<T> >::iterator i = searchStateByName(name);
  if(i==listOfStates.end()) return false;
  T t1 = T(input,output);
  if(addTransition(name, getTerminalName(), t1)) 
     { int tmp = getNoTermStates();
       setNoTermStates(tmp+1);
       return true;}
  else return false;
} 

template<class T>
bool Transducer<T>::setOffInitialState(void)
{
  std::list<State<T> >::iterator i = listOfStates.begin();
  std::list<MetaNode<T> > l = i->getAllExits();
  std::list<MetaNode<T> >::iterator j = l.begin();
  while(!l.empty()) 
    {
     j = l.begin(); 
     if(!deleteTransition(getInitialName(), 
     j->getTransPtr()->getDstPtr()->getName(),j->getTransPtr()->getLabel() ))
        { 
          //int tmp = getNoInitStates();
          //setNoInitStates(tmp-1);  
          return false;
	}   
     i = listOfStates.begin();
     l = i->getAllExits();
    } 
  int tmp = getNoInitStates();
  setNoInitStates(0);  
  return true;
}

template<class T>
bool Transducer<T>::setOffTerminalState(void)
{
  std::list<State<T> >::iterator i = listOfStates.begin();
  ++i;
  std::list<MetaNode<T> > l = i->getAllEntrances();
  std::list<MetaNode<T> >::iterator j;
  while(!l.empty()) 
    {
     j = l.begin(); 
     if(!deleteTransition(j->getTransPtr()->getOrgPtr()->getName(),
        getTerminalName(),j->getTransPtr()->getLabel() ))
        { 
          //int tmp = getNoTermStates();
          //setNoTermStates(tmp-1);  
          return false;
	}   
     i = listOfStates.begin();
     ++i;
     l = i->getAllEntrances();
    } 
  int tmp = getNoTermStates();
  setNoTermStates(0);  
  return true;
}

template<class T>
bool Transducer<T>::setOffInitialState(string name,string input,
                                       const TreeLabel& output)
{ 
  std::list<State<T> >::iterator i = searchStateByName(name);
  if(i==listOfStates.end()) return false;
  T t1 = T(input,output);
  if(deleteTransition(getInitialName(), name, t1)) 
     {
       int tmp = getNoInitStates();
       setNoInitStates(tmp-1);
       return true;
     }
  else return false;
}

template<class T>
bool Transducer<T>::setOffTerminalState(string name,string input,
                                       const TreeLabel& output)
{
  std::list<State<T> >::iterator i = searchStateByName(name);
  if(i==listOfStates.end()) return false;
  T t1 = T(input,output);
  if(deleteTransition(name, getTerminalName(), t1)) 
     { int tmp = getNoTermStates();
       setNoTermStates(tmp-1);
       return true;}
  else return false;
}

template<class T>
Transducer<T>* Transducer<T>::reverse(string name)
{ 
  ITGraph<T>* graph = new ITGraph<T>(this->getName());
  graph->reproduct(*this);
  ITGraph<T>* tmp = graph->reverse(graph->getName());
  delete (graph);
  Transducer<T> *autoT = new Transducer<T>(*tmp,this->getInputAlpha(),
                         this->getOutputAlpha());
  delete tmp;
  autoT->setName(name);
  autoT->setInitialName("i_"+name);
  autoT->setTerminalName("t_"+name);
  return autoT; 
}

//word don't contain the empty word !!! 
template<class T>
Automata<TreeLabel>* Transducer<T>::autoWord(std::vector<string>& word)
{
  std::list<std::list<State<TreeLabel> >::iterator>::iterator iP,iState;
 
  int unsigned size = word.size();

  if (size==1&&isInputEmptyWord(word[0])) //word null = only one state
  {
    // "word" must have empty word at the first element
    Automata<TreeLabel>* tmp=new Automata<TreeLabel>("word_auto",0,
                                 this->getInputAlpha());
    tmp->addState("1");
    tmp->setInitialState("1",this->getInputAlpha()[0][0]);
    tmp->setTerminalState("1",this->getInputAlpha()[0][0]);
    tmp->setNoInitStates();
    tmp->setNoTermStates();
    return tmp;
  }
  char k = '0'; 

  // "word" must have empty word at the first element
  Automata<TreeLabel>* tmp=new Automata<TreeLabel>("word_auto",
                               0,this->getInputAlpha());
  for(int unsigned  i=1;i<=size+1;++i) tmp->addState(intToString(i));
  
  std::list<std::list<State<TreeLabel> >::iterator> lS = 
            tmp->getIterListOfStates();
  tmp->setInitialState(intToString(1),this->getInputAlpha()[0][0]);
  tmp->setTerminalState(intToString(size+1),this->getInputAlpha()[0][0]);
  iState=lS.begin();
  ++iState;++iState;
  int unsigned  j = 0;
  //skip empty word
  for(j=0;j<size;++j)
    { 
      iP = iState;++iP;
      std::vector<string> exp;
      exp.clear(); 
      exp.insert(exp.end(),word[j]);

      TreeLabel lab = TreeLabel(exp, this->getInputAlpha()[0][0]);
      tmp->addTransition((*iState),(*iP), lab); 
      ++iState;
    }
  tmp->setNoInitStates();
  tmp->setNoTermStates();
  return tmp;
}

template<class T>
Transducer<T>* Transducer<T>::productTA_T(Automata<TreeLabel>* auto1,
                   string name)
{
  int i,j;
  int count = 1;
  string n1,n2;
  std::list<std::list<State<TreeLabel> >::iterator>::iterator jState;
  std::list<State<T> >::iterator iState;
  std::list<QueuePos<T> > queue;
  int noLines   = this->getNoStates();noLines += 2;
  int noColumes = auto1->getNoStates();noColumes += 2;
  std::vector<std::list<State<T> >::iterator> thisVector = 
       vector<std::list<State<T> >::iterator>(noLines);
  i=0;
  for(iState=this->listOfStates.begin();
           iState!=this->listOfStates.end();++iState)
    {
      if(i>1) iState->setIdOfState(i);
      thisVector[i++] = iState;
    }

  std::vector<std::list<State<TreeLabel> >::iterator> auto1Vector = 
       vector<std::list<State<TreeLabel> >::iterator>(noColumes);
  std::list<std::list<State<TreeLabel> >::iterator> lS = 
            auto1->getIterListOfStates(); 
  i=0;
  for(jState=lS.begin();
             jState!=lS.end();++jState)
    {
       if(i>1) (*jState)->setIdOfState(i); 
       auto1Vector[i++] = (*jState);
    }

  Transducer<T>* autoTmp = new Transducer<T>(name, 0,
                     this->getInputAlpha(),this->getOutputAlpha());

  std::list<State<T> >::iterator matrix[noLines][noColumes];
  for(i=0;i<noLines;++i)
    for(j=0;j<noColumes;++j) matrix[i][j] = autoTmp->listOfStates.end();
  
  i=j=0;
  
  QueuePos<T> p = 
    QueuePos<T>(i,j,autoTmp->listOfStates.begin()); 
  queue.insert(queue.end(),p);
  
  matrix[i][j] = autoTmp->listOfStates.begin();
  while(!queue.empty())
    {
       i = queue.begin()->i;
       j = queue.begin()->j;
       std::list<MetaNode<T> >::iterator i1;
       std::list<MetaNode<TreeLabel> >::iterator i2;
       std::list<MetaNode<T> > lnExits = thisVector[i]->getAllExits();
       std::list<MetaNode<TreeLabel> > clExits = auto1Vector[j]->getAllExits();
       for(i1=lnExits.begin();i1!=lnExits.end();++i1)
       for(i2=clExits.begin();i2!=clExits.end();++i2)
	{ 
          T lab = i1->getTransPtr()->getLabel();
          //this comparaison between 2 chars
          //lab.input is string !!!
          string s1,s2;
          s1 = lab.getInput();
          s2 = i2->getTransPtr()->getLabel().getInfixString();
          if(s1==s2)
	    { 
              std::list<State<T> >::iterator org = queue.begin()->iter;
              std::list<State<T> >::iterator dst;
              int u = i1->getTransPtr()->getDstPtr()->getIdOfState();
              int v = i2->getTransPtr()->getDstPtr()->getIdOfState();
               
              if(u==-1&&v==-1)
                {   
                  std::list<State<T> >::iterator k =
		     autoTmp->listOfStates.begin();
                  ++k;  
                  //matrix[1][1] = k; 
                  //dst = matrix[1][1];
                  dst = k;
                }
              else if(u==-1||v==-1) continue;
              else 
                {
	       
                 if(matrix[u][v]== autoTmp->listOfStates.end())
                    
		 { 
                  ListOfSymbols hist;
                 
                  hist.insert(hist.end(),thisVector[u]->getName());
                  hist.insert(hist.end(),auto1Vector[v]->getName());
                  std::list<State<T> >::iterator k = 
                       autoTmp->addState(intToString(count), hist, true);
                  if(k!=autoTmp->listOfStates.end())
		    {
                      count++;
                      matrix[u][v] = k;
                      QueuePos<T> p = QueuePos<T>(u,v,k);
                      queue.insert(queue.end(),p);
                      dst = k;
		    }
                  else    
		    {
                      cout<<"Addition of states is invalid !"<<endl;
                      return NULL; 
		    }
		}  
                else dst = matrix[u][v];
	       }

              autoTmp->addTransition(org,dst,lab,true);
	    }
	}
       queue.erase(queue.begin());
    } 

  autoTmp->setNoInitStates();
  autoTmp->setNoTermStates();
  ListOfSymbols hist;
  hist.insert(hist.end(), this->getName() );
  hist.insert(hist.end(), auto1->getName() );
  autoTmp->setHistOfITGraph(hist);
  autoTmp->setHistType(COMPOSE_TA_T_LTOEXP_HISTORY);
  return autoTmp; 
}

template<class T>
Transducer<T>* Transducer<T>::uniformize(string name)
{

  Automata<TreeLabel>* auto1 = this->getDomain("domain");

  //cout<<"OK uni 1"<<endl<<flush;

  Automata<TreeLabel>* t = auto1->determinize("determ");
  delete(auto1);
 
  //cout<<"OK uni 2"<<endl<<flush;

  int i,j;
  int count = 1;
  string n1,n2;
  std::list<State<T> >::iterator iState;
  std::list<QueuePos<T> > queue;
  int noLines   = this->getNoStates();noLines += 2;
  int noColumes = t->getNoStates();noColumes += 2;
  std::vector<std::list<State<T> >::iterator> thisVector = 
       vector<std::list<State<T> >::iterator>(noLines);
  i=0;
  for(iState=this->listOfStates.begin();
           iState!=this->listOfStates.end();++iState)
    {
      if(i>1) iState->setIdOfState(i);
      thisVector[i++] = iState;
    }

  std::vector<std::list<State<TreeLabel> >::iterator> tVector = 
       vector<std::list<State<TreeLabel> >::iterator>(noColumes);
  
  std::list<std::list<State<TreeLabel> >::iterator> lIters;
  std::list<std::list<State<TreeLabel> >::iterator>::iterator iTers;
  lIters = t->getIterListOfStates();

  i=0;
  for(iTers=lIters.begin();
             iTers!=lIters.end();++iTers)
    {
       if(i>1) (*iTers)->setIdOfState(i); 
       tVector[i++] = (*iTers);
    }

  Transducer<T>* tTmp = new Transducer<T>(name, 0, 
      this->getInputAlpha(),this->getOutputAlpha()); 

  std::list<State<T> >::iterator matrix[noLines][noColumes];
  for(i=0;i<noLines;++i)
  for(j=0;j<noColumes;++j) matrix[i][j] = tTmp->listOfStates.end();
  
  i=j=0;
  
  QueuePos<T> p = 
           QueuePos<T>(i,j,tTmp->listOfStates.begin());
  queue.insert(queue.end(),p);
  
  matrix[i][j] = tTmp->listOfStates.begin();
     
  while(!queue.empty())
    {
       i = queue.begin()->i;
       j = queue.begin()->j;
       std::list<MetaNode<T> >::iterator i1;
       std::list<MetaNode<TreeLabel> >::iterator i2;
       std::list<MetaNode<T> > lnExits = thisVector[i]->getAllExits();
       std::list<MetaNode<TreeLabel> > clExits = tVector[j]->getAllExits();
       for(i1=lnExits.begin();i1!=lnExits.end();++i1)
       for(i2=clExits.begin();i2!=clExits.end();++i2)
	{ 
          T lab1 = i1->getTransPtr()->getLabel();
          TreeLabel lab2 = i2->getTransPtr()->getLabel();

          if(lab1.getInput()==lab2.getInfixString())
	    { 
              std::list<State<T> >::iterator org = queue.begin()->iter;
              std::list<State<T> >::iterator dst;
              int u = i1->getTransPtr()->getDstPtr()->getIdOfState();
              int v = i2->getTransPtr()->getDstPtr()->getIdOfState();
               
              if(u==-1&&v==-1)
                {   
                  std::list<State<T> >::iterator k =
                                        tTmp->listOfStates.begin();
                  ++k;  
                  matrix[1][1] = k; 
                  //dst = matrix[1][1];
                  dst = k;
                }
              else if(u==-1||v==-1) continue;
              else
                {
                   
                  if(matrix[u][v]==tTmp->listOfStates.end()) 
		  { 
                  ListOfSymbols hist;
                 
                  hist.insert(hist.end(),thisVector[u]->getName());
                  hist.insert(hist.end(),tVector[v]->getName());
                  std::list<State<T> >::iterator k = 
                       tTmp->addState(intToString(count), hist, true);
                  if(k!=tTmp->listOfStates.end())
		    {
                      count++;
                      matrix[u][v] = k;
                      QueuePos<T> p = QueuePos<T>(u,v,k);
                      queue.insert(queue.end(),p);
                      dst = k;
                      k->setIdOfState(v); // for uniformization
		    }
                  else    
		    {
                      cout<<"Addition of states is invalid !"<<endl;
                      return NULL; 
		    }
		  } 
                  else dst = matrix[u][v];
		} 

              TreeLabel word = lab1.getOutput().choiceWord();

              T lab = T(lab1.getInput(),word);   
              tTmp->addTransition(org,dst,lab,true);
	    }
	}
       queue.erase(queue.begin());
    } 

  delete(t);
  tTmp->setNoInitStates();
  tTmp->setNoTermStates();
  
  //cout<<"after :"<<endl;
  //tTmp->display(0);

  std::list<MetaNode<T> > lnEntrances;
  std::list<MetaNode<T> >::iterator iLn,jLn,kLn;
  int org1=0,org2=0;
  bool sw=false;
  //include terminal mute state  
  for(i=1;i<noLines;++i) 
  for(j=1;j<noColumes;++j)
    {
      if(matrix[i][j]!=tTmp->listOfStates.end())
	{
           lnEntrances = matrix[i][j]->getAllEntrances();
           if(lnEntrances.empty()) continue;
           iLn=lnEntrances.begin();
           while(iLn!=lnEntrances.end())
	     {
              jLn=iLn;
              ++jLn;
              org1 = iLn->getTransPtr()->getOrgPtr()->getIdOfState();
              while(jLn!=lnEntrances.end())
	      {
               kLn=jLn;
               ++jLn; 
               org2 = kLn->getTransPtr()->getOrgPtr()->getIdOfState();
               
               //if(kLn->getTransPtr()->getLabel().getInput()==
               //   iLn->getTransPtr()->getLabel().getInput())
	       //	 {
	       //cout<<"in="<<iLn->getTransPtr()->getLabel().getInput()<<endl; 
	       // cout<<"co1="<<org1<<", co2="<<org2<<endl;
               //   }
               //else
	       // {
            //cout<<"ini="<<iLn->getTransPtr()->getLabel().getInput()<<endl;               //cout<<"ink="<<kLn->getTransPtr()->getLabel().getInput()<<endl;
	       // cout<<"co1="<<org1<<", co2="<<org2<<endl; 

	       //}
         
               if(kLn->getTransPtr()->getLabel().getInput()==
                  iLn->getTransPtr()->getLabel().getInput()&&
                  org1==org2)
		   {
                     T lab = T(kLn->getTransPtr()->getLabel());
                     sw=tTmp->deleteTransition(kLn->getTransPtr()->getOrgPtr(),
                              matrix[i][j],lab);
                     if(sw) lnEntrances.erase(kLn); 
		   }
           
	      }//while
              ++iLn;  
	     }//while
	}             
    } 

  tTmp->setNoInitStates();
  tTmp->setNoTermStates();
  ListOfSymbols hist;
  hist.insert(hist.end(), this->getName() );
  tTmp->setHistOfITGraph(hist);
  tTmp->setHistType(UNIFORMIZE_HISTORY);
  return tTmp;  
}

template<class T>
void Transducer<T>::display(long id)
{ 
  string choice;
  if(this->getNoStates() > MAX_STATE_DISPLAY||
     this->getNoTrans()  > MAX_TRANSITION_DISPLAY)
    {
      do
      { 
        cout << "Transducer is very big !"<<endl;
        cout <<"Number of states = "<<this->getNoStates()<<endl;
        cout <<"Number of transitions = "<<this->getNoTrans()<<endl;
        cout << "Do you want display (y,n) ?";
        cin >> choice;cin.get();
      }
      while(choice!="n"&&choice!="N"&&choice!="y"&&choice!="Y");
      if(choice=="n"||choice=="N") return;
      this->displayBig(id);
    }
  else this->displaySmall();
}

template<class T> 
void Transducer<T>::displaySmall(void)
{ 
   
  std::list<State<T> >::iterator iState,kState,tState;
  std::list<MetaNode<T> >::iterator iMetaNode,i;
  std::list<Transition<T> >::iterator iTrans;
  cout << "\n-----DISPLAY----- " << endl; 
  cout << "\nName: " << this->getName() << endl;
  cout << "Alphabet of input : ";
  std::vector<string> input = getInputAlpha();
  cout<<"("<<input[0]<<") ";
  for(int unsigned iChar=1;iChar<input.size();++iChar)
    cout<<input[iChar]<<" ";
  cout<<endl;

  cout << "Alphabet of output : ";
  std::vector<string> output = getOutputAlpha();
  cout<<"("<<output[0]<<") "; 
  for(int unsigned jChar=1;jChar<output.size();++jChar)
    cout<<output[jChar]<<" ";
  cout<<endl;

  cout << "Number of states: " << this->getNoStates() << endl;
  //T result = T(x);
  kState = this->listOfStates.begin();++kState;++kState;
  for(iState=kState;iState!=this->listOfStates.end(); iState++)
  
    cout << iState->getName()<<" ";
   

  cout << endl;
  int num;  
  num = this->getNoInitStates();
  if(this->getNoStates()==0) num =0;
  cout<<"Number of initial  states: "<< num <<endl;
  if(num>0)
    {
     cout << "Initial  states: " << endl;
     iState = this->listOfStates.begin();
     std::list<MetaNode<T> > listExits = iState->getAllExits();  

     for(iMetaNode=listExits.begin();iMetaNode!=listExits.end(); iMetaNode++)
       {
        if(iMetaNode->getTransPtr()->getDstPtr()->getIdOfState()!=-1) 
        cout<<iMetaNode->getTransPtr()->getDstPtr()->getName()<<" ";
       }
    }

  cout << endl; 

  num = this->getNoTermStates();
  if(this->getNoStates()==0) num =0;
  cout << "Number of terminal states: " << num << endl;
  if(num>0)
    {
      cout << "Terminal  states: " << endl;
      iState = this->listOfStates.begin();
      ++iState; 
      std::list<MetaNode<T> > listEns = iState->getAllEntrances();
      for(iMetaNode=listEns.begin();iMetaNode!=listEns.end(); iMetaNode++)
	{
         if(iMetaNode->getTransPtr()->getOrgPtr()->getIdOfState()!=0) 
         cout << iMetaNode->getTransPtr()->getOrgPtr()->getName()<<" ";
	}
    }

  cout << endl; 
  int count = 0;
  num = this->getNoTrans();
  if(this->getNoStates()==0) num =0;
  cout<<"Number of transitions: "<< num <<endl;
  cout << "Transitions: " << endl;
  if(num>0)
  {
     iState = this->listOfStates.begin();
     std::list<MetaNode<T> > listExits = iState->getAllExits();  

     for(i=listExits.begin();i!=listExits.end(); i++)
       {
         string dstName = i->getTransPtr()->getDstPtr()->getName();
         T lab = T(i->getTransPtr()->getLabel());
          
         if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1)
	   {    
             int gap = 0;
             if(lab.empty())
               {cout<<"--"<<"-> "<<dstName<<endl;++count;}
             else 
               {cout<<"--"<<lab.getInput()<<" / "<<lab.getOutput()<<
                      "-> "<<dstName<<endl;++count;}
             if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	   }
        
       }
     ++iState;
     kState = iState;
     ++iState;  
     tState = iState; 
     for(;iState!=listOfStates.end();++iState)
       { 
         listExits = iState->getAllExits();  
         for(i=listExits.begin();i!=listExits.end(); ++i)
         {
           string dstName = i->getTransPtr()->getDstPtr()->getName();
           string orgName = i->getTransPtr()->getOrgPtr()->getName();
           if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1&&
              this->isInitialState(iState->getName()))
	     { 
               T lab = T(i->getTransPtr()->getLabel()); 
               if(lab.empty())
                 {cout<<orgName<<" --"<<"-> "<<dstName<<endl;++count;}
               else
               {cout<<orgName<<" --"<<lab.getInput()<<" / ";
               cout<<lab.getOutput();cout<<"-> "<<dstName<<endl;++count;}
               if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	     }
         }
       }
     iState = tState; 
     for(;iState!=listOfStates.end();++iState)
       { 
         listExits = iState->getAllExits();  
         for(i=listExits.begin();i!=listExits.end(); ++i)
         {
           string dstName = i->getTransPtr()->getDstPtr()->getName();
           string orgName = i->getTransPtr()->getOrgPtr()->getName();
           if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1&&
              !(this->isInitialState(iState->getName())))
	     { 
               T lab = T(i->getTransPtr()->getLabel()); 
               if(lab.empty())
		 {cout<<orgName<<" --"<<"-> "<<dstName<<endl;++count;}
               else
		 {cout<<orgName<<" --"<<lab.getInput()<<" / "<<
                  lab.getOutput();cout<<"-> "<<dstName<<endl;++count;
                 }
               if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	     }
         }
       }

     iState = kState;;
     listExits = iState->getAllEntrances(); 
      for(i=listExits.begin();i!=listExits.end(); i++)
       {
         string orgName = i->getTransPtr()->getOrgPtr()->getName();
         if(i->getTransPtr()->getOrgPtr()->getIdOfState()==0) orgName = "";
         T lab = T(i->getTransPtr()->getLabel());
         if(lab.empty())
	    {cout<<orgName<<" --"<<"-> "<<endl;++count;} 
         else
         {cout<<orgName<<" --"<<lab.getInput()<<" / ";
          cout<<lab.getOutput();cout<<"-> "<<endl;++count;}
	 if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
       }
   
  }//if (num>0)
  
  if(this->getNoStates()==0)
    {
     std::list<MetaNode<T> > l = listOfStates.begin()->getAllExits();
     if(l.size()>0)
       {
        T lab = T(l.begin()->getTransPtr()->getLabel());
        cout<<"Result = "<< lab.getInput()<<" / "<<lab.getOutput();
       }
    }
     
  cout << endl; 
}

template<class T> 
void  Transducer<T>::displayBig(long id)
{
  //cout<<"id="<<id<<endl;
  int noCols = getNoColWin(id);
  //cout<<"noCol="<<noCols<<endl;   
  std::list<State<T> >::iterator iState,kState,tState;
  std::list<MetaNode<T> >::iterator iMetaNode,i;
  std::list<Transition<T> >::iterator iTrans;
  cout << "\n-----DISPLAY----- " << endl; 
  cout << "\nName: " << this->getName() << endl;

  cout << "Alphabet of input : ";
  std::vector<string> input = getInputAlpha();
  cout<<"("<<input[0]<<") "; 
  for(int unsigned iChar=1;iChar<input.size();++iChar)
    cout<<input[iChar]<<" ";
  cout<<endl;

  cout << "Alphabet of output : ";
  std::vector<string> output = getOutputAlpha();
  cout<<"("<<output[0]<<") "; 
  for(int unsigned jChar=1;jChar<output.size();++jChar)
    cout<<output[jChar]<<" ";
  cout<<endl;

  cout << "Number of states: " << this->getNoStates() << endl;
  //string x="";
  //T result = T(x);
  kState = this->listOfStates.begin();++kState;++kState;
  int offset=0;
  int count = 0;
  int noStateCol = noCols / STATE_COL_SIZE;
  //cout<<"noStateCol="<<noStateCol<<endl;
  
  for(iState=kState;iState!=this->listOfStates.end(); iState++)
  {     
    cout << iState->getName();
    printChar(' ',STATE_COL_SIZE - iState->getName().size());
    ++offset;
    if(offset>= noStateCol) { offset=0;++count;cout<<endl;}
             if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
  }   
  cout << endl;
  int num;  
  offset=0;
  count = 0;
  num = this->getNoInitStates();
  if(this->getNoStates()==0) num =0;
  cout<<"Number of initial  states: "<< num <<endl;
  if(num>0)
    {
     cout << "Initial  states: " << endl;
     iState = this->listOfStates.begin();
     std::list<MetaNode<T> > listExits = iState->getAllExits();  

     for(iMetaNode=listExits.begin();iMetaNode!=listExits.end(); iMetaNode++)
       {
        if(iMetaNode->getTransPtr()->getDstPtr()->getIdOfState()!=-1) 
        cout<<iMetaNode->getTransPtr()->getDstPtr()->getName();
        printChar(' ',STATE_COL_SIZE -
                  iMetaNode->getTransPtr()->getDstPtr()->getName().size());
        ++offset;
        if(offset>= noStateCol) { offset=0;++count;cout<<endl;}
             if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
       }
    }

  cout << endl; 
  offset=0;
  count = 0;
  num = this->getNoTermStates();
  if(this->getNoStates()==0) num =0;
  cout << "Number of terminal states: " << num << endl;
  if(num>0)
    {
      cout << "Terminal  states: " << endl;
      iState = this->listOfStates.begin();
      ++iState; 
      std::list<MetaNode<T> > listEns = iState->getAllEntrances();
      for(iMetaNode=listEns.begin();iMetaNode!=listEns.end(); iMetaNode++)
	{
         if(iMetaNode->getTransPtr()->getOrgPtr()->getIdOfState()!=0) 
         cout << iMetaNode->getTransPtr()->getOrgPtr()->getName();
         printChar(' ',STATE_COL_SIZE -
                  iMetaNode->getTransPtr()->getOrgPtr()->getName().size());
        ++offset;
        if(offset>= noStateCol) { offset=0;++count;cout<<endl;}
             if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	}
    }

  cout << endl; 
 
  num = this->getNoTrans();
  if(this->getNoStates()==0) num =0;
  cout<<"Number of transitions: "<< num <<endl;
  cout << "Transitions: " << endl;
  if(num>0)
  {
     offset = 0; 
     count  = 0;
     int noTransCol = noCols / TRANS_COL_SIZE;
     //cout<<"noTransCol="<<noTransCol<<endl; 
     iState = this->listOfStates.begin();
     std::list<MetaNode<T> > listExits = iState->getAllExits();  

     for(i=listExits.begin();i!=listExits.end(); i++)
       {
         string dstName = i->getTransPtr()->getDstPtr()->getName();
         T lab = T(i->getTransPtr()->getLabel()); 
         if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1)
	   { 
             int gap=0;
             if(lab.empty())
	     {cout<<" --"<<" ->"<<dstName<<endl;gap=6;++count;}                               else
	     {cout<<" --"<<lab.getInput()<<" / "<<
		      lab.getOutput()<<"-> "<<dstName;gap=9;++count;}
             printChar(' ',TRANS_COL_SIZE-
                       (gap+dstName.size()+lab.size()));
             ++offset;
             if(offset>=noTransCol) { offset=0;++count;cout<<endl;}
             if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	   }
        
       }
     ++iState;
     kState = iState;
     ++iState;  
     tState = iState; 
     for(;iState!=listOfStates.end();++iState)
       { 
         listExits = iState->getAllExits();  
         for(i=listExits.begin();i!=listExits.end(); ++i)
         { 
           string dstName = i->getTransPtr()->getDstPtr()->getName();
           string orgName = i->getTransPtr()->getOrgPtr()->getName();
           if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1&&
              this->isInitialState(iState->getName()))
	     { 
               T lab = T(i->getTransPtr()->getLabel());   
               int gap = 0;
               if(lab.empty())
		 {cout<<orgName<<" --"<<"-> "<<dstName;gap=6;}
               else  
               {cout<<orgName<<" --"<<lab.getInput()<<" / "
                    <<lab.getOutput()<<"-> "<<dstName;gap=9;}

               printChar(' ',TRANS_COL_SIZE-
               (gap+orgName.size()+dstName.size()+lab.size()));
               ++offset;
               if(offset>=noTransCol) { offset=0;++count;cout<<endl;} 
               if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	     }
         }
       }
     iState = tState; 
     for(;iState!=listOfStates.end();++iState)
       { 
         listExits = iState->getAllExits();  
         for(i=listExits.begin();i!=listExits.end(); ++i)
         {
           string dstName = i->getTransPtr()->getDstPtr()->getName();
           string orgName = i->getTransPtr()->getOrgPtr()->getName();
           if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1&&
              !(this->isInitialState(iState->getName())))
	     { 
               int gap =0;
               T lab = T(i->getTransPtr()->getLabel());
               if(lab.empty())
		 {cout<<orgName<<" --"<<"-> "<<dstName;gap=6;}
               else
		 {cout<<orgName<<" --"<<lab.getInput()<<" / "
                      <<lab.getOutput()<<"-> "<<dstName;gap=9;}
               printChar(' ',TRANS_COL_SIZE-
               (gap+orgName.size()+dstName.size()+lab.size()));
               ++offset;
               if(offset>=noTransCol) { offset=0;++count;cout<<endl;} 
               if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	     }
         }
       }

     iState = kState;;
     listExits = iState->getAllEntrances(); 
      for(i=listExits.begin();i!=listExits.end(); i++)
       {
         string orgName = i->getTransPtr()->getOrgPtr()->getName();
         if(i->getTransPtr()->getOrgPtr()->getIdOfState()==0) orgName = "";
         T lab = T(i->getTransPtr()->getLabel()); 
         int gap =0;
         if(lab.empty())
	   {cout<<orgName<<" --"<<"-> ";gap=6;}
         else
	   {cout<<orgName<<" --"<<lab.getInput()<<" / "
                <<lab.getOutput()<<"-> ";gap=9;}

         printChar(' ',TRANS_COL_SIZE-
                       (gap+orgName.size()+lab.size()));
         ++offset;
         if(offset>=noTransCol) { offset=0;++count;cout<<endl;} 
	 if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
       }
   
  }//if (num>0)
  
  if(this->getNoStates()==0)
    {
     std::list<MetaNode<T> > l = listOfStates.begin()->getAllExits();
     if(l.size()>0)
       {
        T lab = T(l.begin()->getTransPtr()->getLabel());
        cout<<"Result = "<< lab.getInput()<<" / "<<lab.getOutput();
       }
    }     
  cout << endl;     
}

template<class T>
Transducer<T>::Transducer(string filename, string name, long pos):
ITGraph<T>(name)
{
  std::ifstream	inAutos(filename.c_str());

  if(!inAutos)
     {
	std::cerr << "Can't open : " << filename << "'\n";
	return;
     }

  inAutos.seekg(pos);
  string separatorF = "";
  char sepa = SEPARATOR_FILE;
  separatorF = separatorF + sepa;
  string separatorA = "";
  sepa = SEPARATOR_AUTO;
  separatorA = separatorA + sepa;
  bool stop = false;
  
  int id; 
  if (!(inAutos>>id ))  
     {std::cerr<<"Read error: id !"<<endl;inAutos.close();return;}
  //skip character end of line
  inAutos.get();
  this->setId(id); 
  //cout <<"iD= "<< id<<endl;

  int deterFlag; 
  if (!(inAutos>>deterFlag))  
	      {std::cerr<<"Read error: deterFlag !"<<endl;return;}
  //skip character end of line
  inAutos.get(); 
  this->setDetermFlag(deterFlag);
  //cout <<"deterflag= "<< deterFlag<<endl;

  int codeterFlag; 
  if (!(inAutos>>codeterFlag))  
	      {std::cerr<<"Read error: CodeterFlag !"<<endl;return;}
  //skip character end of line
  inAutos.get(); 
  this->setCodetermFlag(codeterFlag);   
  //cout <<"codeterflag= "<< codeterFlag<<endl;

  int ambiFlag; 
  if (!(inAutos>>ambiFlag))  
	      {std::cerr<<"Read error: ambiFlag !"<<endl;return;}
  //skip character end of line
  inAutos.get(); 
  this->setAmbiguousFlag(ambiFlag);
  //cout <<"ambiflag= "<< ambiFlag<<endl;

  int comFlag; 
  if (!(inAutos>>comFlag))  
	   {std::cerr<<"Read error:comFlag !"<<endl;return;} 
  //skip character end of line
  inAutos.get(); 
  this->setCompleteFlag(comFlag);
  //cout <<"comflag= "<< comFlag<<endl;

  int norFlag; 
  if (!(inAutos>>norFlag))  
	      {std::cerr<<"Read error: norFlag !"<<endl;return;}
  //skip character end of line
  inAutos.get(); 
  this->setNormalFlag(norFlag);

  //read alphaset
  int unsigned alphaSize; 
  if (!(inAutos>>alphaSize))  
	      {std::cerr<<"Read error:alphaSize 1 !"<<endl;return;} 
  //skip character end of line
  inAutos.get(); 

  std::vector<string> inputAlpha =std::vector<string>(alphaSize);
  for(int unsigned index=0;index<alphaSize;++index)
  if(!std::getline(inAutos, inputAlpha[index])) 
	   {std::cerr<<"Read error: alpha 1!"<<endl;return;}
  this->setInputAlpha(inputAlpha);
  
  if (!(inAutos>>alphaSize))  
	      {std::cerr<<"Read error:alphaSize 2  !"<<endl;return;}
  //skip character end of line
  inAutos.get(); 

  std::vector<string> outputAlpha =std::vector<string>(alphaSize);
  for(int unsigned index=0;index<alphaSize;++index)
  if(!std::getline(inAutos, outputAlpha[index])) 
	   {std::cerr<<"Read error: alpha2 !"<<endl;return;}
  this->setOutputAlpha(outputAlpha);

  int hisType; 
  if (!(inAutos>>hisType))  
	      {std::cerr<<"Read error: hisType !"<<endl;return;}
  //skip character end of line
  inAutos.get(); 

  this->setHistType(hisType); 
  //cout <<"histype= "<< hisType<<endl;

  //read history of automaton
  //terminated by string NULL 
  ListOfSymbols history;
  string hist;
  bool error = false;
  if(!std::getline(inAutos, hist))
	   {std::cerr<<"Read error: history !"<<endl;return;}
  while(hist!="")
	   {
              history.insert(history.end(),hist);
              //cout<<"\nLoop his !"<<endl; 
              if(!std::getline(inAutos, hist)) 
		{error = true;break;} 
	   }
        
   //cout<<"\nhis OK !"<<endl;
 
  if(error) {std::cerr<<"Read error: history !"<<endl;
             inAutos.close();return;}
  this->setHistOfITGraph(history);

  //read states of automaton
  //terminated by id==-2
  int idState;
  error = false;
  string stateName;
  do
	   {
              if(!(inAutos>>idState))  
		{error=true;break;}   
              if(idState==-2) { inAutos.get();break;}
              //skip character end of line
              inAutos.get(); 
              if(!std::getline(inAutos, stateName)) 
		{error=true;break;}   
              bool accFlag;
              if (!(inAutos>>accFlag))  
		{error=true;break;}   
              inAutos.get(); 
              bool coaccFlag;
              if (!(inAutos>>coaccFlag))  
		 {error=true;break;}   
              inAutos.get(); 
              string hist;
            
              if(!std::getline(inAutos, hist))
	        {std::cerr<<"Read error: history !"<<endl;return;}
              history.clear();
              while(hist!="")
	      {
                history.insert(history.end(),hist);
                //cout<<"\nLoop his !"<<endl; 
                if(!std::getline(inAutos, hist)) 
		   {error = true;break;} 
	      }  

              if(error) break;
              State<T>* state = new State<T>(idState,stateName,history,
                                accFlag, coaccFlag);   
              this->addState(state);
              delete(state);  
   }
   while(1); 
   if(error) {std::cerr<<"Read error: state !"<<endl;
              inAutos.close();return;}

   //read transitions of automaton
   //terminated by id==SEPARATOR_AUTO
        
   error = false;
   do
   { 
              string separatorA = "";
              char sepa = SEPARATOR_AUTO;
              separatorA = separatorA + sepa;
              string idTrans;
              if (!(std::getline(inAutos,idTrans)))
		{error=true;break;}
	      if(idTrans==separatorA) {error = false; break; }
              //cout<<"\nidt = "<<idTrans;

              //skip character end of line
              //inAutos.get();
              string org;
              if(!std::getline(inAutos, org)) 
                {error=true;break;}
              //cout<<"\norg = "<<org;
   
              string dst;
              if(!std::getline(inAutos, dst)) 
		{error=true;break;}   
              //cout<<"\ndst = "<<dst;
             
              string lab1; 
              if(!std::getline(inAutos, lab1)) 
		{error=true;break;}
                //cout<<"\nlab = "<<lab;

              int unsigned infixSize;
              if (!(inAutos>>infixSize))  
		 {error=true;break;}   
              inAutos.get(); 

	      std::vector<string> lab2 =std::vector<string>(infixSize);
              for(int unsigned i=0;i<infixSize;++i)
              if(!std::getline(inAutos, lab2[i])) 
		{error=true;break;}
              //cout<<"\nlab = "<<lab;

               T label = T(lab1, TreeLabel(lab2,outputAlpha[0][0]));
               this->addTransition(org,dst,label);
	     
   }while(1);
   if(error) {std::cerr<<"Read error:trans !"<<endl;
              inAutos.close();return;}

  inAutos.close();  
  this->setNoInitStates();
  this->setNoTermStates();
}

template<class T>
bool Transducer<T>::store(string filename)
{
  std::ofstream outAutos(filename.c_str(),ios::out|ios::ate);
  if(!outAutos) 
    {cout<<"Problem of filename or permission !"<<endl;return false;}

  outAutos.seekp(-2,ios::end);// go to SEPARATOR_FILE 
  if (!(outAutos<<this->getName()<<endl)) 
      {   std::cerr << "Write error: s1  !"<<endl;
          outAutos.close();
	  return false; 
      } 

   int flag = this->who();
   if (!(outAutos<<flag<<endl))  
      {   std::cerr << "Write error: s21  !"<<endl; 
          outAutos.close();return false; }

   if (!(outAutos<<this->getId()<<endl))  
      {   std::cerr << "Write error: s2  !"<<endl; 
          outAutos.close();return false; }
      
  
   if (!(outAutos<<this->getDetermFlag()<<endl))  
      {   std::cerr << "Write error: s3  !"<<endl; 
          outAutos.close();return false; }        
 
   if (!(outAutos<<this->getCodetermFlag()<<endl))  
      {   std::cerr << "Write error :s4  !"<<endl; 
          outAutos.close();return false; }
         
   if (!(outAutos<<this->getAmbiguousFlag()<<endl))  
      {   std::cerr << "Write error: s5   !"<<endl; 
          outAutos.close();return false; }     
  
   if (!(outAutos<<this->getCompleteFlag()<<endl))  
      {   std::cerr << "Write error: s   !"<<endl; 
          outAutos.close();return false; }       
  
   if (!(outAutos<<this->getNormalFlag()<<endl))  
      {   std::cerr << "Write error: s7  !"<<endl; 
          outAutos.close();return false; }       
   
   //write alphasets
   if (!(outAutos<<this->getInputAlphaSize()<<endl))  
      {   std::cerr << "Write error: s71  !"<<endl; 
          outAutos.close();return false; }    
   std::vector<string> inAlpha = this->getInputAlpha();
   for(int unsigned index=0;index<this->getInputAlphaSize();++index)
      if(!(outAutos<<inAlpha[index]<<endl)) 
      {   std::cerr << "Write error: s81  !"<<endl; 
          outAutos.close();return false;}

   if (!(outAutos<<this->getOutputAlphaSize()<<endl))  
      {   std::cerr << "Write error: s72  !"<<endl; 
          outAutos.close();return false; }    
   std::vector<string> outAlpha = this->getOutputAlpha();
   for(int unsigned index=0;index<this->getOutputAlphaSize();++index)
      if(!(outAutos<<outAlpha[index]<<endl)) 
      {   std::cerr << "Write error: s82  !"<<endl; 
          outAutos.close();return false;}
 
 
   if (!(outAutos<<this->getHistType()<<endl))  
      {   std::cerr << "Write error: s9  !"<<endl; 
          outAutos.close();return false; }         
   //cout<<"HIS= "<<this->getHistType() <<endl;

   bool error = false;
   ListOfSymbols l = this->getHistOfITGraph();
   ListOfSymbols::iterator i = l.begin();
   while(i!=l.end())
      {
	if(!(outAutos<<*i<<endl)) 
		{error = true;break;}
              ++i;     
      }
   if(error) {   std::cerr << "Write error s10  !"<<endl; 
                 outAutos.close();return false; }
 
   //mark od end of history
   if(!(outAutos<<""<<endl)) 
   {   std::cerr << "Write error: s11  !"<<endl; 
       outAutos.close();return false; }        
        

   //write states of automaton
   //terminated by id==-2
   error = false;
   
   std::list<State<T> > sL = listOfStates;
   std::list<State<T> >::iterator iS = sL.begin();
   for(;iS!=sL.end();++iS)
    {
      if (!(outAutos<<iS->getIdOfState()<<endl))  
	 {error=true;break;}
      if(!(outAutos<<iS->getName()<<endl)) 
	 {error = true;break;}
      if (!(outAutos<<iS->getAccessibleFlag()<<endl))  
	 {error=true;break;} 
      if (!(outAutos<<iS->getCoaccessibleFlag()<<endl))  
	 {error=true;break;} 
      
      ListOfSymbols hist = iS->getHistory();
      ListOfSymbols::iterator iH = hist.begin();        
      while(iH!=hist.end())
       {
	 if(!(outAutos<<*iH<<endl)) 
	    {error = true;break;}
          ++iH;    
       }
      if(error) {   std::cerr << "Write error: s12  !"<<endl;
                    outAutos.close(); return false; }
 
      //mark od end of history
      if(!(outAutos<<""<<endl)) 
      {   std::cerr << "Write error: s13  !"<<endl; 
          outAutos.close();return false; }
     
   }
   if(error) {   std::cerr << "Write error: s14  !"<<endl; 
                 outAutos.close();return false; }
   
   //mark of end of states
   if (!(outAutos<<-2<<endl))  
	 {   std::cerr << "Write error: s15  !"<<endl; 
             outAutos.close();return false; } 

   //write transitions of automaton
   //terminated by id==NULL(0)
   error = false;
   std::list<Transition<T> >  sT = listOfTrans;
   std::list<Transition<T> >::iterator iT = sT.begin();
   for(;iT!=sT.end();++iT)
    { 
           
         if (!(outAutos<<iT->getIdOfTrans()<<endl))  
	 {error=true;break;}
         if(!(outAutos<<iT->getOrgPtr()->getName()<<endl)) 
		{error = true;break;}    
         if(!(outAutos<<iT->getDstPtr()->getName()<<endl)) 
		{error = true;break;}
  
         if(!(outAutos<<iT->getLabel().getInput()<<endl)) 
		{error = true;break;}
         
         int unsigned infixSize = iT->getLabel().getOutput().getInfixSize();
	 std::vector<string> lab = iT->getLabel().getOutput().getInfixSerie();
         if(!(outAutos<<iT->getLabel().getOutput().getInfixSize()<<endl)) 
		{error = true;break;}  
         for(int unsigned i=0;i<infixSize;++i)
         if(!(outAutos<<lab[i]<<endl)) 
		{error = true;break;}
        
    }
  if(error) {   std::cerr << "Write error: s16  !"<<endl; 
                outAutos.close();return false; } 
  
  //mark of end of transitions
  string separatorA = "";
  char sepa = SEPARATOR_AUTO;
  separatorA = separatorA + sepa;
  if (!(outAutos<<separatorA<<endl))  
     {   std::cerr << "Write error: s17  !"<<endl; 
         outAutos.close();return false; }
  //mark of end of file
  string separatorF = "";
  sepa = SEPARATOR_FILE;
  separatorF = separatorF + sepa;
  if (!(outAutos<<separatorF<<endl))  
     {   std::cerr << "Write error: s18  !"<<endl; 
         outAutos.close();return false; }

  outAutos.close();
  return true;
}

/*********************************************************************/
/*            methodes of LTOLTransducer                             */
/*********************************************************************/

template<class T>
LTOLTransducer<T>* LTOLTransducer<T>::reverseInOut(string name)
{
  LTOLTransducer<T>* tmp = new LTOLTransducer<T>(this->getName(),0, 
                           getInputAlpha(),getOutputAlpha());
  tmp->reproduct(*this);
  tmp->setName(name);
  tmp->setInitialName("i_"+name);
  tmp->setTerminalName("t_"+name);

  std::list<State<T> >::iterator sPtr,i = this->listOfStates.begin();
  std::list<MetaNode<T> > exits = i->getAllExits();
  std::list<MetaNode<T> >::iterator ie;
  ListOfSymbols hist;
  string n;
  int count = tmp->getNoStates();
  ++count;
  for(ie=exits.begin();ie!=exits.end();++ie)
     {
       if(!ie->getTransPtr()->getLabel().getOutput().empty())
          {

            //cout<<"outputX= "<<ie->getTransPtr()->getLabel().getOutput()
            //    <<endl;

            n = TreeLabel::intToString(count++); 
            sPtr = tmp->addState(n,hist,true);
            string input="";
            TreeLabel output =  TreeLabel(this->getOutputAlpha()[0][0]);
            T lab =T(input,output);
            tmp->addTransition(ie->getTransPtr()->getOrgPtr(),sPtr,lab);
            input  = ie->getTransPtr()->getLabel().getInput();
            output = ie->getTransPtr()->getLabel().getOutput();
            lab = T(input,output);
            tmp->addTransition(sPtr,ie->getTransPtr()->getDstPtr(),lab);
          } 
     }

  ++i;
  exits = i->getAllEntrances();
  for(ie=exits.begin();ie!=exits.end();++ie)
     {
       if(!ie->getTransPtr()->getLabel().getOutput().empty())
          {
            //cout<<"outputE= "<<ie->getTransPtr()->getLabel().getOutput()
            //    <<endl;

            n = TreeLabel::intToString(count++); 
            sPtr = tmp->addState(n,hist,true); 
            string input  = ie->getTransPtr()->getLabel().getInput();
            TreeLabel output = ie->getTransPtr()->getLabel().getOutput();
            T lab = T(input,output);           
            tmp->addTransition(ie->getTransPtr()->getOrgPtr(),sPtr,lab);

            input="";
            output =  TreeLabel(this->getOutputAlpha()[0][0]);
            lab =T(input,output);
            tmp->addTransition(sPtr,ie->getTransPtr()->getDstPtr(),lab);
          } 
     }

  std::list<Transition<T> >::iterator j;

  std::vector<string> t = this->getInputAlpha(); 
  std::vector<string> exp = this->getOutputAlpha();
  tmp->setInputAlpha(exp);
  tmp->setOutputAlpha(t);

  for(j=tmp->listOfTrans.begin();j!=tmp->listOfTrans.end();++j) 
    {
       string input;
       TreeLabel output =TreeLabel(this->getInputAlpha()[0][0]); 
       if(j->getLabel().getInput().empty())
          output =TreeLabel(this->getInputAlpha()[0][0]); 
       else 
	 {
          std::vector<string> exp;
          exp.insert(exp.end(),j->getLabel().getInput());
          output =TreeLabel(exp,this->getInputAlpha()[0][0]);
	 } 
       if(j->getLabel().getOutput().empty())
          input="";
       else
          input = j->getLabel().getOutput().getInfixString(); 

       T lab = T(input,output);       
       j->setLabel(lab); 
    }

  hist.insert(hist.end(), this->getName());
  tmp->setHistOfITGraph(hist);
  tmp->setHistType(REVERSE_INOUT_LTOL_HISTORY);
  return tmp;
}

template<class T>
LTOLTransducer<T>* LTOLTransducer<T>::reverse(string name)
{
  Transducer<T>* tmp1 = Transducer<T>::reverse(name);
  LTOLTransducer<T>* tmp = new LTOLTransducer<T>(*tmp1); 
  delete(tmp1);
  return tmp; 
}

template<class T>
LTOLTransducer<T>::LTOLTransducer(const LTOLTransducer<T>& t):
Transducer<T>(t->getName(),0,t.getInputAlpha(),
              t.getOutputAlpha())
{
  this->reproduct(t);
}

template<class T>
LTOLTransducer<T>* LTOLTransducer<T>::uniformize(string name)
{
   Transducer<T>* tmp1 = Transducer<T>::uniformize(name);
   LTOLTransducer<T>* tmp = new  LTOLTransducer<T>(*tmp1);
   delete(tmp1);
   return tmp;
}

template<class T>
void LTOLTransducer<T>::setDeterministic(void)
{
  std::vector<string> inAlpha  = this->getInputAlpha();
  std::vector<string>::iterator inStart = inAlpha.begin();
  std::vector<string> outAlpha = this->getOutputAlpha();
  std::vector<string>::iterator outStart = outAlpha.begin();
  std::map<string,string> reached;
  //  std::list<State<T> > listS 
  std::list<State<T> >::iterator kState = listOfStates.begin();
  std::list<State<T> >::iterator iState; 
  //skip two states
  ++kState;++kState;
  std::list<MetaNode<T> >::iterator iNode;

  std::list<MetaNode<T> > listNodes;
  // if numberOfInitStates > 1 : non deterministic
  if(this->getNoInitStates() != 1) 
    {
      this->setDetermFlag(0);
      return;
    }
 
  for(iState=kState;iState!=listOfStates.end();++iState)
    {  //get exits transitions 
      listNodes = iState->getAllExits();
      reached.clear();
      for(iNode=listNodes.begin();iNode!=listNodes.end();++iNode)
	 { 
           T lab = iNode->getTransPtr()->getLabel();
           if(iNode->getTransPtr()->getDstPtr()->getIdOfState()!=-1)
	     {
              string input = lab.getInput();
              string output= lab.getOutput().getInfixString();
              char c1 = input[0];
              string str1="";str1=str1+c1;
              if(find(++inStart,inAlpha.end(), str1) == inAlpha.end())
              { 
               this->setDetermFlag(0);
               return;
              }
              char c2 = output[0];
              string str2="";str2=str2+c2;
              if(find(++outStart, outAlpha.end(),str2) == outAlpha.end())
              { 
               this->setDetermFlag(0);
               return;
              }
              string couple = "";couple=couple+c1+c2;

              if(reached.find(couple) != reached.end())
              { 
               this->setDetermFlag(0);
               return;
              }
              reached[couple] = couple;
	     }
         }
    } 
    this->setDetermFlag(1);
    return;
}

template<class T>
LTOLTransducer<T>* LTOLTransducer<T>::compose2T(LTOLTransducer<T>* t,
                    string name)
{
  //LTOLTransducer<T>* this1 = this->reverseInOut();
  int i,j;
  int count = 1;
  string n1,n2;
  std::list<State<T> >::iterator iState;
  std::list<QueuePos<T> > queue;
  int noLines   = this->getNoStates();noLines += 2;
  int noColumes = t->getNoStates();noColumes += 2;
  std::vector<std::list<State<T> >::iterator> thisVector = 
       vector<std::list<State<T> >::iterator>(noLines);
  i=0;
  for(iState=this->listOfStates.begin();
           iState!=this->listOfStates.end();++iState)
    {
      if(i>1) iState->setIdOfState(i);
      thisVector[i++] = iState;
    }

  std::vector<std::list<State<T> >::iterator> tVector = 
       vector<std::list<State<T> >::iterator>(noColumes);
  
  i=0;
  for(iState=t->listOfStates.begin();
             iState!=t->listOfStates.end();++iState)
    {
       if(i>1) iState->setIdOfState(i); 
       tVector[i++] = iState;
    }

  LTOLTransducer<T>* tTmp = new LTOLTransducer<T>(name, 0, 
      this->getInputAlpha(),this->getOutputAlpha()); 
  std::list<State<T> >::iterator matrix[noLines][noColumes];
  for(i=0;i<noLines;++i)
  for(j=0;j<noColumes;++j) matrix[i][j] = tTmp->listOfStates.end();
  
  i=j=0;
  
  QueuePos<T> p = 
           QueuePos<T>(i,j,tTmp->listOfStates.begin());
  queue.insert(queue.end(),p);
  
  matrix[i][j] = tTmp->listOfStates.begin();
     
  while(!queue.empty())
    {
       i = queue.begin()->i;
       j = queue.begin()->j;
       std::list<MetaNode<T> >::iterator i1;
       std::list<MetaNode<T> >::iterator i2;
       std::list<MetaNode<T> > lnExits = thisVector[i]->getAllExits();
       std::list<MetaNode<T> > clExits = tVector[j]->getAllExits();
       for(i1=lnExits.begin();i1!=lnExits.end();++i1)
       for(i2=clExits.begin();i2!=clExits.end();++i2)
	{ 
          T lab1 = i1->getTransPtr()->getLabel();
          T lab2 = i2->getTransPtr()->getLabel();

          if(lab1.getOutput().getInfixString()==lab2.getInput())
	    { 
              std::list<State<T> >::iterator org = queue.begin()->iter;
              std::list<State<T> >::iterator dst;
              int u = i1->getTransPtr()->getDstPtr()->getIdOfState();
              int v = i2->getTransPtr()->getDstPtr()->getIdOfState();
               
              if(u==-1&&v==-1)
                {   
                  std::list<State<T> >::iterator k =
                                        tTmp->listOfStates.begin();
                  ++k;  
                  //matrix[1][1] = k; 
                  //dst = matrix[1][1];
                  dst = k;
                }
              else if(u==-1||v==-1) continue;
              else
                {
                   
                  if(matrix[u][v]==tTmp->listOfStates.end()) 
		  { 
                  ListOfSymbols hist;
                 
                  hist.insert(hist.end(),thisVector[u]->getName());
                  hist.insert(hist.end(),tVector[v]->getName());
                  std::list<State<T> >::iterator k = 
                       tTmp->addState(intToString(count), hist, true);
                  if(k!=tTmp->listOfStates.end())
		    {
                      count++;
                      matrix[u][v] = k;
                      QueuePos<T> p = QueuePos<T>(u,v,k);
                      queue.insert(queue.end(),p);
                      dst = k;
		    }
                  else    
		    {
                      cout<<"Addition of states is invalid !"<<endl;
                      return NULL; 
		    }
		  } 
                  else dst = matrix[u][v];
		}

              T lab = T(lab1.getInput(),lab2.getOutput());   
              tTmp->addTransition(org,dst,lab,true);
	    }
	}
       queue.erase(queue.begin());
    } 

  tTmp->setNoInitStates();
  tTmp->setNoTermStates();
  ListOfSymbols hist;
  hist.insert(hist.end(), this->getName() );
  hist.insert(hist.end(), t->getName() );
  tTmp->setHistOfITGraph(hist);
  tTmp->setHistType(COMPOSE_TT_LTOL_HISTORY);
  return tTmp;
}

/***********************************************************************/
/*            The methodes of LTOEXPTransducer                         */
/***********************************************************************/

template<class T>
LTOEXPTransducer<T>::LTOEXPTransducer(const LTOEXPTransducer<T>& t):
Transducer<T>(t.getName(),0,t.getInputAlpha(),t.getOutputAlpha())
{
  this->reproduct(t);
}

template<class T>
LTOEXPTransducer<T>::LTOEXPTransducer(const WTOEXPTransducer<T>& t):
Transducer<T>(t.getName(),0,t.getInputAlpha(),t.getOutputAlpha())
{
  this->reproduct(t);
}

template<class T>
LTOEXPTransducer<T>* LTOEXPTransducer<T>::reverse(string name)
{
  Transducer<T>* tmp1 = Transducer<T>::reverse(name);
  LTOEXPTransducer<T>* tmp = new LTOEXPTransducer<T>(*tmp1);
  delete(tmp1);
  return tmp; 
}

template<class T>
bool LTOEXPTransducer<T>::testLTOW(void)
{
  std::list<Transition<T> >::iterator i = listOfTrans.begin();
  for(;i!=listOfTrans.end();++i) 
    {
      if(!(i->getLabel().getOutput().testWord())) return false;
    }
  return true;
}

template<class T>
LTOEXPTransducer<T>* LTOEXPTransducer<T>::reverseInOut(string name)
{
   WTOEXPTransducer<T> tmp1 = WTOEXPTransducer<T>(*this);
   LTOEXPTransducer<T>* tmp  = tmp1.reverseInOut(name);
   return tmp; 
}

template<class T>
LTOEXPTransducer<T>* LTOEXPTransducer<T>::uniformize(string name)
{
   Transducer<T>* tmp1 = Transducer<T>::uniformize(name); 
   LTOEXPTransducer<T>* tmp = new  LTOEXPTransducer<T>(*tmp1);
   delete(tmp1);
   return tmp;
}

//recognize an expression 
//flag : if word is an expression of terminal transition
template<class T>
void LTOEXPTransducer<T>::traceExp(std::vector<string> alpha,TreeLabel& exp,
std::list<State<T> >::iterator org1, 
std::list<Trace<T> >& trace,bool flag)
{ 
  //cout<<"exp= "<<exp<<", org= "<<org1->getName()<<endl<<flush;

  Automata<TreeLabel>* auto1 = exp.expToAuto("exp_auto",alpha);

  //cout<<"\nauto1 :";
  //auto1->display(0);

  LTOEXPTransducer<T> t =  LTOEXPTransducer<T>(*this);
  
  std::list<State<T> >::iterator ij = listOfStates.begin();
  ++ij;++ij;

  if(!flag)
    {
     t.setOffTerminalState();
     for(;ij!=listOfStates.end();++ij)
     { string x="";
       t.setOnTerminalState(ij->getName(),x,
         TreeLabel(this->getOutputAlpha()[0][0]));
     }
    }

  if(!org1->isInitialDummy())
    {
       string x="";
       t.setOffInitialState();  
       t.setOnInitialState(org1->getName(),x,
         TreeLabel(this->getOutputAlpha()[0][0]));
    }
    
  //cout<<"\nt= :";
  //t.display(0);
  //cin.get();

  /*********************************************************************/
  int i,j;
  int count = 1;
  string n1,n2;
  std::list<std::list<State<TreeLabel> >::iterator>::iterator jState;
  std::list<State<T> >::iterator iState;
  std::list<QueuePos<T> > queue;
  int noLines   = t.getNoStates();noLines += 2;
  int noColumes = auto1->getNoStates();noColumes += 2;
  std::vector<std::list<State<T> >::iterator> thisVector = 
       vector<std::list<State<T> >::iterator>(noLines);
  i=0;
  for(iState=t.listOfStates.begin();
           iState!=t.listOfStates.end();++iState)
    {
      if(i>1) iState->setIdOfState(i);
      thisVector[i++] = iState;
    }

  std::vector<std::list<State<TreeLabel> >::iterator> auto1Vector = 
       vector<std::list<State<TreeLabel> >::iterator>(noColumes);
  std::list<std::list<State<TreeLabel> >::iterator> lS = 
            auto1->getIterListOfStates(); 
  i=0;
  for(jState=lS.begin();
             jState!=lS.end();++jState)
    {
       if(i>1) (*jState)->setIdOfState(i); 
       auto1Vector[i++] = (*jState);
    }

   LTOEXPTransducer<T>* autoTmp = new  LTOEXPTransducer<T>("autoTmp", 0,
                     t.getInputAlpha(),t.getOutputAlpha());

  std::list<State<T> >::iterator matrix[noLines][noColumes];
  for(i=0;i<noLines;++i)
    for(j=0;j<noColumes;++j) matrix[i][j] = autoTmp->listOfStates.end();
  
  
  i=j=0;
  
  QueuePos<T> p = 
    QueuePos<T>(i,j,autoTmp->listOfStates.begin()); 
  queue.insert(queue.end(),p);
  
  matrix[i][j] = autoTmp->listOfStates.begin();
  matrix[i][j]->setIdOfState(0);// for the next

  while(!queue.empty())
    {
       i = queue.begin()->i;
       j = queue.begin()->j;
       std::list<MetaNode<T> >::iterator i1;
       std::list<MetaNode<TreeLabel> >::iterator i2;
       std::list<MetaNode<T> > lnExits = thisVector[i]->getAllExits();
       std::list<MetaNode<TreeLabel> > clExits = auto1Vector[j]->getAllExits();
       for(i1=lnExits.begin();i1!=lnExits.end();++i1)
       for(i2=clExits.begin();i2!=clExits.end();++i2)
	{ 
          //this comparaison between 2 chars
          T lab = i1->getTransPtr()->getLabel();
          //lab.input is string !!!
          string s1,s2;
          s1 = lab.getInput();
          s2 = i2->getTransPtr()->getLabel().getInfixString();

          if(s1==s2)
	    { 
              std::list<State<T> >::iterator org = queue.begin()->iter;
              std::list<State<T> >::iterator dst;
              int u = i1->getTransPtr()->getDstPtr()->getIdOfState();
              int v = i2->getTransPtr()->getDstPtr()->getIdOfState();
               
              if(u==-1&&v==-1)
                {   
                  std::list<State<T> >::iterator k =
		     autoTmp->listOfStates.begin();
                  ++k;  
                  //matrix[1][1] = k; 
                  //dst = matrix[1][1];
                  dst = k; 
                  k->setIdOfState(u);// for the next
                }
              else if(u==-1||v==-1) continue;
              else 
                {
		 
                 if(matrix[u][v]== autoTmp->listOfStates.end())
                    
		 { 
                  ListOfSymbols hist;
                 
                  hist.insert(hist.end(),thisVector[u]->getName());
                  hist.insert(hist.end(),auto1Vector[v]->getName());
                  std::list<State<T> >::iterator k = 
                       autoTmp->addState(intToString(count), hist, true);

                  k->setIdOfState(u);// for the next

                  if(k!=autoTmp->listOfStates.end())
		    {
                      count++;
                      matrix[u][v] = k;
                      QueuePos<T> p = QueuePos<T>(u,v,k);
                      queue.insert(queue.end(),p);
                      dst = k;
		    }
                  else    
		    {
                      cout<<"Addition of states is invalid !"<<endl;
                      return; 
		    }
		}  
                else dst = matrix[u][v];
	       }
              autoTmp->addTransition(org,dst,lab,true);
	    }
	}
       queue.erase(queue.begin());
    } 

  autoTmp->setNoInitStates();
  autoTmp->setNoTermStates();
  ListOfSymbols hist;
  hist.insert(hist.end(), t.getName() );
  hist.insert(hist.end(), auto1->getName() );
  autoTmp->setHistOfITGraph(hist);
  autoTmp->setHistType(COMPOSE_TA_T_LTOEXP_HISTORY);

  /*********************************************************************/
  
  //cout<<"before "<<endl;
  //autoTmp->display(0); 

  if(flag)
    {
       Automata<TreeLabel>* domain = autoTmp->getDomain("domain");
       domain->trimAuto(""); 
       
       if(!domain->emptyLang())
	 {
           GeneralAuto<TreeLabel>* g = autoTmp->getImage("image");

           //cout<<"before language ="<<endl;
           //g->display(0);

           std::vector<string> infix;
           g->language(infix);
           if(infix.size()==0) return ;
           TreeLabel exp = TreeLabel(infix,this->getOutputAlpha()[0][0]);
          
           Trace<T> t1;
           t1.path = exp;
           t1.mark = -1;
           trace.insert(trace.end(),t1);
           delete(g);
	 }

       delete(domain);
       delete(auto1);
       delete(autoTmp);    
       return;
    }

  std::list<State<T> >::iterator iS = autoTmp->listOfStates.begin();
  ++iS;//pointer to "t_"
  std::list<MetaNode<T> > enters = iS->getAllEntrances(); 
  std::list<MetaNode<T> >::iterator iE = enters.begin();
  for(;iE!=enters.end();++iE)
    {  
       int dst = iE->getTransPtr()->getOrgPtr()->getIdOfState();

       LTOEXPTransducer<T> tmp1 =  LTOEXPTransducer<T>(*autoTmp);

       T lab = iE->getTransPtr()->getLabel();
       string terminal = iE->getTransPtr()->getOrgPtr()->getName();
       tmp1.setOffTerminalState();
       tmp1.setOnTerminalState(terminal,lab.getInput(),lab.getOutput());

       //cout<<"after "<<endl;
       //tmp1.display(0); 

       
       //cout<<" name dst= "<<iE->getTransPtr()->getOrgPtr()->getName()
       //    <<endl<<flush;
       //cout<<" dst= "<<dst<<endl<<flush;      

       Automata<TreeLabel>* domain = tmp1.getDomain("domain");
       domain->trimAuto("");  

       if(!domain->emptyLang())
	 {
           GeneralAuto<TreeLabel>* g = tmp1.getImage("image");
           
           //cout<<"before language ="<<endl;
           //g->display(0);

           //TreeLabel exp(this->getOutputAlpha()[0][0]);
           std::vector<string> infix;
           g->language(infix);
           if(infix.size()==0) continue;

           //std::vector<string> infix = exp.createInfixSerie(exp.getTree());
           //exp.setInfixSerie(infix);

           //if infix.size==1 and infix[0] = this->getOutputAlpha()[0][0]
           //then "exp" is empty word           

           exp = TreeLabel(infix,this->getOutputAlpha()[0][0]);
       
           Trace<T> t1;
           t1.path = exp;
           t1.mark = dst; 
           
           trace.insert(trace.end(),t1);
	   //cout<<"path= "<<t1.path<<endl;
	   //cout<<"mark= "<<t1.mark<<endl;
           delete(g);
	 }  

       delete(domain);
     }
  
   delete(auto1);
   delete(autoTmp);  
}

template<class T> 
std::list<Trace<T> > 
LTOEXPTransducer<T>::cleanTrace2(std::list<Trace<T> >& trace)
{ 
  std::list<Trace<T> > tmp;
  std::list<Trace<T> >::iterator jTrace;
  std::list<Trace<T> >::iterator iTrace = trace.begin();
  int count=0;
  
  for(;iTrace!=trace.end();++iTrace)
    { 
     
      if(tmp.empty()) tmp.insert(tmp.end(),*iTrace); 
      else
	{ 
         bool stop = false;
         for(jTrace=tmp.begin();jTrace!=tmp.end();++jTrace)
          if(jTrace->mark==iTrace->mark) 
            {stop=true; 
             jTrace->path = jTrace->path + iTrace->path;  
             break;}
         if(!stop) tmp.insert(tmp.end(), *iTrace);
	} 
    }
  return tmp;  
}


//compute [t * this] : it is inverse !!!
template<class T>
LTOEXPTransducer<T>* LTOEXPTransducer<T>::composeLTOEXP(
LTOEXPTransducer<T>* tm, string name)
{
  LTOEXPTransducer<T> t =  LTOEXPTransducer<T>(*tm);
  int i,j;
  int count = 1;
  string n1,n2;
  std::list<State<T> >::iterator iState;
  std::list<QueuePos<T> > queue;
  int noLines   = this->getNoStates();noLines += 2;
  int noColumes = t.getNoStates();noColumes += 2;
  std::vector<std::list<State<T> >::iterator> thisVector = 
       vector<std::list<State<T> >::iterator>(noLines);

  i=0;
  for(iState=this->listOfStates.begin();
           iState!=this->listOfStates.end();++iState)
    {
      if(i>1) iState->setIdOfState(i);
      thisVector[i++] = iState;
    }

  std::vector<std::list<State<T> >::iterator> tVector = 
       vector<std::list<State<T> >::iterator>(noColumes);
  
  i=0;
  for(iState=t.listOfStates.begin();
             iState!=t.listOfStates.end();++iState)
    {
       if(i>1) iState->setIdOfState(i); 
       tVector[i++] = iState;
    }

  LTOEXPTransducer<T>* tTmp = new LTOEXPTransducer<T>(name, 0, 
                       this->getInputAlpha(),t.getOutputAlpha()); 
  std::list<State<T> >::iterator matrix[noLines][noColumes];
  for(i=0;i<noLines;++i)
  for(j=0;j<noColumes;++j) matrix[i][j] = tTmp->listOfStates.end();
  
  i=j=0;
  
  QueuePos<T> p = 
           QueuePos<T>(i,j,tTmp->listOfStates.begin());
  queue.insert(queue.end(),p);
  
  matrix[i][j] = tTmp->listOfStates.begin();
     
  while(!queue.empty())
    {
       i = queue.begin()->i;
       j = queue.begin()->j;
       std::list<MetaNode<T> >::iterator i1;
       
       std::list<MetaNode<T> > lnExits = thisVector[i]->getAllExits();
       //cout<<"line= "<<i<<" colume= "<<j<<endl<<flush;
       for(i1=lnExits.begin();i1!=lnExits.end();++i1)
	 { 
          T lab1 = i1->getTransPtr()->getLabel();
          std::list<Trace<T> > trace;trace.clear();

          //cout<<"input= "<<lab1.getInput()<<" ouput = "
          //    <<lab1.getOutput().getInfixString()<<endl<<flush;

          bool terminal = false;
          if(i1->getTransPtr()->getDstPtr()->getIdOfState()==-1) 
             terminal = true;
          else terminal = false;
          TreeLabel exp = lab1.getOutput(); 
          
          //cout<<"\nbefore trace, inStr="<<exp;
          //if(exp.empty()) cout<<"\empty ";
          //else  cout<<"\n no empty "<<endl;
          //exp.print();
            
          t.traceExp(this->getOutputAlpha(),exp,tVector[j],trace,terminal);
	   
	  //cout<<"size= "<<trace.size()<<endl<<flush;cin.get();
              
              std::list<Trace<T> >::iterator iTrace;
              std::list<Trace<T> > traceMap = cleanTrace2(trace);

              
              for(iTrace=traceMap.begin();iTrace!=traceMap.end();++iTrace)
		{
                  //cout<<"path="<<iTrace->path<<endl<<flush;
                  //cout<<"dst="<<iTrace->mark->getName()<<endl<<flush;
                  std::list<State<T> >::iterator org = queue.begin()->iter;
                  std::list<State<T> >::iterator dst;
                  int u = i1->getTransPtr()->getDstPtr()->getIdOfState();
                  int v = iTrace->mark;
              
                  if(u==-1&&v==-1)
                  {   
                    std::list<State<T> >::iterator k =
                                        tTmp->listOfStates.begin();
                    ++k;   
                    dst = k;
                  }
                  else if((v==-1)||(u==-1))  continue;
                  else    
                  { 
                   
                    if(matrix[u][v]==tTmp->listOfStates.end()) 
		      { 
                       ListOfSymbols hist;
                       hist.insert(hist.end(),thisVector[u]->getName());
                       hist.insert(hist.end(),tVector[v]->getName());
                       
                       std::list<State<T> >::iterator k = 
                            tTmp->addState(intToString(count), hist, true);

                       //cout<<"add state= "<<thisVector[u]->getName()<<","<<
		       //	 tVector[v]->getName()<<endl;

                       if(k!=tTmp->listOfStates.end())
		       {
                        count++;
                        matrix[u][v] = k;
                        QueuePos<T> p = QueuePos<T>(u,v,k);
                        queue.insert(queue.end(),p);
                        dst = k;
		       }
                       else    
		       {
                        cout<<"Addition of state is invalid !"<<endl;
                        return NULL; 
		       }
		      }  
                    else dst = matrix[u][v];
		  }
               
                  //std::vector<string> exp = iTrace->path.createInfixSerie();
                  //iTrace->path.setInfixSerie(exp); 
                  T lab = T(lab1.getInput(),iTrace->path);
                   
                  tTmp->addTransition(org,dst,lab,true);
		}//end of for "iTrace" 
	   
         }//end of for
       queue.erase(queue.begin());
    }//end of while

  tTmp->setNoInitStates();
  tTmp->setNoTermStates();
  ListOfSymbols hist;
  hist.insert(hist.end(), this->getName() );
  hist.insert(hist.end(), t.getName() );
  tTmp->setHistOfITGraph(hist);
  tTmp->setHistType(COMPOSE_TT_LTOEXP_HISTORY);
  return tTmp;
}

/***********************************************************************/
/*            The methodes of WTOEXPTransducer                         */
/***********************************************************************/

template<class T>
WTOEXPTransducer<T>::WTOEXPTransducer(const WTOEXPTransducer<T>& t):
Transducer<T>(t.getName(),0,t.getInputAlpha(),t.getOutputAlpha())
{
  this->reproduct(t);
}

template<class T>
WTOEXPTransducer<T>::WTOEXPTransducer(const LTOEXPTransducer<T>& t):
Transducer<T>(t.getName(),0,t.getInputAlpha(),t.getOutputAlpha())
{
  this->reproduct(t);
}

template<class T>
GeneralAuto<TreeLabel>* WTOEXPTransducer<T>::getDomain(string name)
{
  Automata<TreeLabel>*    tmp1 = this->getDomain(name);
  GeneralAuto<TreeLabel>* tmp  = new GeneralAuto<TreeLabel>(*tmp1);
  delete(tmp1);
  return tmp;
} 
template<class T>
WTOEXPTransducer<T>* WTOEXPTransducer<T>::reverse(string name)
{
  Transducer<T>* tmp1 = Transducer<T>::reverse(name);
  WTOEXPTransducer<T>* tmp = new WTOEXPTransducer<T>(*tmp1);
  delete(tmp1);
  return tmp; 
}


template<class T>
WTOEXPTransducer<T>* WTOEXPTransducer<T>::wordToLetter(string name)
{
  std::list<State<T> >::iterator iS = this->listOfStates.begin();
  std::list<State<T> >::iterator sPtr1,sPtr2,iInit;

  std::list<MetaNode<T> > exits;
  std::list<MetaNode<T> >::iterator ie;
  WTOEXPTransducer<T>* tmp =  new WTOEXPTransducer<T>(name,0,
                              getInputAlpha(),getOutputAlpha());

  int numStates   = this->getNoStates();numStates += 2;
  std::vector<std::list<State<T> >::iterator> stateV = 
       vector<std::list<State<T> >::iterator>(numStates);

  std::list<std::list<State<T> >::iterator> lS = tmp->getIterListOfStates();
  std::list<std::list<State<T> >::iterator>::iterator iter; 
  iter = lS.begin();      
  stateV[0] = *iter;
  ++iter;
  stateV[1] = *iter;

  int i=0; int count = 1;
  for(iInit=this->listOfStates.begin();iInit!=this->listOfStates.end();++iInit)
    {
      ListOfSymbols his; 
      his.clear();
      if(i>1) 
        {
           iInit->setIdOfState(i);
           string n; 
           n = TreeLabel::intToString(count++);
           his.insert(his.end(), iInit->getName());
           sPtr1 = tmp->addState(n,his,true);
           sPtr1->setIdOfState(i);
           stateV[i] = sPtr1;
	}
      ++i;
    }
 
  string n1,n2;
  for(iS = listOfStates.begin();iS!= listOfStates.end();++iS)
    {
       int id1;
       id1  = iS->getIdOfState();
       if(id1==-1) id1 = 1;
       exits = iS->getAllExits();       
       for(ie=exits.begin();ie!=exits.end();++ie)
	 {
             string s,t; 
             sPtr2 = stateV[id1];
             s = ie->getTransPtr()->getLabel().getInput();
             if((!isInputEmptyWord(s))&&s.size()>1)
	       { 
                   for(int unsigned i=0;i<s.size()-1;++i)
                   { 
                     t="";t=t+s[i];
                     ListOfSymbols his;
                     his.clear();his.insert(his.end(),"added_state");
                     n2 = TreeLabel::intToString(count++);
                     sPtr1 = sPtr2;
                     sPtr2 = tmp->addState(n2,his,true);

                     if(i==0)
		       {
                         T lab=T(t,ie->getTransPtr()->getLabel().getOutput());
                         tmp->addTransition(sPtr1,sPtr2,lab);
		       } 
                     else
                       {
                        TreeLabel lab1=TreeLabel(this->getOutputAlpha()[0][0]);
                        T lab=T(t,lab1); 
                        tmp->addTransition(sPtr1,sPtr2,lab);
                       }
		   } 

                   int unsigned k = s.size()-1; 
                   t="";t=t+s[k];
                   sPtr1 = sPtr2;
                   int id2;
                   id2  = ie->getTransPtr()->getDstPtr()->getIdOfState();
                   if(id2==-1) id2 = 1;
                   sPtr2 = stateV[id2];
                  
                   TreeLabel lab1=TreeLabel(this->getOutputAlpha()[0][0]);
                   T lab=T(t,lab1); 
                   tmp->addTransition(sPtr1,sPtr2,lab);
	       }
	 
	     else //for the transitions with empty input and
	          // the initial states( input is always empty !!!)  
	       {    
                  sPtr1 = sPtr2;
                  int id3;
                  id3  = ie->getTransPtr()->getDstPtr()->getIdOfState();
                  if(id3==-1) id3 = 1; 
                  sPtr2 = stateV[id3];
                  TreeLabel lab1=ie->getTransPtr()->getLabel().getOutput();
                  T lab=T(s,lab1); 
                  tmp->addTransition(sPtr1,sPtr2,lab);
	       }
	 }
    }
  tmp->setNoInitStates();
  tmp->setNoTermStates();
  return tmp;
}


template<class T>
void WTOEXPTransducer<T>::starZ(TreeLabel**& matrix, int size)
{
  //line for "from", colume for "to"
  std::list<State<TreeLabel> >::iterator sPtr;
  string emptyW = matrix[0][0].getEmptyWord();

  std::vector<std::list<State<TreeLabel> >::iterator> inits = 
       vector<std::list<State<TreeLabel> >::iterator>(size);
  std::vector<std::list<State<TreeLabel> >::iterator> terms = 
       vector<std::list<State<TreeLabel> >::iterator>(size);
  int i=0,j=0;
  GeneralAuto<TreeLabel>* tmp = 
              new GeneralAuto<TreeLabel>("tmp",0,this->getOutputAlpha());

  ListOfSymbols his; 

  int count = 1; 
  for(i=0;i<size;++i)
    {
      string n; 
      TreeLabel lab(emptyW[0]);

      n = TreeLabel::intToString(count++);
      sPtr = tmp->addState(n,his,true);

      n = TreeLabel::intToString(count++);
      inits[i] = tmp->addState(n,his,true);
      tmp->setInitialState(n); 
      tmp->addTransition(inits[i], sPtr,lab,true); 

      n = TreeLabel::intToString(count++); 
      terms[i] = tmp->addState(n,his,true);

      //pay attention !!! index = id - 2 
      terms[i]->setIdOfState(i+2);
      tmp->setTerminalState(n);
      tmp->addTransition(sPtr,terms[i],lab,true);  
    }
 
  std::list<MetaNode<TreeLabel> > l,l1,l2;
  std::list<MetaNode<TreeLabel> >::iterator li,li1,li2;

  for(i=0;i<size;++i)
  for(j=0;j<size;++j)
    {
      l1 = inits[i]->getAllExits();
      li1=l1.begin();
      l2 = inits[j]->getAllExits();
      li2=l2.begin();
      if(!matrix[i][j].isZero())  
	{

          tmp->addTransition(li1->getTransPtr()->getDstPtr(),
           li2->getTransPtr()->getDstPtr(),matrix[i][j],true); 
	}
    }
  
  for(i=0;i<size;++i)
    {
      l = inits[i]->getAllExits();
      tmp->deleteState(l.begin()->getTransPtr()->getDstPtr());
    }

  for(i=0;i<size;++i)
  for(j=0;j<size;++j)
    matrix[i][j] = TreeLabel(true); // zero

  for(i=0;i<size;++i)
    {
      l = inits[i]->getAllExits();
      for(li=l.begin();li!=l.end();++li)
	{
	  std::vector<string> exp;
          TreeLabel dummy;
          int k =li->getTransPtr()->getDstPtr()->getIdOfState()-2;   
          exp=dummy.createInfixSerie(li->getTransPtr()->getLabel().getTree());
          TreeLabel lab = TreeLabel(exp,this->getOutputAlpha()[0][0]);
          matrix[i][k]=lab;

          cout<<"z["<<i<<"]["<<j<<"]= ";
          matrix[i][k].print();

	}
    }
}

template<class T>
LTOEXPTransducer<T>* WTOEXPTransducer<T>::delEmptyInput(string name)
{ 
  int i=0,j=0,k=0;
  std::list<State<T> >::iterator sPtr,sPtr1,sPtr2,iInit;
  std::list<State<TreeLabel> >::iterator iTree1,iTree2,xi; 
  string emptyW = this->getOutputAlpha()[0];
  std::list<MetaNode<T> > exits;
  std::list<MetaNode<T> >::iterator ie; 
  int numStates = this->getNoStates();

  std::vector<std::list<State<T> >::iterator> vectorD;
  vectorD.resize(numStates+2);
 
  std::vector<std::list<State<TreeLabel> >::iterator> inits = 
       vector<std::list<State<TreeLabel> >::iterator>(numStates+2);
  std::vector<std::list<State<TreeLabel> >::iterator> terms = 
       vector<std::list<State<TreeLabel> >::iterator>(numStates+2);

  
  std::list<std::list<State<T> >::iterator> lS = this->getIterListOfStates();
  std::list<std::list<State<T> >::iterator>::iterator iter; 
  iter = lS.begin();      
  vectorD[0] = *iter;
  ++iter;
  vectorD[1] = *iter;

  i=0; 
  for(iInit=this->listOfStates.begin();iInit!=this->listOfStates.end();++iInit)
    { 
      if(i>1) { iInit->setIdOfState(i); vectorD[i] = iInit;}
      ++i;
    }

  //building et computing star Z
 
 
  GeneralAuto<TreeLabel>* zzz = 
              new GeneralAuto<TreeLabel>("zzz",0,this->getOutputAlpha());

  ListOfSymbols his; 

  int count = 1; 
  for(i=0;i<numStates+2;++i)
    {
      string n; 
      TreeLabel lab(emptyW[0]);

      n = TreeLabel::intToString(count++);
      iTree1 = zzz->addState(n,his,true);

      n = TreeLabel::intToString(count++);
      inits[i] = zzz->addState(n,his,true);
      zzz->setInitialState(n); 
      zzz->addTransition(inits[i], iTree1,lab,true); 

      n = TreeLabel::intToString(count++); 
      terms[i] = zzz->addState(n,his,true);

      //pay attention !!! index = id - 2 
      terms[i]->setIdOfState(i+2);
      zzz->setTerminalState(n);
      zzz->addTransition(iTree1,terms[i],lab,true);  
    }
 
  std::list<MetaNode<TreeLabel> > l,l1,l2;
  std::list<MetaNode<TreeLabel> >::iterator li,li1,li2;

  for(i=0;i<numStates+2;++i)
  {
    l1 = inits[i]->getAllExits();
    li1=l1.begin(); iTree1 = li1->getTransPtr()->getDstPtr();
    for(j=0;j<numStates+2;++j)
    {
      l2 = inits[j]->getAllExits();
      li2=l2.begin(); iTree2 = li2->getTransPtr()->getDstPtr();
     
      TreeLabel lab;
      lab =  TreeLabel(true);
      exits = vectorD[i]->getAllExits();      
      for(ie =exits.begin();ie!=exits.end();++ie)
	{
          int k = ie->getTransPtr()->getDstPtr()->getIdOfState();
          if(k==-1) k=1;
          if(k==j&&ie->getTransPtr()->getLabel().getInput().empty())
	    {
             lab = lab + (ie->getTransPtr()->getLabel().getOutput());       
            }
	} 
    
      if(!lab.isZero())
       {  zzz->addTransition(iTree1,iTree2,lab,true); 
       //cout<<"\nZ["<<i<<"]["<<j<<"]= ";lab.print();
       } 
    }	
  }
  
  for(i=0;i<numStates+2;++i)
    {
      l = inits[i]->getAllExits();
      zzz->deleteState(l.begin()->getTransPtr()->getDstPtr());
    }

  
  LTOEXPTransducer<T>* tmp  =  new LTOEXPTransducer<T>(name,0,
                               getInputAlpha(),getOutputAlpha());
 
  string n;
  std::vector<std::list<State<T> >::iterator> stateV;
  stateV.resize(numStates+4);

  lS = tmp->getIterListOfStates(); 
  iter = lS.begin();      
  stateV[0] = *iter;
  ++iter;
  stateV[1] = *iter;

  //cout<<"OK2 Em"<<endl<<flush;
  count = 1;
  for(i=0;i<numStates+2;++i)
    { 
      n = TreeLabel::intToString(count++);      
      sPtr = tmp->addState(n,his,true);
      sPtr->setIdOfState(i+2);
      stateV[i+2] = sPtr;
    } 

  //cout<<"OK3 Em"<<endl<<flush;

  string input = "";
  TreeLabel lab1 = TreeLabel(this->getOutputAlpha()[0][0]);
  T lab = T(input,lab1);
  tmp->setOnInitialState(TreeLabel::intToString(1),input,lab1);
  //string org,dst;
  std::list<State<T> >::iterator org,dst;
  //product between  star Z and G,product between  star Z and 
  
  //cout<<"OK31 Em"<<endl<<flush;

  for(i=0;i<numStates+2;++i)
    { 
      //cout<<"OK3 Em"<<endl<<flush;
      org = stateV[i+2];
      for(j=0;j<numStates+2;++j)
      {
        dst = stateV[j+2]; 
        for(k=0;k<numStates+2;++k)
	 {
              l1 = inits[i]->getAllExits();
              TreeLabel treeLab;
              treeLab = TreeLabel(true);               
              for(li=l1.begin();li!=l1.end();++li)
		{
                  std::vector<string> exp;
                  int u =li->getTransPtr()->getDstPtr()->getIdOfState()-2;
                  if(u==k)   
		    {
                     exp=treeLab.createInfixSerie(
                         li->getTransPtr()->getLabel().getTree());
                     treeLab = TreeLabel(exp,this->getOutputAlpha()[0][0]);
		    }
		}

              if(!treeLab.isZero())
		{
                  //cout<<"\ntreeLab= ";treeLab.print(); 
                  exits = vectorD[k]->getAllExits();
                  for(ie=exits.begin();ie!=exits.end();++ie)
		    {
                     int v = ie->getTransPtr()->getDstPtr()->getIdOfState();
                     if(v==-1) v=1;
                     string input = ie->getTransPtr()->getLabel().getInput();
                     if(v==j&&(!input.empty()))   
		     {
                        TreeLabel lab1;
                        lab1 =
                        treeLab / ie->getTransPtr()->getLabel().getOutput() ;
                        T lab = T(input,lab1);
		        tmp->addTransition(org,dst,lab,true);
		     }
		    }

		}
	 }//for k

      }//for j
	  
      //final transitions                     
      l1 = inits[i]->getAllExits();
      TreeLabel treeLab;
      treeLab = TreeLabel(true);               
      for(li=l1.begin();li!=l1.end();++li)
	 {
           std::vector<string> exp;
           int u;
           u =li->getTransPtr()->getDstPtr()->getIdOfState()-2;
           if(u==1)   
	     {
                exp=treeLab.createInfixSerie(
                    li->getTransPtr()->getLabel().getTree());
                treeLab = TreeLabel(exp,this->getOutputAlpha()[0][0]);
	     }
	 }    

      if(!treeLab.isZero())
	{
          string n = stateV[i+2]->getName();
          string input = "";
          tmp->setOnTerminalState(n,input,treeLab);
	}
    }// for i

  //cout<<"OK5 Em"<<endl<<flush;
  return tmp;
}

template<class T>
LTOEXPTransducer<T>* WTOEXPTransducer<T>::toLTOEXP(string name)
{
  WTOEXPTransducer<T>* tmp1 = this->wordToLetter(name);
  if(!tmp1->testInputIsEmpty())
    {
     LTOEXPTransducer<T>* lTmp =  
           new LTOEXPTransducer<T>(*tmp1);

     lTmp->setNoInitStates();
     lTmp->setNoTermStates();
     ListOfSymbols hist;
     hist.insert(hist.end(), this->getName());
     lTmp->setHistOfITGraph(hist);
     lTmp->setHistType(FROM_WTOEXP_HISTORY);
     return lTmp;
    }

  LTOEXPTransducer<T>* tmp  = tmp1->delEmptyInput(name);
  delete(tmp1);
  tmp->setNoInitStates();
  tmp->setNoTermStates();
  ListOfSymbols hist;
  hist.insert(hist.end(), this->getName());
  tmp->setHistOfITGraph(hist);
  tmp->setHistType(FROM_WTOEXP_HISTORY);
  return tmp;
}

template<class T>
bool WTOEXPTransducer<T>::testInputIsEmpty(void)
{
  std::list<Transition<T> > lT = this->listOfTrans;
  std::list<Transition<T> >::iterator iT;
  for(iT=lT.begin();iT!=lT.end();++iT)
      if(iT->getOrgPtr()->getIdOfState()!=0&&
         iT->getDstPtr()->getIdOfState()!=-1&&
         iT->getLabel().getInput().empty()) return true;
  return false; 
}

template<class T>
bool    WTOEXPTransducer<T>::testWTOW(void)
{
  std::list<Transition<T> >::iterator i = listOfTrans.begin();
  for(;i!=listOfTrans.end();++i) 
    {
      if(!(i->getLabel().getOutput().testWord())) return false;
    }
  return true;
}

template<class T>
LTOEXPTransducer<T>* WTOEXPTransducer<T>::reverseInOut(string name)
{
  WTOEXPTransducer<T>* tmp1 = new WTOEXPTransducer<T>(name,0, 
                              getInputAlpha(),getOutputAlpha());
  int numStates = this->getNoStates();
  std::list<State<T> >::iterator iS,iInit,sPtr,sPtr1,sPtr2;
  std::vector<std::list<State<T> >::iterator> vectorS,vectorD;

  vectorS.resize(numStates+2);
  vectorD.resize(numStates+2);
  
  vectorD[0] = tmp1->searchStateByName(tmp1->getInitialName());
  vectorD[1] = tmp1->searchStateByName(tmp1->getTerminalName()); 

  int i=0; 
  int count = 1; 
  string n;
  ListOfSymbols his;
  for(iInit=this->listOfStates.begin();iInit!=this->listOfStates.end();++iInit)
    { 
      if(i>1) 
        { 
          iInit->setIdOfState(i); 
          n = TreeLabel::intToString(count++);      
          sPtr = tmp1->addState(n,his,true);
          sPtr->setIdOfState(i); 
          vectorD[i] = sPtr;
        }
      vectorS[i] = iInit;
      ++i;
    }

  std::list<State<TreeLabel> >::iterator iSExp;
  std::list<State<T> >::iterator orgP,dstP;
  std::list<Transition<TreeLabel> >::iterator iTExp;
  std::list<std::list<State<TreeLabel> >::iterator> iterL;
  std::list<std::list<State<TreeLabel> >::iterator>::iterator iter;
  std::list<std::list<Transition<TreeLabel> >::iterator> iterLT;
  std::list<std::list<Transition<TreeLabel> >::iterator>::iterator iterT;
  std::list<Transition<T> >::iterator iT;
  int sizeV,org,dst;
  int orgS,dstS,orgT,dstT; 
  //std::vector<std::list<State<TreeLabel> >::iterator> vectorExp;
  for(iT=this->listOfTrans.begin();iT!=this->listOfTrans.end();++iT)
    {
 
      orgP = iT->getOrgPtr();
      dstP = iT->getDstPtr();
         
      orgS  = iT->getOrgPtr()->getIdOfState();
      dstS  = iT->getDstPtr()->getIdOfState();
      if(dstS==-1) dstS = 1; 
      if(!iT->getLabel().getOutput().empty())
      { 
       
        //cout<<"eta="<<endl;
        //eta->display(0);

        sizeV = vectorD.size();
  
        if(orgP->getIdOfState()==0&&
           dstP->getIdOfState()==-1)
	  { 
	    //cout<<"OK 2"<<endl<<flush;

             sizeV+=2;
             vectorD.resize(sizeV);
             n = TreeLabel::intToString(count++);     
             sPtr1 = tmp1->addState(n,his,true);
             orgT = i; 
             vectorD[i] = sPtr1;
             sPtr1->setIdOfState(i++);
             n = TreeLabel::intToString(count++);     
             sPtr2 = tmp1->addState(n,his,true);
             dstT = i; 
             vectorD[i] = sPtr2;
             sPtr2->setIdOfState(i++);
             TreeLabel output;
             string input="";
             output = TreeLabel(this->getOutputAlpha()[0][0]); 
             T lab = T(input,output);
             tmp1->addTransition(vectorD[orgS],sPtr1,lab,true);
             tmp1->addTransition(sPtr2,vectorD[dstS],lab,true);
             orgS = orgT;
             dstS = dstT; 

             //cout<<"OK 3"<<endl<<flush;

	  }
        else if(orgP->getIdOfState()==0)
	  {
	    //cout<<"OK 4"<<endl<<flush;
       
             sizeV++;
             vectorD.resize(sizeV);
             n = TreeLabel::intToString(count++);     
             sPtr1 = tmp1->addState(n,his,true);
             orgT = i; 
             vectorD[i] = sPtr1;
             sPtr1->setIdOfState(i++);
             TreeLabel output;
             string input="";
             output = TreeLabel(this->getOutputAlpha()[0][0]); 
             T lab = T(input,output);
             tmp1->addTransition(vectorD[orgS],sPtr1,lab,true);
             orgS = orgT;

             //cout<<"OK 5"<<endl<<flush;
	  }
        else if(dstP->getIdOfState()==-1)
	  {   
	    //cout<<"OK 6"<<endl<<flush;

             sizeV++;
             vectorD.resize(sizeV);
             n = TreeLabel::intToString(count++);     
             sPtr2 = tmp1->addState(n,his,true);
             dstT = i; 
             vectorD[i] = sPtr2;
             sPtr2->setIdOfState(i++);
             TreeLabel output;
             string input="";
             output = TreeLabel(this->getOutputAlpha()[0][0]); 
             T lab = T(input,output);
             tmp1->addTransition(sPtr2,vectorD[dstS],lab,true);
             dstS = dstT;
  
             //cout<<"OK 7"<<endl<<flush;
	  }

        if(!iT->getLabel().getOutput().testWord())
	{
        TreeLabel exp = iT->getLabel().getOutput();
	std::vector<string> alpha = this->getOutputAlpha();
        Automata<TreeLabel>* eta=exp.expToAuto("exp",alpha);

        //sizeV >= 2
        vectorD.resize(sizeV + eta->getNoStates()); 
        iterL = eta->getIterListOfStates();
        iter =iterL.begin();  
        ++iter;
        ++iter;
 
        for(;iter!=iterL.end();++iter)
          { 
           (*iter)->setIdOfState(i); 
           n = TreeLabel::intToString(count++);      
           sPtr = tmp1->addState(n,his,true);
           sPtr->setIdOfState(i);  
           vectorD[i] = sPtr;
           ++i;
          }
        //cout<<"OK 8"<<endl<<flush;

        iterLT = eta->getIterListOfTrans();
        
        for(iterT=iterLT.begin();iterT!=iterLT.end();++iterT)
          {
            if((*iterT)->getOrgPtr()->getIdOfState()==0)
	      { 
                TreeLabel output;
                string input=iT->getLabel().getInput();
                output = TreeLabel(this->getOutputAlpha()[0][0]); 
                T lab = T(input,output);
                dst = (*iterT)->getDstPtr()->getIdOfState();
                if(dst==-1) dst = dstS;
                tmp1->addTransition(vectorD[orgS],vectorD[dst],lab,true);

                //cout<<"OK 9"<<endl<<flush;

	      }
            else if((*iterT)->getDstPtr()->getIdOfState()==-1)
	      { 
                TreeLabel output;
                string input="";
                output = TreeLabel(this->getOutputAlpha()[0][0]); 
                T lab = T(input,output);
                org = (*iterT)->getOrgPtr()->getIdOfState();
                tmp1->addTransition(vectorD[org],vectorD[dstS],lab,true); 
                
                //cout<<"OK 10"<<endl<<flush;
	      }
            else
	      { 
                TreeLabel output;
                string input="";
                output = (*iterT)->getLabel();
                output.setEmptyWord(this->getOutputAlpha()[0]); 
                T lab = T(input,output);
                org = (*iterT)->getOrgPtr()->getIdOfState();
                dst = (*iterT)->getDstPtr()->getIdOfState(); 
                tmp1->addTransition(vectorD[org],vectorD[dst],lab,true);
 
                //cout<<"OK 11"<<endl<<flush;
	      }
          }//for(iterT)
	      
        delete(eta);
	}//testWord
        else
	{
          T lab = iT->getLabel();
          tmp1->addTransition(vectorD[orgS],vectorD[dstS],lab,true);
	}
      }
      else
      {
        T lab = iT->getLabel();
        tmp1->addTransition(vectorD[orgS],vectorD[dstS],lab,true);
        
        //cout<<"OK 12"<<endl<<flush;
      }
     
    }

  std::list<Transition<T> >::iterator j;
  std::vector<string> t = this->getInputAlpha(); 
  std::vector<string> ti=this->getOutputAlpha();
  tmp1->setInputAlpha(ti);
  tmp1->setOutputAlpha(t);
   
  string input;
  TreeLabel output(this->getInputAlpha()[0][0]);
  std::vector<string> exp;
  std::list<std::list<Transition<T> >::iterator> lT1;
  std::list<std::list<Transition<T> >::iterator>::iterator ilT1;
  lT1 = tmp1->getIterListOfTrans();

  //cout<<"OK 13"<<endl<<flush;

  //cout<<"before reverse: "<<endl<<flush;
  //tmp1->display(0);

  for(ilT1=lT1.begin();ilT1!=lT1.end();++ilT1) 
    {
       if((*ilT1)->getLabel().getInput().empty())
           output = TreeLabel(this->getInputAlpha()[0][0]);
       else
	 {
	   Interface<TreeLabel>::strToExp((*ilT1)->getLabel().getInput(),exp);
           output  = TreeLabel(exp,this->getInputAlpha()[0][0]);
	 }
       if((*ilT1)->getLabel().getOutput().empty())
         input = "";
       else
         input  = (*ilT1)->getLabel().getOutput().getInfixString();
        
       T lab = T(input,output);
       (*ilT1)->setLabel(lab);
    }

  //cout<<"reverse: "<<endl<<flush;
  //tmp1->display(0);

  LTOEXPTransducer<T>* tmp = tmp1->toLTOEXP(name);
  delete(tmp1);
 
  //cout<<"resul: "<<endl<<flush;
  //tmp->display(0);

  ListOfSymbols hist;
  hist.insert(hist.end(), this->getName());
  tmp->setHistOfITGraph(hist);
  tmp->setHistType(REVERSE_INOUT_WTOEXP_HISTORY);
  return tmp;  
}






